package com.ejie.m62fb.dao;

import com.ejie.x38.dto.Pagination;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.model.M62fbCalles;

/**
 * M62fbCallesDaoImpl generated by UDA, 22-oct-2012 12:14:19.
 * 
XXXX
 */

@Repository
@Transactional
public class M62fbCallesDaoImpl implements M62fbCallesDao {
	private JdbcTemplate jdbcTemplate;
	private RowMapper<M62fbCalles> rwMap = new RowMapper<M62fbCalles>() {
		public M62fbCalles mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new M62fbCalles(resultSet.getString("CODPROVINCIA"),
					resultSet.getString("CODMUNICIPIO"),
					resultSet.getString("CODCALLE"),
					resultSet.getString("DESCALLECAS"));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Finds a single row in the M62fbCalles table.
	 * 
	 * @param m62fbcalles
	 *            Pagination
	 * @return M62fbCalles
	 */
	@Transactional(readOnly = true, timeout = 500)
	public M62fbCalles find(M62fbCalles m62fbcalles) {
		String query = "SELECT t1.COD_PROVINCIA CODPROVINCIA, t1.COD_MUNICIPIO CODMUNICIPIO, t1.COD_CALLE CODCALLE, t1.DES_CALLE_CAS DESCALLECAS FROM M62A11V0S00_CALLES t1  WHERE t1.COD_PROVINCIA = ?   AND t1.COD_MUNICIPIO = ?   AND t1.COD_CALLE = ?   AND t1.DES_CALLE_CAS = ?   ";

		List<M62fbCalles> m62fbcallesList = this.jdbcTemplate.query(query,
				this.rwMap, m62fbcalles.getCodProvincia(),
				m62fbcalles.getCodMunicipio(), m62fbcalles.getCodCalle(),
				m62fbcalles.getDesCalleCas());
		return (M62fbCalles) DataAccessUtils.uniqueResult(m62fbcallesList);
	}

	/**
	 * Finds a List of rows in the M62fbCalles table.
	 * 
	 * @param m62fbcalles
	 *            M62fbCalles
	 * @param pagination
	 *            Pagination
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbCalles> findAll(M62fbCalles m62fbcalles,
			Pagination pagination) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_PROVINCIA CODPROVINCIA,t1.COD_MUNICIPIO CODMUNICIPIO,t1.COD_CALLE CODCALLE,t1.DES_CALLE_CAS DESCALLECAS ");
		query.append("FROM M62A11V0S00_CALLES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbcalles);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbCalles>) this.jdbcTemplate.query(query.toString(),
				this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbCalles table.
	 * 
	 * @param m62fbcalles
	 *            M62fbCalles
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllCount(M62fbCalles m62fbcalles) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62A11V0S00_CALLES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbcalles);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Finds rows in the M62fbCalles table using like.
	 * 
	 * @param m62fbcalles
	 *            M62fbCalles
	 * @param pagination
	 *            Pagination
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbCalles> findAllLike(M62fbCalles m62fbcalles,
			Pagination pagination, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_PROVINCIA CODPROVINCIA,t1.COD_MUNICIPIO CODMUNICIPIO,t1.COD_CALLE CODCALLE,t1.DES_CALLE_CAS DESCALLECAS ");
		query.append("FROM M62A11V0S00_CALLES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this
				.getWhereLikeMap(m62fbcalles, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbCalles>) this.jdbcTemplate.query(query.toString(),
				this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbCalles table using like.
	 * 
	 * @param m62fbcalles
	 *            M62fbCalles
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllLikeCount(M62fbCalles m62fbcalles, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62A11V0S00_CALLES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this
				.getWhereLikeMap(m62fbcalles, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbCalles entity
	 * 
	 * @param m62fbcalles
	 *            M62fbCalles Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(M62fbCalles m62fbcalles) {

		StringBuffer where = new StringBuffer(
				M62fbCallesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbcalles != null && m62fbcalles.getCodProvincia() != null) {
			where.append(" AND t1.COD_PROVINCIA = ?");
			params.add(m62fbcalles.getCodProvincia());
		}
		if (m62fbcalles != null && m62fbcalles.getCodMunicipio() != null) {
			where.append(" AND t1.COD_MUNICIPIO = ?");
			params.add(m62fbcalles.getCodMunicipio());
		}
		if (m62fbcalles != null && m62fbcalles.getCodCalle() != null) {
			where.append(" AND t1.COD_CALLE = ?");
			params.add(m62fbcalles.getCodCalle());
		}
		if (m62fbcalles != null && m62fbcalles.getDesCalleCas() != null) {
			where.append(" AND t1.DES_CALLE_CAS = ?");
			params.add(m62fbcalles.getDesCalleCas());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbCalles entity
	 * 
	 * @param m62fbcalles
	 *            M62fbCalles Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereLikeMap(M62fbCalles m62fbcalles,
			Boolean startsWith) {

		StringBuffer where = new StringBuffer(
				M62fbCallesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbcalles != null && m62fbcalles.getCodProvincia() != null) {
			where.append(" AND UPPER(t1.COD_PROVINCIA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbcalles.getCodProvincia().toUpperCase() + "%");
			} else {
				params.add("%" + m62fbcalles.getCodProvincia().toUpperCase()
						+ "%");
			}
			where.append(" AND t1.COD_PROVINCIA IS NOT NULL");
		}
		if (m62fbcalles != null && m62fbcalles.getCodMunicipio() != null) {
			where.append(" AND UPPER(t1.COD_MUNICIPIO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbcalles.getCodMunicipio().toUpperCase() + "%");
			} else {
				params.add("%" + m62fbcalles.getCodMunicipio().toUpperCase()
						+ "%");
			}
			where.append(" AND t1.COD_MUNICIPIO IS NOT NULL");
		}
		if (m62fbcalles != null && m62fbcalles.getCodCalle() != null) {
			where.append(" AND UPPER(t1.COD_CALLE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbcalles.getCodCalle().toUpperCase() + "%");
			} else {
				params.add("%" + m62fbcalles.getCodCalle().toUpperCase() + "%");
			}
			where.append(" AND t1.COD_CALLE IS NOT NULL");
		}
		if (m62fbcalles != null && m62fbcalles.getDesCalleCas() != null) {
			where.append(" AND UPPER(t1.DES_CALLE_CAS) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbcalles.getDesCalleCas().toUpperCase() + "%");
			} else {
				params.add("%" + m62fbcalles.getDesCalleCas().toUpperCase()
						+ "%");
			}
			where.append(" AND t1.DES_CALLE_CAS IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
