package com.ejie.m62fb.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.i18n.I18nString;
import com.ejie.m62fb.model.M62fbProvincias;
import com.ejie.x38.dto.Pagination;

/**
 * M62fbProvinciasDaoImpl generated by UDA, 22-oct-2012 12:14:19.
 * 
XXXX
 */

@Repository
@Transactional
public class M62fbProvinciasDaoImpl implements M62fbProvinciasDao {
	private JdbcTemplate jdbcTemplate;
	private RowMapper<M62fbProvincias> rwMap = new RowMapper<M62fbProvincias>() {
		public M62fbProvincias mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			// Valor por defecto???
			HashMap<String, String> mp = new HashMap<String, String>();
			mp.put("es", resultSet.getString("DESPROVICAS"));
			mp.put("eu", resultSet.getString("DESPROVIEUS"));

			return new M62fbProvincias(resultSet.getString("CODPROVINCIA"),
					new I18nString(resultSet.getString("DESPROVICAS"), mp));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Finds a single row in the M62fbProvincias table.
	 * 
	 * @param m62fbprovincias
	 *            Pagination
	 * @return M62fbProvincias
	 */
	@Transactional(readOnly = true, timeout = 500)
	public M62fbProvincias find(M62fbProvincias m62fbprovincias) {
		String query = "SELECT t1.COD_PROVINCIA CODPROVINCIA, t1.DES_PROVI_CAS DESPROVICAS, t1.DES_PROVI_EUS DESPROVIEUS FROM M62A09V0S00_PROVINCIAS t1  WHERE t1.COD_PROVINCIA = ?   AND t1.DES_PROVI_CAS = ?  ";

		List<M62fbProvincias> m62fbprovinciasList = this.jdbcTemplate.query(
				query, this.rwMap, m62fbprovincias.getCodProvincia(),
				m62fbprovincias.getDesProviCas());
		return (M62fbProvincias) DataAccessUtils
				.uniqueResult(m62fbprovinciasList);
	}

	/**
	 * Finds a List of rows in the M62fbProvincias table.
	 * 
	 * @param m62fbprovincias
	 *            M62fbProvincias
	 * @param pagination
	 *            Pagination
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbProvincias> findAll(M62fbProvincias m62fbprovincias,
			Pagination pagination) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_PROVINCIA CODPROVINCIA,t1.DES_PROVI_CAS DESPROVICAS, t1.DES_PROVI_EUS DESPROVIEUS ");
		query.append("FROM M62A09V0S00_PROVINCIAS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbprovincias);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbProvincias>) this.jdbcTemplate.query(
				query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbProvincias table.
	 * 
	 * @param m62fbprovincias
	 *            M62fbProvincias
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllCount(M62fbProvincias m62fbprovincias) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62A09V0S00_PROVINCIAS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbprovincias);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Finds rows in the M62fbProvincias table using like.
	 * 
	 * @param m62fbprovincias
	 *            M62fbProvincias
	 * @param pagination
	 *            Pagination
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbProvincias> findAllLike(M62fbProvincias m62fbprovincias,
			Pagination pagination, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_PROVINCIA CODPROVINCIA,t1.DES_PROVI_CAS DESPROVICAS,t1.DES_PROVI_EUS DESPROVIEUS,t1.DEC_PROVI_CAS DECPROVICAS,t1.DEC_PROVI_EUS DECPROVIEUS,t1.FEC_ALTA FECALTA,t1.DIR_IP_USUARIO DIRIPUSUARIO,t1.FEH_ULT_MODIF FEHULTMODIF,t1.COD_USU_ULT_MODIF CODUSUULTMODIF,t1.IDE_REGISTRO IDEREGISTRO ");
		query.append("FROM M62A09V0S00_PROVINCIAS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(m62fbprovincias,
				startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbProvincias>) this.jdbcTemplate.query(
				query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbProvincias table using like.
	 * 
	 * @param m62fbprovincias
	 *            M62fbProvincias
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllLikeCount(M62fbProvincias m62fbprovincias,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62A09V0S00_PROVINCIAS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(m62fbprovincias,
				startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbProvincias entity
	 * 
	 * @param m62fbprovincias
	 *            M62fbProvincias Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(M62fbProvincias m62fbprovincias) {

		StringBuffer where = new StringBuffer(
				M62fbProvinciasDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbprovincias != null
				&& m62fbprovincias.getCodProvincia() != null) {
			where.append(" AND t1.COD_PROVINCIA = ?");
			params.add(m62fbprovincias.getCodProvincia());
		}
		/*
		 * if (m62fbprovincias != null && m62fbprovincias.getDesProviCas() !=
		 * null) { where.append(" AND t1.DES_PROVI_CAS = ?");
		 * params.add(m62fbprovincias.getDesProviCas()); }
		 */

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbProvincias entity
	 * 
	 * @param m62fbprovincias
	 *            M62fbProvincias Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereLikeMap(M62fbProvincias m62fbprovincias,
			Boolean startsWith) {

		StringBuffer where = new StringBuffer(
				M62fbProvinciasDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbprovincias != null
				&& m62fbprovincias.getCodProvincia() != null) {
			where.append(" AND UPPER(t1.COD_PROVINCIA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbprovincias.getCodProvincia().toUpperCase()
						+ "%");
			} else {
				params.add("%"
						+ m62fbprovincias.getCodProvincia().toUpperCase() + "%");
			}
			where.append(" AND t1.COD_PROVINCIA IS NOT NULL");
		}
		/*
		 * if (m62fbprovincias != null && m62fbprovincias.getDesProviCas() !=
		 * null) {
		 * where.append(" AND UPPER(t1.DES_PROVI_CAS) like ? ESCAPE  '\\'"); if
		 * (startsWith) {
		 * params.add(m62fbprovincias.getDesProviCas().toUpperCase() + "%"); }
		 * else { params.add("%" +
		 * m62fbprovincias.getDesProviCas().toUpperCase() + "%"); }
		 * where.append(" AND t1.DES_PROVI_CAS IS NOT NULL"); }
		 */

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
