package com.ejie.m62fb.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.i18n.I18nString;
import com.ejie.m62fb.model.M62fbRegAcceso;
import com.ejie.x38.dto.Pagination;

/**
 * M62fbRegAccesoDaoImpl generated by UDA, 24-oct-2012 16:48:53.
 * 
XXXX
 */

@Repository
@Transactional
public class M62fbRegAccesoDaoImpl implements M62fbRegAccesoDao {
	private JdbcTemplate jdbcTemplate;
	private RowMapper<M62fbRegAcceso> rwMap = new RowMapper<M62fbRegAcceso>() {
		public M62fbRegAcceso mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			// Valor por defecto???
			HashMap<String, String> mp = new HashMap<String, String>();
			mp.put("es", resultSet.getString("DESTABLACAS"));
			mp.put("eu", resultSet.getString("DESTABLAEUS"));

			return new M62fbRegAcceso(resultSet.getString("CODTABLA"),
					new I18nString(resultSet.getString("DESTABLACAS"), mp));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Inserts a single row in the M62fbRegAcceso table.
	 * 
	 * @param m62fbregacceso
	 *            Pagination
	 * @return M62fbRegAcceso
	 */
	public M62fbRegAcceso add(M62fbRegAcceso m62fbregacceso) {
		String query = "INSERT INTO M62P28V0S00_REG_ACC_USO (COD_TABLA, DES_TABLA_CAS) VALUES (?,?)";
		this.jdbcTemplate.update(query, m62fbregacceso.getCodTabla(),
				m62fbregacceso.getDesTablaCas());
		return m62fbregacceso;
	}

	/**
	 * Updates a single row in the M62fbRegAcceso table.
	 * 
	 * @param m62fbregacceso
	 *            Pagination
	 * @return M62fbRegAcceso
	 */
	public M62fbRegAcceso update(M62fbRegAcceso m62fbregacceso) {
		String query = "UPDATE M62P28V0S00_REG_ACC_USO SET  WHERE COD_TABLA=? AND DES_TABLA_CAS=?";
		this.jdbcTemplate.update(query, m62fbregacceso.getCodTabla(),
				m62fbregacceso.getDesTablaCas());
		return m62fbregacceso;
	}

	/**
	 * Finds a single row in the M62fbRegAcceso table.
	 * 
	 * @param m62fbregacceso
	 *            Pagination
	 * @return M62fbRegAcceso
	 */
	@Transactional(readOnly = true, timeout = 500)
	public M62fbRegAcceso find(M62fbRegAcceso m62fbregacceso) {
		String query = "SELECT t1.COD_TABLA CODTABLA, t1.DES_TABLA_CAS DESTABLACAS, t1.DES_TABLA_EUS DESTABLAEUS FROM M62P28V0S00_REG_ACC_USO t1  WHERE t1.COD_TABLA = ?   AND t1.DES_TABLA_CAS = ?  ";

		List<M62fbRegAcceso> m62fbregaccesoList = this.jdbcTemplate.query(
				query, this.rwMap, m62fbregacceso.getCodTabla(),
				m62fbregacceso.getDesTablaCas());
		return (M62fbRegAcceso) DataAccessUtils
				.uniqueResult(m62fbregaccesoList);
	}

	/**
	 * Removes a single row in the M62fbRegAcceso table.
	 * 
	 * @param m62fbregacceso
	 *            Pagination
	 * @return
	 */
	public void remove(M62fbRegAcceso m62fbregacceso) {
		String query = "DELETE FROM M62P28V0S00_REG_ACC_USO WHERE COD_TABLA=? AND DES_TABLA_CAS=?";
		this.jdbcTemplate.update(query, m62fbregacceso.getCodTabla(),
				m62fbregacceso.getDesTablaCas());
	}

	/**
	 * Finds a List of rows in the M62fbRegAcceso table.
	 * 
	 * @param m62fbregacceso
	 *            M62fbRegAcceso
	 * @param pagination
	 *            Pagination
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbRegAcceso> findAll(M62fbRegAcceso m62fbregacceso,
			Pagination pagination) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_TABLA CODTABLA,t1.DES_TABLA_CAS DESTABLACAS, t1.DES_TABLA_EUS DESTABLAEUS ");
		query.append("FROM M62P28V0S00_REG_ACC_USO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbregacceso);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);
		query.append(" ORDER BY t1.DES_TABLA_CAS ASC");

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbRegAcceso>) this.jdbcTemplate.query(query.toString(),
				this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbRegAcceso table.
	 * 
	 * @param m62fbregacceso
	 *            M62fbRegAcceso
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllCount(M62fbRegAcceso m62fbregacceso) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62P28V0S00_REG_ACC_USO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbregacceso);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Finds rows in the M62fbRegAcceso table using like.
	 * 
	 * @param m62fbregacceso
	 *            M62fbRegAcceso
	 * @param pagination
	 *            Pagination
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbRegAcceso> findAllLike(M62fbRegAcceso m62fbregacceso,
			Pagination pagination, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_TABLA CODTABLA,t1.DES_TABLA_CAS DESTABLACAS, t1.DES_TABLA_EUS DESTABLAEUS ");
		query.append("FROM M62P28V0S00_REG_ACC_USO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(m62fbregacceso,
				startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbRegAcceso>) this.jdbcTemplate.query(query.toString(),
				this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbRegAcceso table using like.
	 * 
	 * @param m62fbregacceso
	 *            M62fbRegAcceso
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllLikeCount(M62fbRegAcceso m62fbregacceso,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62P28V0S00_REG_ACC_USO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(m62fbregacceso,
				startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbRegAcceso entity
	 * 
	 * @param m62fbregacceso
	 *            M62fbRegAcceso Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(M62fbRegAcceso m62fbregacceso) {

		StringBuffer where = new StringBuffer(
				M62fbRegAccesoDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbregacceso != null && m62fbregacceso.getCodTabla() != null) {
			where.append(" AND t1.COD_TABLA = ?");
			params.add(m62fbregacceso.getCodTabla());
		}
		if (m62fbregacceso != null && m62fbregacceso.getDesTablaCas() != null) {
			where.append(" AND t1.DES_TABLA_CAS = ?");
			params.add(m62fbregacceso.getDesTablaCas());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbRegAcceso entity
	 * 
	 * @param m62fbregacceso
	 *            M62fbRegAcceso Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereLikeMap(M62fbRegAcceso m62fbregacceso,
			Boolean startsWith) {

		StringBuffer where = new StringBuffer(
				M62fbRegAccesoDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbregacceso != null && m62fbregacceso.getCodTabla() != null) {
			where.append(" AND UPPER(t1.COD_TABLA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbregacceso.getCodTabla().toUpperCase() + "%");
			} else {
				params.add("%" + m62fbregacceso.getCodTabla().toUpperCase()
						+ "%");
			}
			where.append(" AND t1.COD_TABLA IS NOT NULL");
		}
		/*
		 * if (m62fbregacceso != null && m62fbregacceso.getDesTablaCas() !=
		 * null) {
		 * where.append(" AND UPPER(t1.DES_TABLA_CAS) like ? ESCAPE  '\\'"); if
		 * (startsWith) {
		 * params.add(m62fbregacceso.getDesTablaCas().toUpperCase() + "%"); }
		 * else { params.add("%" + m62fbregacceso.getDesTablaCas().toUpperCase()
		 * + "%"); } where.append(" AND t1.DES_TABLA_CAS IS NOT NULL"); }
		 */

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
