package com.ejie.m62fb.dao;

import com.ejie.x38.dto.Pagination;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.i18n.I18nString;
import com.ejie.m62fb.model.M62fbRegTenencia;

/**
 * M62fbRegTenenciaDaoImpl generated by UDA, 19-nov-2012 13:36:51.
 * 
XXXX
 */

@Repository
@Transactional
public class M62fbRegTenenciaDaoImpl implements M62fbRegTenenciaDao {
	private JdbcTemplate jdbcTemplate;
	private RowMapper<M62fbRegTenencia> rwMap = new RowMapper<M62fbRegTenencia>() {
		public M62fbRegTenencia mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			// Valor por defecto???
			HashMap<String, String> mp = new HashMap<String, String>();
			mp.put("es", resultSet.getString("DESTABLACAS"));
			mp.put("eu", resultSet.getString("DESTABLAEUS"));

			return new M62fbRegTenencia(resultSet.getString("CODTABLA"),
					new I18nString(resultSet.getString("DESTABLACAS"), mp));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Updates a single row in the M62fbRegTenencia table.
	 * 
	 * @param m62fbregtenencia
	 *            Pagination
	 * @return M62fbRegTenencia
	 */
	public M62fbRegTenencia update(M62fbRegTenencia m62fbregtenencia) {
		String query = "UPDATE M62P28V0S00_REG_TENENCIA SET  WHERE COD_TABLA=? ";
		this.jdbcTemplate.update(query, m62fbregtenencia.getCodTabla());
		return m62fbregtenencia;
	}

	/**
	 * Finds a single row in the M62fbRegTenencia table.
	 * 
	 * @param m62fbregtenencia
	 *            Pagination
	 * @return M62fbRegTenencia
	 */
	@Transactional(readOnly = true, timeout = 500)
	public M62fbRegTenencia find(M62fbRegTenencia m62fbregtenencia) {
		String query = "SELECT t1.COD_TABLA CODTABLA, t1.DES_TABLA_CAS DESTABLACAS, t1.DES_TABLA_EUS DESTABLAEUS FROM M62P28V0S00_REG_TENENCIA t1  WHERE t1.COD_TABLA = ?   ";

		List<M62fbRegTenencia> m62fbregtenenciaList = this.jdbcTemplate.query(
				query, this.rwMap, m62fbregtenencia.getCodTabla());
		return (M62fbRegTenencia) DataAccessUtils
				.uniqueResult(m62fbregtenenciaList);
	}

	/**
	 * Removes a single row in the M62fbRegTenencia table.
	 * 
	 * @param m62fbregtenencia
	 *            Pagination
	 * @return
	 */
	public void remove(M62fbRegTenencia m62fbregtenencia) {
		String query = "DELETE FROM M62P28V0S00_REG_TENENCIA WHERE COD_TABLA=?";
		this.jdbcTemplate.update(query, m62fbregtenencia.getCodTabla());
	}

	/**
	 * Finds a List of rows in the M62fbRegTenencia table.
	 * 
	 * @param m62fbregtenencia
	 *            M62fbRegTenencia
	 * @param pagination
	 *            Pagination
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbRegTenencia> findAll(M62fbRegTenencia m62fbregtenencia,
			Pagination pagination) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_TABLA CODTABLA,t1.DES_TABLA_CAS DESTABLACAS,t1.DES_TABLA_EUS DESTABLAEUS ");
		query.append("FROM M62P28V0S00_REG_TENENCIA t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbregtenencia);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);
		query.append(" ORDER BY t1.DES_TABLA_CAS ASC");

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbRegTenencia>) this.jdbcTemplate.query(
				query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbRegTenencia table.
	 * 
	 * @param m62fbregtenencia
	 *            M62fbRegTenencia
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllCount(M62fbRegTenencia m62fbregtenencia) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62P28V0S00_REG_TENENCIA t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(m62fbregtenencia);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Finds rows in the M62fbRegTenencia table using like.
	 * 
	 * @param m62fbregtenencia
	 *            M62fbRegTenencia
	 * @param pagination
	 *            Pagination
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	@Transactional(readOnly = true, timeout = 500)
	public List<M62fbRegTenencia> findAllLike(
			M62fbRegTenencia m62fbregtenencia, Pagination pagination,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.COD_TABLA CODTABLA,t1.DES_TABLA_CAS DESTABLACAS,t1.DES_TABLA_EUS DESTABLAEUS ");
		query.append("FROM M62P28V0S00_REG_TENENCIA t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(m62fbregtenencia,
				startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (pagination != null) {
			query = pagination.getPaginationQuery(query);
		}

		return (List<M62fbRegTenencia>) this.jdbcTemplate.query(
				query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Counts rows in the M62fbRegTenencia table using like.
	 * 
	 * @param m62fbregtenencia
	 *            M62fbRegTenencia
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllLikeCount(M62fbRegTenencia m62fbregtenencia,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT COUNT(1) FROM M62P28V0S00_REG_TENENCIA t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(m62fbregtenencia,
				startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbRegTenencia entity
	 * 
	 * @param m62fbregtenencia
	 *            M62fbRegTenencia Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(M62fbRegTenencia m62fbregtenencia) {

		StringBuffer where = new StringBuffer(
				M62fbRegTenenciaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbregtenencia != null && m62fbregtenencia.getCodTabla() != null) {
			where.append(" AND t1.COD_TABLA = ?");
			params.add(m62fbregtenencia.getCodTabla());
		}
		/*
		 * if (m62fbregtenencia != null && m62fbregtenencia.getDesTablaCas() !=
		 * null) { where.append(" AND t1.DES_TABLA_CAS = ?");
		 * params.add(m62fbregtenencia.getDesTablaCas()); }
		 */

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the M62fbRegTenencia entity
	 * 
	 * @param m62fbregtenencia
	 *            M62fbRegTenencia Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereLikeMap(M62fbRegTenencia m62fbregtenencia,
			Boolean startsWith) {

		StringBuffer where = new StringBuffer(
				M62fbRegTenenciaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (m62fbregtenencia != null && m62fbregtenencia.getCodTabla() != null) {
			where.append(" AND UPPER(t1.COD_TABLA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(m62fbregtenencia.getCodTabla().toUpperCase() + "%");
			} else {
				params.add("%" + m62fbregtenencia.getCodTabla().toUpperCase()
						+ "%");
			}
			where.append(" AND t1.COD_TABLA IS NOT NULL");
		}
		/*
		 * if (m62fbregtenencia != null && m62fbregtenencia.getDesTablaCas() !=
		 * null) {
		 * where.append(" AND UPPER(t1.DES_TABLA_CAS) like ? ESCAPE  '\\'"); if
		 * (startsWith) {
		 * params.add(m62fbregtenencia.getDesTablaCas().toUpperCase() + "%"); }
		 * else { params.add("%" +
		 * m62fbregtenencia.getDesTablaCas().toUpperCase() + "%"); }
		 * where.append(" AND t1.DES_TABLA_CAS IS NOT NULL"); }
		 */

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;

	@Override
	public M62fbRegTenencia add(M62fbRegTenencia m62fbregtenencia) {
		// TODO Auto-generated method stub
		return null;
	}
}
