package com.ejie.m62fb.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.utils.constants.IberErrorsMsgConstants;
import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.dao.registroVPP.DireccionesDao;
import com.ejie.m62fb.dao.registroVPP.ElementosDaoSql;
import com.ejie.m62fb.dao.registroVPP.plsql.M62FuncionesPLSQL;
import com.ejie.m62fb.model.Elemento;
import com.ejie.m62fb.model.persona.Persona;
import com.ejie.m62fb.model.plsql.PlSqlOutputError;
import com.ejie.m62fb.model.tipos.TipoClave;
import com.ejie.m62fb.model.tipos.TipoEstadoCivil;
import com.ejie.m62fb.model.tipos.TipoSexo;
import com.ejie.m62fb.utils.sql.BDUtils;
import com.ejie.m62fb.utils.sql.UtilsResulSet;
import com.ejie.t17i.dao.PaisDao;
import com.ejie.t17i.model.Pais;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * PersonaDaoImpl generated by UDA, 24-oct-2012 16:55:00.
 * 
XXXX
 */

@Repository
@Transactional
public class PersonaDaoImpl implements PersonaDao {
	private JdbcTemplate jdbcTemplate;
	@Autowired
	private ComboP28Dao comboP28Dao;
	@Autowired
	private DireccionesDao direccionesDao;
	@Autowired
	private PaisDao paisDao;
	private final RowMapper<Persona> rwMap = new RowMapper<Persona>() {
		@Override
		public Persona mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			Persona persona = new Persona(resultSet.getBigDecimal(PersonaDaoSql.PK));
			//COD_TIPO_CLAVE
			TipoClave tipoClave= new TipoClave(	resultSet.getString("COD_TIPO_CLAVE"));
			persona.setTipoCodigoId((TipoClave) comboP28Dao.find(tipoClave));
			persona.setCodigoId(resultSet.getString("NIF_TITULAR"));
			//NUM_CORREL_NIF
			persona.setNumCorrelNif(resultSet.getInt("NUM_CORREL_NIF"));
			//NOM_NOMBRE
			persona.setNombre(resultSet.getString("NOM_NOMBRE"));
			//NOM_APELLIDO_1
			persona.setApellido1(resultSet.getString("NOM_APELLIDO_1"));
			//NOM_APELLIDO_2
			persona.setApellido2(resultSet.getString("NOM_APELLIDO_2"));
			// Direccion
			persona.setDireccion(UtilsResulSet.getDireccionCompleta(resultSet,
					null, direccionesDao));
			persona.setTelefno1(resultSet.getString("NTELEFNO"));
			//NUM_TLFNO_2
			persona.setTelefno2(resultSet.getString("NUM_TLFNO_2"));
			//EMAIL
			persona.setEmail(resultSet.getString("EMAIL"));
			//FEC_NACIMIENTO
			persona.setFecNacimiento(resultSet.getDate("FEC_NACIMIENTO"));
			//IND_SEXO
			TipoSexo sexo= new TipoSexo(resultSet.getString("IND_SEXO"));
			persona.setSexo(sexo.getCodTabla()!=null?(TipoSexo) comboP28Dao.find(sexo):null);
			//IND_ESTADO_CIVIL
			TipoEstadoCivil estadoCivil= new TipoEstadoCivil(resultSet.getString("IND_ESTADO_CIVIL"));
			persona.setEstadoCivil(estadoCivil.getCodTabla()!=null?(TipoEstadoCivil) comboP28Dao.find(estadoCivil):null);
			//COD_NACIONALIDAD
			String nacionalidad=resultSet.getString("COD_NACIONALIDAD");
			if (nacionalidad != null && !nacionalidad.isEmpty()){
				persona.setNacionalidad(paisDao.find(new Pais(nacionalidad)));
			}
			UtilsResulSet.getAbstractClass(resultSet, persona, null);
			return persona;
		}
	};
	private final RowMapper<Persona> rwMapPK = new RowMapper<Persona>() {
		@Override
		public Persona mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new Persona(resultSet.getBigDecimal(PersonaDaoSql.PK));
		}
	};
//	private final RowMapper<Persona> rwMapSVDI = new RowMapper<Persona>() {
//		@Override
//		public Persona mapRow(ResultSet resultSet, int rowNum)
//				throws SQLException {
//			Persona persona = new Persona();
//			
//			//NOM_NOMBRE
//			persona.setNombre(resultSet.getString("NOM_TITULAR"));
//			//NOM_APELLIDO_1
//			persona.setApellido1(resultSet.getString("APE1_TITULAR"));
//			//NOM_APELLIDO_2
//			persona.setApellido2(resultSet.getString("APE2_TITULAR"));
//			//FEC_NACIMIENTO
//			persona.setFecNacimiento(resultSet.getDate("FEC_NAC_TITULAR"));
//			//IND_SEXO
//			TipoSexo sexo= new TipoSexo(resultSet.getString("SEXO_TITULAR").equalsIgnoreCase("")?TipoSexo.VARON:TipoSexo.MUJER);
//			persona.setSexo(sexo.getCodTabla()!=null?(TipoSexo) comboP28Dao.find(sexo):null);
//			//COD_NACIONALIDAD
//			String nacionalidad=resultSet.getString("NAC_TITULAR");
//			if (nacionalidad != null && !nacionalidad.equalsIgnoreCase(AppConstantes.PAIS_ESPANYA_SVDI)){
//				persona.setNacionalidad(new Pais(AppConstantes.PAIS_ESPANYA,AppConstantes.PAIS_ESPANYA_TXT));
//			}
//			return persona;
//		}
//	};
	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Inserts a single row in the Persona table.
	 * 
	 * @param persona
	 *            Pagination
	 * @return Persona
	 */
	@Override
	public Persona add(Persona persona) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		m62FuncionesPLSQL.addTitular(persona);
		return persona;
	
	}

	/**
	 * Updates a single row in the Persona table.
	 * 
	 * @param persona
	 *            Pagination
	 * @return Persona
	 */
	@Override
	public Persona update(Persona persona) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		m62FuncionesPLSQL.editTitular(persona);
		return persona;
	}
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Persona findForUpdate(Persona persona) {
		Persona personaBD;
		try {
			personaBD = this.jdbcTemplate.queryForObject(
					PersonaDaoSql.SQL_FIND_FOR_UPDATE, this.rwMap,
					persona.getIdPersona(), persona.getFecHorUltimaModif());
		} catch (EmptyResultDataAccessException e) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		if (personaBD == null) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}

		return personaBD;
	}
	/**
	 * Finds a single row in the Persona table.
	 * 
	 * @param persona
	 *            Pagination
	 * @return Persona
	 */
	@Override
	@Transactional(readOnly = true, timeout = 500)
	public Persona find(Persona persona) {
		String query = PersonaDaoSql.SQL_FIND;
		List<Persona> personaList = this.jdbcTemplate.query(query, this.rwMap,
				persona.getIdPersona());
		return DataAccessUtils.uniqueResult(personaList);
	}

	/**
	 * Removes a single row in the Persona table.
	 * 
	 * @param persona
	 *            Pagination
	 * @return
	 */
	@Override
	public void remove(Persona persona) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		PlSqlOutputError output=m62FuncionesPLSQL.borrarPersona(persona.getIdPersona());
		if (output.getCodError()!=null && output.getCodError().longValue()>0)
		{
			throw new UFException(output.getTextError());
		}
	}

//	/**
//	 * Finds a List of rows in the Persona table.
//	 * 
//	 * @param persona
//	 *            Persona
//	 * @param pagination
//	 *            Pagination
//	 * @return List
//	 */
//	@Override
//	@Transactional(readOnly = true, timeout = 500)
//	public List<Persona> findAll(Persona persona, Pagination pagination) {
//		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_FIND_ALL);
//
//		// Where clause & Params
//		Map<String, ?> mapaWhere = this.getWhereMap(persona);
//		StringBuilder where = new StringBuilder(PersonaDaoSql.SQL_WHERE_ALL);
//		where.append(mapaWhere.get("query"));
//		query.append(where);
//
//		List<?> params = (List<?>) mapaWhere.get("params");
//
//		if (pagination != null) {
//			query = pagination.getPaginationQuery(query);
//		}
//
//		return this.jdbcTemplate.query(query.toString(), this.rwMap,
//				params.toArray());
//	}

//	/**
//	 * Counts rows in the Persona table.
//	 * 
//	 * @param persona
//	 *            Persona
//	 * @return Long
//	 */
//	@Override
//	@Transactional(readOnly = true, timeout = 500)
//	public Long findAllCount(Persona persona) {
//		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_FIND_ALL_COUNT);
//
//		// Where clause & Params
//		Map<String, ?> mapaWhere = this.getWhereMap(persona);
//		StringBuilder where = new StringBuilder(PersonaDaoSql.SQL_WHERE_ALL);
//		where.append(mapaWhere.get("query"));
//		query.append(where);
//
//		List<?> params = (List<?>) mapaWhere.get("params");
//
//		return this.jdbcTemplate.queryForLong(query.toString(),
//				params.toArray());
//	}
	/**
	 * Finds a List of rows in the Persona table.
	 * 
	 * @param persona
	 *            Persona
	 * @param pagination
	 *            Pagination
	 * @return List
	 */
	@Override
	@Transactional(readOnly = true, timeout = 500)
	public List<Persona> findAll(Persona persona, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(persona);
		StringBuilder where = new StringBuilder(PersonaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}else {
			query.append(" AND ROWNUM < 20");
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/**
	 * Finds rows in the M62r12t0 table using like.
	 * 
	 * @param elemento
	 *            Elemento
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Elemento>
	 */
	@Override
	@Transactional(readOnly = true, timeout = 500)
	public List<Persona> findAllLike(Persona persona,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_FIND_ALL);
		
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(persona, startsWith);
		StringBuilder where = new StringBuilder(PersonaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}else {
			query.append(" AND ROWNUM < 20");
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}
	/**
	 * Reorder the data list of M62r12t0 selected for rup_table
	 * 
	 * @param elemento
	 *            Elemento
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Elementos>>
	 */
	@Override
	public List<TableRowDto<Persona>> reorderSelection(Persona persona,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_FIND_ALL);

		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(persona, startsWith);
		StringBuilder where = new StringBuilder(PersonaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(
				query, jqGridRequestDto, Elemento.class, filterParamList,
				ElementosDaoSql.PK);

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Persona>(this.rwMapPK,
						jqGridRequestDto), filterParamList.toArray());
			}
	
	/**
	 * Counts rows in the Persona table using like.
	 * 
	 * @param elemento
	 *            Elemento
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true, timeout = 500)
	public Long findAllLikeCount(Persona persona, Boolean startsWith) {
		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_FIND_ALL_COUNT);
		
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(persona, startsWith);
		StringBuilder where = new StringBuilder(PersonaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);
		
		List<?> params = (List<?>) mapaWhere.get("params");
		
		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}
			
	/**
	 * Comprueba si existe un NIF en BD.
	 * 
	 * @param nifTitular
	 *            String
	 * @return boolean
	 */
	@Override
	@Transactional(readOnly = true, timeout = 500)
	public Persona findByNIF(String nifTitular) {
		StringBuilder query = new StringBuilder(PersonaDaoSql.SQL_EXISTE_NIF);
		
		List<Persona> personaList = this.jdbcTemplate.query(query.toString(), this.rwMapPK,
				nifTitular);
		return DataAccessUtils.uniqueResult(personaList);
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Persona entity
	 * 
	 * @param Persona
	 *            Persona Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Persona persona) {

		StringBuilder where = new StringBuilder(
				PersonaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (persona != null && persona.getIdPersona() != null) {
			where.append(" AND ID_PERSONA = ?");
			params.add(persona.getIdPersona());
		}
		if (persona != null && persona.getTipoCodigoId() != null && persona.getTipoCodigoId().getCodTabla() != null) {
			where.append(" AND COD_TIPO_CLAVE = ?");
			params.add(persona.getTipoCodigoId().getCodTabla());
		}
		where.append(" AND ( COD_TIPO_CLAVE <> 'E' AND COD_TIPO_CLAVE <> 'F' )");
		if (persona != null && persona.getCodigoId() != null) {
			where.append(" AND NIF_TITULAR = ?");
			params.add(persona.getCodigoId());
		}
		if (persona != null && persona.getApellido1() != null) {
			where.append(" AND NOM_APELLIDO_1 = ?");
			params.add(persona.getApellido1());
		}
		if (persona != null && persona.getApellido2() != null) {
			where.append(" AND NOM_APELLIDO_2 = ?");
			params.add(persona.getApellido2());
		}
		if (persona != null && persona.getNombre() != null) {
			where.append(" AND NOM_NOMBRE = ?");
			params.add(persona.getNombre());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Persona entity
	 * 
	 * @param Persona
	 *            Persona Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereLikeMap(Persona persona, Boolean startsWith) {

		StringBuilder where = new StringBuilder(
				PersonaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (persona != null && persona.getIdPersona() != null) {
			where.append(" AND ID_PERSONA = ?");
			params.add(persona.getIdPersona());
		}
		if (persona != null && persona.getTipoCodigoId() != null && persona.getTipoCodigoId().getCodTabla() != null) {
			where.append(" AND COD_TIPO_CLAVE = ?");
			params.add(persona.getTipoCodigoId().getCodTabla());
		}
		where.append(" AND ( COD_TIPO_CLAVE <> 'E' AND COD_TIPO_CLAVE <> 'F' )");
		if (persona != null && persona.getCodigoId() != null) {
			where.append(" AND UPPER(NIF_TITULAR) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(persona.getCodigoId().toUpperCase() + "%");
			} else {
				params.add("%" + persona.getCodigoId().toUpperCase() + "%");
			}
			where.append(" AND NIF_TITULAR IS NOT NULL");
		}
		if (persona != null && persona.getApellido1() != null) {
			where.append(" AND UPPER(NOM_APELLIDO_1) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(persona.getApellido1().toUpperCase() + "%");
			} else {
				params.add("%" + persona.getApellido1().toUpperCase() + "%");
			}
			where.append(" AND NOM_APELLIDO_1 IS NOT NULL");
		}
		if (persona != null && persona.getApellido2() != null) {
			where.append(" AND UPPER(NOM_APELLIDO_2) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(persona.getApellido2().toUpperCase() + "%");
			} else {
				params.add("%" + persona.getApellido2().toUpperCase() + "%");
			}
			where.append(" AND NOM_APELLIDO_2 IS NOT NULL");
		}
		if (persona != null && persona.getNombre() != null) {
			if (persona.isbFiltroLupa()){
				where.append(" AND UPPER(NOM_NOMBRE) like ? ESCAPE  '\\'");
			}else{
				where.append(" AND ( UPPER(NOM_NOMBRE) || UPPER(NOM_APELLIDO_1) || UPPER(NOM_APELLIDO_2)) like ? ESCAPE  '\\'");
				
			}
			if (startsWith) {
				params.add(persona.getNombre().replaceAll(" ", "").toUpperCase() + "%");
			} else {
				params.add("%" + persona.getNombre().replaceAll(" ", "").toUpperCase() + "%");
			}
			where.append(" AND NOM_NOMBRE IS NOT NULL");
		}
		if (persona != null && persona.getDireccion() != null && persona.getDireccion().getProvincia() != null) {
			BDUtils.crearFiltroDireccion(where, params, "",persona.getDireccion());
		}
		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	/**
	 * Inserta una petición para llamar al SVDI
	 * @param persona
	 * @return
	 */
	public String insertPeticionSVDI(String tipoCodigoId, String codigoId,
			 String apellido1, String fechaNacimiento){
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		String idPeticion=m62FuncionesPLSQL.insertPeticionSVDI(tipoCodigoId,codigoId,apellido1,fechaNacimiento);
		return idPeticion;
		
	}
	/**
	 * Devuelve el resultado de una petición del SVDI
	 * @param persona
	 * @return
	 */
	public Persona getResultadoSVDI(String idPeticion){
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		Persona persona=m62FuncionesPLSQL.recuperarPeticionSVDI(idPeticion);
		return persona;
//		String query = PersonaDaoSql.SQL_FIND;
//		List<Persona> personaList = this.jdbcTemplate.query(query, this.rwMapSVDI,idPeticion);
//		return DataAccessUtils.uniqueResult(personaList);
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
