package com.ejie.m62fb.dao.registroVPP;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.utils.constants.IberErrorsMsgConstants;
import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.utils.format.FormatUtils;
import com.ejie.m62fb.dao.ComboP28Dao;
import com.ejie.m62fb.dao.PersonaDao;
import com.ejie.m62fb.dao.SQLGenericas;
import com.ejie.m62fb.dao.registroVPP.plsql.M62FuncionesPLSQL;
import com.ejie.m62fb.model.ContratoAlquiler;
import com.ejie.m62fb.model.ContratoAlquilerActMasiva;
import com.ejie.m62fb.model.CuentaBancaria;
import com.ejie.m62fb.model.ExpedienteVivienda;
import com.ejie.m62fb.model.Inquilino;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.persona.Persona;
import com.ejie.m62fb.model.tipos.TipoContratoAlquiler;
import com.ejie.m62fb.model.tipos.TipoSiNo;
import com.ejie.m62fb.utils.sql.BDUtils;
import com.ejie.m62fb.utils.sql.UtilsResulSet;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ContratoAlquilerDaoImpl.
 * 
XXXX
 */
@Repository
@Transactional
public class ContratoAlquilerDaoImpl implements ContratoAlquilerDao {
	private JdbcTemplate jdbcTemplate;

	@Autowired
	@SuppressWarnings("rawtypes")
	private ComboP28Dao comboP28Dao;

	@Autowired
	private PersonaDao personaDao;

	@Autowired
	private ReloadableResourceBundleMessageSource messageErrorsSource;

	/*
	 * ROW_MAPPERS
	 */
	private final RowMapper<ContratoAlquiler> rwMap = new RowMapper<ContratoAlquiler>() {
		@Override
		public ContratoAlquiler mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			ContratoAlquiler contratoAlquiler = new ContratoAlquiler();

			// Id Vivienda
			contratoAlquiler.setVivienda(new Vivienda());
			contratoAlquiler.getVivienda().setNumVivienda(
					UtilsResulSet.getBigDecimal(resultSet, "NUMVIVIENDA"));

			// Num. Expediente
			contratoAlquiler.setExpedienteVivienda(new ExpedienteVivienda());
			contratoAlquiler.getExpedienteVivienda().setNumExpediente(
					resultSet.getString("NUMEXPEDIENTE"));

			// Id Inquilino
			contratoAlquiler.setInquilino(new Inquilino());
			contratoAlquiler.getInquilino().setNumInquilino(
					resultSet.getBigDecimal("NUMINQUILINO"));

			// Fecha Inicio
			contratoAlquiler.setFecInicio(resultSet.getTimestamp("FECINICIO"));

			// Fecha Fin
			contratoAlquiler.setFecFin(resultSet.getTimestamp("FECFIN"));

			// Importe Precio Renta
			contratoAlquiler.setImpPrecioRenta(resultSet
					.getBigDecimal("IMPPRECIORENTA"));

			// IMC Precio Renta
			contratoAlquiler.setImcPrecioRenta(resultSet
					.getBigDecimal("IMCPRECIORENTA"));

			// Fecha Envio Contrato
			contratoAlquiler.setFecEnvContrato(resultSet
					.getTimestamp("FECENVCONTRATO"));

			// Fecha Envio Fin Contrato
			contratoAlquiler.setFecEnvFinContrato(resultSet
					.getTimestamp("FECENVFINCONTRATO"));

			// Importe Fianza
			contratoAlquiler.setImpFianza(resultSet.getBigDecimal("IMPFIANZA"));

			// IMC Fianza
			contratoAlquiler.setImcFianza(resultSet.getBigDecimal("IMCFIANZA"));

			// Datos Cuenta Bancaria
			contratoAlquiler.setCuentaBancaria(new CuentaBancaria());
			contratoAlquiler.getCuentaBancaria().setCodEntidad(
					resultSet.getString("CODENTIDAD"));
			contratoAlquiler.getCuentaBancaria().setCodSucursal(
					resultSet.getString("CODSUCURSAL"));
			contratoAlquiler.getCuentaBancaria().setCodDigitoControl(
					resultSet.getString("CODDIGITOCONTROL"));
			contratoAlquiler.getCuentaBancaria().setCodCuenta(
					resultSet.getString("CODCUENTA"));

			// NIF Pagador Extra
			contratoAlquiler.setNifPagadorRenta(resultSet
					.getString("NIFPAGADORRENTA"));

			// Indicador Visado
			contratoAlquiler.setIndVisado(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDVISADO")));

			// Fecha Visado
			contratoAlquiler.setFecVisado(resultSet.getTimestamp("FECVISADO"));

			// ID Contrato
			contratoAlquiler.setIdContrato(resultSet.getInt("IDCONTRATO"));

			// Fecha Ent. Contrato
			contratoAlquiler.setFecEntContrato(resultSet
					.getTimestamp("FECENTCONTRATO"));

			// Fecha Empadronamiento
			contratoAlquiler.setFecEmpadron(resultSet
					.getTimestamp("FECEMPADRON"));

			// Texto Observaciones
			contratoAlquiler.setTexObservaciones(resultSet
					.getString("TEXOBSERVACIONES"));

			// Codigo Tipo Contrato Alquiler
			TipoContratoAlquiler tipoContratoAlquiler = new TipoContratoAlquiler(
					resultSet.getString("CODTIPOCONTRATO"));
			contratoAlquiler
					.setTipoContratoAlquiler((TipoContratoAlquiler) comboP28Dao
							.find(tipoContratoAlquiler));

			// ID Pagador Renta
			Persona pagadorRenta = new Persona(
					resultSet.getBigDecimal("IDPAGADORRENTA"));
			contratoAlquiler.setPagadorRenta(personaDao.find(pagadorRenta));

			// Fecha Inicio Fac.
			contratoAlquiler.setFecIniFac(resultSet.getTimestamp("FECINIFAC"));

			// Indicador Solicita Ayuda
			contratoAlquiler.setIndSolicitaAyuda(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDSOLICITAAYUDA")));

			// Indicador Vivienda Adaptada
			contratoAlquiler.setIndVivAdaptada(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDVIVADAPTADA")));

			// Fecha Escritura
			contratoAlquiler.setFecEscritura(resultSet
					.getTimestamp("FECESCRITURA"));

			// Fecha Contrato
			contratoAlquiler.setFecContrato(resultSet
					.getTimestamp("FECCONTRATO"));

			// Indicador Renuncia
			contratoAlquiler.setIndRenuncia(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDRENUNCIA")));

			// Fecha Renuncia
			contratoAlquiler.setFecRenuncia(resultSet
					.getTimestamp("FECRENUNCIA"));

			// Indicador Denegacion
			contratoAlquiler.setIndDenegacion(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDDENEGACION")));

			// Fecha Denegacion
			contratoAlquiler.setFecDenegacion(resultSet
					.getTimestamp("FECDENEGACION"));

			// DIRECCION
			contratoAlquiler.setDireccion(UtilsResulSet.getDireccionCompleta(
					resultSet, null));

			// AbstractClass
			UtilsResulSet.getAbstractClass(resultSet, contratoAlquiler,
					ContratoAlquilerDaoSql.ALIAS_CONTRATOALQ);

			return contratoAlquiler;
		}
	};

	private final RowMapper<ContratoAlquilerActMasiva> rwMapActMasiva = new RowMapper<ContratoAlquilerActMasiva>() {
		@Override
		public ContratoAlquilerActMasiva mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			ContratoAlquilerActMasiva contratoAlquilerActMasiva = new ContratoAlquilerActMasiva();

			contratoAlquilerActMasiva.setVivienda(new Vivienda());
			contratoAlquilerActMasiva.getVivienda().setNumVivienda(
					UtilsResulSet.getBigDecimal(resultSet, "NUMVIVIENDA"));
			// Fecha Inicio
			contratoAlquilerActMasiva.setFecInicio(resultSet
					.getTimestamp("FECINICIO"));
			// Fecha Fin
			contratoAlquilerActMasiva.setFecFin(resultSet
					.getTimestamp("FECFIN"));
			// Num contrato
			contratoAlquilerActMasiva.setNumContrato(resultSet
					.getString("NUMCONTRATO"));
			// Inquilinos
			contratoAlquilerActMasiva.setInquilinos(resultSet
					.getString("INQUILINOS"));

			return contratoAlquilerActMasiva;
		}
	};

	private final RowMapper<ContratoAlquiler> rwMapPK = new RowMapper<ContratoAlquiler>() {
		@Override
		public ContratoAlquiler mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new ContratoAlquiler(resultSet.getInt("ID_CONTRATO"));
		}
	};

	private final RowMapper<ContratoAlquiler> rwMapValidaciones = new RowMapper<ContratoAlquiler>() {
		@Override
		public ContratoAlquiler mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			ContratoAlquiler contratoAlquiler = new ContratoAlquiler();

			// Id Inquilino
			contratoAlquiler.setInquilino(new Inquilino());
			contratoAlquiler.getInquilino().setNumInquilino(
					resultSet.getBigDecimal("NUMINQUILINO"));

			// ID Contrato
			contratoAlquiler.setIdContrato(resultSet.getInt("IDCONTRATO"));

			return contratoAlquiler;
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return ContratoAlquiler
	 */
	@Override
	public ContratoAlquiler add(ContratoAlquiler contratoAlquiler) {
		String query = ContratoAlquilerDaoSql.SQL_INSERT;
		// Campos para la generación del contrato vivienda
		// Id Vivienda
		BigDecimal getIdViviendaAux = null;
		if (contratoAlquiler.getVivienda() != null
				&& contratoAlquiler.getVivienda().getNumVivienda() != null) {
			getIdViviendaAux = contratoAlquiler.getVivienda().getNumVivienda();
		}
		// Num. Expediente
		String getNumExpedienteAux = null;
		if (contratoAlquiler.getExpedienteVivienda() != null
				&& contratoAlquiler.getExpedienteVivienda().getNumExpediente() != null) {
			getNumExpedienteAux = contratoAlquiler.getExpedienteVivienda()
					.getNumExpediente();
		}
		// Tipo Contrato Alquiler (codigo)
		String getCodTipoContrato = null;
		if (contratoAlquiler.getTipoContratoAlquiler() != null
				&& contratoAlquiler.getTipoContratoAlquiler().getCodTabla() != null) {
			getCodTipoContrato = contratoAlquiler.getTipoContratoAlquiler()
					.getCodTabla();
		}
		// Codigo Entidad
		String getCodEntidad = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodEntidad() != null) {
			getCodEntidad = contratoAlquiler.getCuentaBancaria()
					.getCodEntidad();
		}
		// Codigo Sucursal
		String getCodSucursal = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodSucursal() != null) {
			getCodSucursal = contratoAlquiler.getCuentaBancaria()
					.getCodSucursal();
		}
		// Codigo Digito Control
		String getCodDigitoControl = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodDigitoControl() != null) {
			getCodDigitoControl = contratoAlquiler.getCuentaBancaria()
					.getCodDigitoControl();
		}
		// Codigo Cuenta
		String getCodCuenta = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodCuenta() != null) {
			getCodCuenta = contratoAlquiler.getCuentaBancaria().getCodCuenta();
		}
		// Id Pagador Renta
		BigDecimal getIdPagadorRenta = null;
		if (contratoAlquiler.getPagadorRenta() != null
				&& contratoAlquiler.getPagadorRenta().getIdPersona() != null) {
			getIdPagadorRenta = contratoAlquiler.getPagadorRenta()
					.getIdPersona();
		}
		// Id Inquilino: maximo NUM_INQUILINO de la tabla CONTRATO_ALQUILER
		// incrementandolo en 1. Si no hay contratos es 1.
		if (contratoAlquiler.getInquilino() == null) {
			contratoAlquiler.setInquilino(new Inquilino());
		}
		contratoAlquiler.getInquilino().setNumInquilino(
				this.findNumInquilinoMaxMasUno(getIdViviendaAux));

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Id Vivienda
		params.add(getIdViviendaAux);
		// Num. Expediente
		params.add(getNumExpedienteAux);
		// Id Inquilino calculado
		params.add(contratoAlquiler.getInquilino().getNumInquilino());
		// Fecha Inicio
		params.add(contratoAlquiler.getFecInicio());
		// Fecha Fin
		params.add(contratoAlquiler.getFecFin());
		// Importe precio renta
		params.add(contratoAlquiler.getImpPrecioRenta());
		// Importe (euros) precio renta
		params.add(contratoAlquiler.getImcPrecioRenta());
		// Fecha Envio Contrato
		params.add(contratoAlquiler.getFecEnvContrato());
		// Fecha Envio Fin Contrato
		params.add(contratoAlquiler.getFecEnvFinContrato());
		// Importe Fianza
		params.add(contratoAlquiler.getImpFianza());
		// Importe (euros) Fianza
		params.add(contratoAlquiler.getImcFianza());
		// Codigo Entidad
		params.add(getCodEntidad);
		// Codigo Sucursal
		params.add(getCodSucursal);
		// Codigo Digito Control
		params.add(getCodDigitoControl);
		// Codigo Cuenta
		params.add(getCodCuenta);
		// Indicador Visado
		if (contratoAlquiler.getIndVisado() != null) {
			params.add(contratoAlquiler.getIndVisado().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Visado
		params.add(contratoAlquiler.getFecVisado());
		// Id Contrato
		params.add(contratoAlquiler.getIdContrato());
		// Fecha Entrada Contrato
		params.add(contratoAlquiler.getFecEntContrato());
		// Fecha Empadronamiento
		params.add(contratoAlquiler.getFecEmpadron());
		// Texto Observaciones
		params.add(contratoAlquiler.getTexObservaciones());
		// Tipo Contrato Alquiler (codigo)
		params.add(getCodTipoContrato);
		// Id Pagador Renta
		params.add(getIdPagadorRenta);
		// Fecha Inicio Factura
		params.add(contratoAlquiler.getFecIniFac());
		// Indicador Solicita Ayuda
		if (contratoAlquiler.getIndSolicitaAyuda() != null) {
			params.add(contratoAlquiler.getIndSolicitaAyuda().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Vivienda Adaptada
		if (contratoAlquiler.getIndVivAdaptada() != null) {
			params.add(contratoAlquiler.getIndVivAdaptada().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Escritura
		params.add(contratoAlquiler.getFecEscritura());
		// Fecha Contrato
		params.add(contratoAlquiler.getFecContrato());
		// Indicador Renuncia
		if (contratoAlquiler.getIndRenuncia() != null) {
			params.add(contratoAlquiler.getIndRenuncia().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Renuncia
		params.add(contratoAlquiler.getFecRenuncia());
		// Indicador Denegacion
		if (contratoAlquiler.getIndDenegacion() != null) {
			params.add(contratoAlquiler.getIndDenegacion().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Denegacion
		params.add(contratoAlquiler.getFecDenegacion());

		// Direccion
		SQLGenericas
				.getParamsDireccion(params, contratoAlquiler.getDireccion());
		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, contratoAlquiler);

		this.jdbcTemplate.update(query, params.toArray());
		return contratoAlquiler;
	}

	/**
	 * Updates a single row in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return ContratoAlquiler
	 */
	@Override
	public ContratoAlquiler update(ContratoAlquiler contratoAlquiler) {
		String query = ContratoAlquilerDaoSql.SQL_UPDATE;
		// Campos para la generación del contrato vivienda
		
		// Num. Expediente
		String getNumExpedienteAux = null;
		if (contratoAlquiler.getExpedienteVivienda() != null
				&& contratoAlquiler.getExpedienteVivienda().getNumExpediente() != null) {
			getNumExpedienteAux = contratoAlquiler.getExpedienteVivienda()
					.getNumExpediente();
		}
		// Tipo Contrato Alquiler (codigo)
		String getCodTipoContrato = null;
		if (contratoAlquiler.getTipoContratoAlquiler() != null
				&& contratoAlquiler.getTipoContratoAlquiler().getCodTabla() != null) {
			getCodTipoContrato = contratoAlquiler.getTipoContratoAlquiler()
					.getCodTabla();
		}
		// Codigo Entidad
		String getCodEntidad = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodEntidad() != null) {
			getCodEntidad = contratoAlquiler.getCuentaBancaria()
					.getCodEntidad();
		}
		// Codigo Sucursal
		String getCodSucursal = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodSucursal() != null) {
			getCodSucursal = contratoAlquiler.getCuentaBancaria()
					.getCodSucursal();
		}
		// Codigo Digito Control
		String getCodDigitoControl = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodDigitoControl() != null) {
			getCodDigitoControl = contratoAlquiler.getCuentaBancaria()
					.getCodDigitoControl();
		}
		// Codigo Cuenta
		String getCodCuenta = null;
		if (contratoAlquiler.getCuentaBancaria() != null
				&& contratoAlquiler.getCuentaBancaria().getCodCuenta() != null) {
			getCodCuenta = contratoAlquiler.getCuentaBancaria().getCodCuenta();
		}
		// Id Pagador Renta
		BigDecimal getIdPagadorRenta = null;
		if (contratoAlquiler.getPagadorRenta() != null
				&& contratoAlquiler.getPagadorRenta().getIdPersona() != null) {
			getIdPagadorRenta = contratoAlquiler.getPagadorRenta()
					.getIdPersona();
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Num. Expediente
		params.add(getNumExpedienteAux);
		// Fecha Inicio
		params.add(contratoAlquiler.getFecInicio());
		// Fecha Fin
		params.add(contratoAlquiler.getFecFin());
		// Importe precio renta
		params.add(contratoAlquiler.getImpPrecioRenta());
		// Importe (euros) precio renta
		params.add(contratoAlquiler.getImcPrecioRenta());
		// Fecha Envio Contrato
		params.add(contratoAlquiler.getFecEnvContrato());
		// Fecha Envio Fin Contrato
		params.add(contratoAlquiler.getFecEnvFinContrato());
		// Importe Fianza
		params.add(contratoAlquiler.getImpFianza());
		// Importe (euros) Fianza
		params.add(contratoAlquiler.getImcFianza());
		// Codigo Entidad
		params.add(getCodEntidad);
		// Codigo Sucursal
		params.add(getCodSucursal);
		// Codigo Digito Control
		params.add(getCodDigitoControl);
		// Codigo Cuenta
		params.add(getCodCuenta);
		// Indicador Visado
		if (contratoAlquiler.getIndVisado() != null) {
			params.add(contratoAlquiler.getIndVisado().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Visado
		params.add(contratoAlquiler.getFecVisado());
		// Id Contrato
		params.add(contratoAlquiler.getIdContrato());
		// Fecha Entrada Contrato
		params.add(contratoAlquiler.getFecEntContrato());
		// Fecha Empadronamiento
		params.add(contratoAlquiler.getFecEmpadron());
		// Texto Observaciones
		params.add(contratoAlquiler.getTexObservaciones());
		// Tipo Contrato Alquiler (codigo)
		params.add(getCodTipoContrato);
		// Id Pagador Renta
		params.add(getIdPagadorRenta);
		// Fecha Inicio Factura
		params.add(contratoAlquiler.getFecIniFac());
		// Indicador Solicita Ayuda
		if (contratoAlquiler.getIndSolicitaAyuda() != null) {
			params.add(contratoAlquiler.getIndSolicitaAyuda().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Vivienda Adaptada
		if (contratoAlquiler.getIndVivAdaptada() != null) {
			params.add(contratoAlquiler.getIndVivAdaptada().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Escritura
		params.add(contratoAlquiler.getFecEscritura());
		// Fecha Contrato
		params.add(contratoAlquiler.getFecContrato());
		// Indicador Renuncia
		if (contratoAlquiler.getIndRenuncia() != null) {
			params.add(contratoAlquiler.getIndRenuncia().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Renuncia
		params.add(contratoAlquiler.getFecRenuncia());
		// Indicador Denegacion
		if (contratoAlquiler.getIndDenegacion() != null) {
			params.add(contratoAlquiler.getIndDenegacion().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Denegacion
		params.add(contratoAlquiler.getFecDenegacion());

		// Direccion
		SQLGenericas
				.getParamsDireccion(params, contratoAlquiler.getDireccion());
		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, contratoAlquiler);

		// Id Contrato
		params.add(contratoAlquiler.getIdContrato());

		this.jdbcTemplate.update(query, params.toArray());
		return contratoAlquiler;
	}

	/**
	 * Finds a single row in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return ContratoAlquiler
	 */
	@Override
	@Transactional(readOnly = true)
	public ContratoAlquiler find(ContratoAlquiler contratoAlquiler) {
		String query = ContratoAlquilerDaoSql.SQL_FIND;
		List<ContratoAlquiler> contratoAlquilerList = this.jdbcTemplate.query(query, this.rwMap, contratoAlquiler.getIdContrato());
		return DataAccessUtils.uniqueResult(contratoAlquilerList);
	}

	/**
	 * FindForUpdate
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return ContratoAlquiler
	 */
	@Override
	public ContratoAlquiler findForUpdate(ContratoAlquiler contratoAlquiler) {
		ContratoAlquiler contratoAlquilerBD;
		try {
			contratoAlquilerBD = this.jdbcTemplate.queryForObject(ContratoAlquilerDaoSql.SQL_FIND_FOR_UPDATE, this.rwMap, contratoAlquiler.getIdContrato(), contratoAlquiler.getFecHorUltimaModif());
		} catch (EmptyResultDataAccessException e) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		if (contratoAlquilerBD == null) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		return contratoAlquilerBD;
	}

	/**
	 * Removes a single row in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return
	 */
	@Override
	public void remove(ContratoAlquiler contratoAlquiler) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(this.jdbcTemplate.getDataSource());
		m62FuncionesPLSQL.borrarContratoAlquiler(contratoAlquiler.getIdContrato());
	}

	/**
	 * Finds a list of rows in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ContratoAlquiler>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoAlquiler> findAll(ContratoAlquiler contratoAlquiler,
			JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(contratoAlquiler);
		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/**
	 * Finds a list of rows in the ContratoAlquiler view.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ContratoAlquiler>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoAlquilerActMasiva> findAllActMasiva(
			ContratoAlquilerActMasiva contratoAlquilerActMasiva,
			JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_ALL_ACT_MASIVA);

		// Where clause & Params
		// En este caso solo llegará el num_vivienda
		Map<String, ?> mapaWhere = this
				.getWhereMapActMasiva(contratoAlquilerActMasiva);
		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMapActMasiva,
				params.toArray());
	}

	/**
	 * Finds rows in the ContratoAlquiler table using like.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ContratoAlquiler>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoAlquiler> findAllLike(
			ContratoAlquiler contratoAlquiler,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(contratoAlquiler,
				startsWith);
		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllCount(ContratoAlquiler contratoAlquiler) {
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(contratoAlquiler);
		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Counts rows in the ContratoAlquiler table using like.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllLikeCount(ContratoAlquiler contratoAlquiler,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(contratoAlquiler,
				startsWith);
		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Reorder the data list of ContratoAlquiler selected for rup_table
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ContratoAlquiler>>
	 */
	@Override
	public List<TableRowDto<ContratoAlquiler>> reorderSelection(
			ContratoAlquiler contratoAlquiler,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_ALL);

		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(contratoAlquiler,
				startsWith);
		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(
				query, jqGridRequestDto, ContratoAlquiler.class,
				filterParamList, ContratoAlquilerDaoSql.PK);

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<ContratoAlquiler>(this.rwMapPK,
						jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Returns a Date. Funcion que calcula la fecha fin prorroga forzosa
	 * teniendo en cuenta la fecha inicio, fecha fin y fecha entrada.
	 * 
	 * @param numExpediente
	 *            String
	 * @param idVivienda
	 *            BigDecimal
	 * @return PlSqlOutputError
	 */
	@Override
	@Transactional(readOnly = true)
	public Date calcFecFinProrForzosa(Date fecInicio, Date fecFin,
			Date fecEntrada) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		return m62FuncionesPLSQL.calcularFecFinProrroga(fecInicio, fecFin,
				fecEntrada);
	}

	/**
	 * Returns a BigDecimal. Máximo NUM_INQUILINO que hay para una vivienda en
	 * la tabla CONTRATO_ALQUILER e incrementandolo en 1.
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 * @return BigDecimal
	 */
	@Override
	@Transactional(readOnly = true)
	public BigDecimal findNumInquilinoMaxMasUno(BigDecimal numVivienda) {
		StringBuilder query = new StringBuilder(
				ContratoAlquilerDaoSql.SQL_FIND_NUMINQU_MAX);
		Long registrosCount = this.jdbcTemplate.queryForLong(query.toString(),
				numVivienda);
		return new BigDecimal(registrosCount);

	}

	/**
	 * Funcion que valida la coherencia entre las fechas inicio y fin. Consulta
	 * si estamos en el ultimo contrato. Si estamos en el ultimo contrato, la
	 * fecha inicio debe ser mayor que la fecha inicio del resto de contratos.
	 * Si no estamos en el ultimo contrato, hay que comprobar el rango de fechas
	 * inicio/fin.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return String
	 */
	@Override
	@Transactional(readOnly = true)
	public String validarFechasIniFin(ContratoAlquiler contratoAlquiler) {
		String resultado = null;
		String query = ContratoAlquilerDaoSql.SQL_FIND_ULTIMO_CONTRATO;

		// Id Vivienda
		BigDecimal getIdViviendaAux = null;
		if (contratoAlquiler.getVivienda() != null
				&& contratoAlquiler.getVivienda().getNumVivienda() != null) {
			getIdViviendaAux = contratoAlquiler.getVivienda().getNumVivienda();
		}
		// Id Inquilino
		BigDecimal getIdInquilinoAux = BigDecimal.ZERO;
		if (contratoAlquiler.getInquilino() != null
				&& contratoAlquiler.getInquilino().getNumInquilino() != null) {
			getIdInquilinoAux = contratoAlquiler.getInquilino()
					.getNumInquilino();
		}
		List<Object> params = new ArrayList<Object>();
		// Id Vivienda
		params.add(getIdViviendaAux);
		int ultimoContrato = this.jdbcTemplate.queryForInt(query,
				params.toArray());

		// Si el valor devuelto por la query coincide con el de nuestro
		// contrato, estamos en el ultimo contrato. Si no, no.
		int numInquilinoModificado = 0;
		if (contratoAlquiler.getInquilino() != null
				&& contratoAlquiler.getInquilino().getNumInquilino() != null) {
			numInquilinoModificado = contratoAlquiler.getInquilino()
					.getNumInquilino().intValue();
		}
		if (ultimoContrato == numInquilinoModificado
				|| numInquilinoModificado == 0) {
			// Ultimo contrato (o cuando estamos en el ALTA) -> la fecha inicio
			// debe ser mayor que la fecha inicio del resto de contratos
			String query2 = ContratoAlquilerDaoSql.SQL_FECHA_INI_MAYOR;
			params = new ArrayList<Object>();
			// Fecha Inicio
			params.add(contratoAlquiler.getFecInicio());
			// Id Inquilino
			params.add(getIdInquilinoAux);
			// Id Vivienda
			params.add(getIdViviendaAux);
			int resultadoQuery2 = this.jdbcTemplate.queryForInt(query2,
					params.toArray());
			if (resultadoQuery2 > 0) {
				resultado = IberErrorsMsgConstants.FECHA_INICIO_MAYOR;
			}
		} else {
			// No ultimo -> el rango de fechas inicio/fin introducidos debe ser
			// valido
			String query2 = ContratoAlquilerDaoSql.SQL_RANGO_FECHA_INI_FIN;
			params = new ArrayList<Object>();
			// Fecha Inicio
			params.add(contratoAlquiler.getFecInicio());
			// Id Inquilino
			params.add(getIdInquilinoAux);
			// Fecha Fin
			params.add(contratoAlquiler.getFecFin());
			// Id Inquilino
			params.add(getIdInquilinoAux);
			// Id Inquilino
			params.add(getIdInquilinoAux);
			// Id Vivienda
			params.add(getIdViviendaAux);
			int resultadoQuery2 = this.jdbcTemplate.queryForInt(query2,
					params.toArray());
			if (resultadoQuery2 > 0) {
				resultado = messageErrorsSource.getMessage(
						IberErrorsMsgConstants.RANGO_FECHAS_NO_VALIDO, null,
						LocaleContextHolder.getLocale());
			}
		}
		return resultado;
	}

	/**
	 * Si estamos modificando el ultimo contrato de la vivienda, se valida si
	 * debemos modificar la fecha fin del contrato anterior, bien porque este
	 * sea nulo o sea superior a la fecha inicio de nuestro contrato, con el día
	 * anterior a la fecha de inicio de nuestro contrato.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return Integer
	 */
	@Override
	@Transactional(readOnly = true)
	public Integer validarFechaFinContrato(ContratoAlquiler contratoAlquiler) {
		Integer resultado = null;
		String query = ContratoAlquilerDaoSql.SQL_FIND_CONTRATO_ANTERIOR;

		// Id Vivienda
		BigDecimal getIdViviendaAux = null;
		if (contratoAlquiler.getVivienda() != null
				&& contratoAlquiler.getVivienda().getNumVivienda() != null) {
			getIdViviendaAux = contratoAlquiler.getVivienda().getNumVivienda();
		}
		// Id Inquilino
		BigDecimal getIdInquilinoAux = null;
		if (contratoAlquiler.getInquilino() != null
				&& contratoAlquiler.getInquilino().getNumInquilino() != null) {
			getIdInquilinoAux = contratoAlquiler.getInquilino()
					.getNumInquilino();
		} else {
			getIdInquilinoAux = this
					.findNumInquilinoMaxMasUno(getIdViviendaAux);
		}
		List<Object> params = new ArrayList<Object>();
		// Fecha Inicio
		params.add(contratoAlquiler.getFecInicio());
		// Fecha Fin
		params.add(contratoAlquiler.getFecFin());
		// Fecha Inicio
		params.add(contratoAlquiler.getFecInicio());
		// Id Inquilino
		params.add(getIdInquilinoAux);
		// Id Vivienda
		params.add(getIdViviendaAux);

		List<ContratoAlquiler> resultadoQuery = this.jdbcTemplate.query(
				query.toString(), this.rwMapValidaciones, params.toArray());

		// Si la consulta realizada devuelve un ID_CONTRATO es cuando
		// preguntaremos al usuario sobre la modificacion de la fecha fin del
		// contrato, por lo tanto devolvemos true.
		if (!resultadoQuery.isEmpty()) {
//			if ((resultadoQuery.get(0)).getIdContrato() != null
//					&& !(resultadoQuery.get(0)).getIdContrato().equals("")) {
				resultado = ((resultadoQuery.get(0)).getInquilino()
						.getNumInquilino()).intValue();
//			}
		}
		return resultado;
	}

	/**
	 * Updates a single row in the ContratoAlquiler table.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler
	 * @return ContratoAlquiler
	 */
	@Override
	public ContratoAlquiler updateFecFinContratoAnterior(
			ContratoAlquiler contratoAlquiler, Integer numInquilino) {
		String query = ContratoAlquilerDaoSql.SQL_UPDATE_CONTR_ANTERIOR;

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Fecha Fin (restamos un día a la Fecha Inicio del contrato que estamos
		// modificando)
		params.add(FormatUtils.addRemoveDaysToDate(
				contratoAlquiler.getFecInicio(), -1));
		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, contratoAlquiler);
		// numVivienda
		params.add(contratoAlquiler.getVivienda().getNumVivienda());
		// numInquilino
		params.add(numInquilino);

		this.jdbcTemplate.update(query, params.toArray());
		return contratoAlquiler;
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ContratoAlquiler entity
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(ContratoAlquiler contratoAlquiler) {

		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (contratoAlquiler != null) {
			if (contratoAlquiler.getVivienda() != null) {
				// NumVivienda
				if (contratoAlquiler.getVivienda().getNumVivienda() != null) {
					BDUtils.crearFiltroIgual(where, params,
							"CONTRATOALQ.NUM_VIVIENDA", contratoAlquiler
									.getVivienda().getNumVivienda());
				}
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	private Map<String, ?> getWhereMapActMasiva(
			ContratoAlquiler contratoAlquiler) {

		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (contratoAlquiler != null) {
			if (contratoAlquiler.getVivienda() != null) {
				// NumVivienda
				if (contratoAlquiler.getVivienda().getNumVivienda() != null) {
					BDUtils.crearFiltroIgual(where, params,
							"CONTRATOALQVIVACT.NUM_VIVIENDA", contratoAlquiler
									.getVivienda().getNumVivienda());
				}
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ContratoAlquiler entity
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(
			ContratoAlquiler contratoAlquiler, Boolean startsWith) {

		StringBuilder where = new StringBuilder(
				ContratoAlquilerDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (contratoAlquiler != null) {
			if (contratoAlquiler.getVivienda() != null) {
				// NumVivienda
				if (contratoAlquiler.getVivienda().getNumVivienda() != null) {
					BDUtils.crearFiltroIgual(where, params,
							"CONTRATOALQ.NUM_VIVIENDA", contratoAlquiler
									.getVivienda().getNumVivienda());
				}
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
