package com.ejie.m62fb.dao.registroVPP;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.utils.constants.IberErrorsMsgConstants;
import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.dao.ComboP28Dao;
import com.ejie.m62fb.dao.SQLGenericas;
import com.ejie.m62fb.dao.registroVPP.plsql.M62FuncionesPLSQL;
import com.ejie.m62fb.model.ContratoVivienda;
import com.ejie.m62fb.model.ContratoViviendaActMasiva;
import com.ejie.m62fb.model.ExpedienteVivienda;
import com.ejie.m62fb.model.Propietario;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.plsql.PlSqlOutputError;
import com.ejie.m62fb.model.tipos.TipoSiNo;
import com.ejie.m62fb.model.tipos.TipoTransmision;
import com.ejie.m62fb.utils.sql.BDUtils;
import com.ejie.m62fb.utils.sql.UtilsResulSet;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ContratoViviendaDaoImpl.
 * 
XXXX
 */
@Repository
@Transactional
public class ContratoViviendaDaoImpl implements ContratoViviendaDao {
	private JdbcTemplate jdbcTemplate;
	private static final Logger logger = LoggerFactory.getLogger(ContratoViviendaDaoImpl.class);
	@Autowired
	@SuppressWarnings("rawtypes")
	private ComboP28Dao comboP28Dao;

	/*
	 * ROW_MAPPERS
	 */
	private final RowMapper<ContratoVivienda> rwMap = new RowMapper<ContratoVivienda>() {
		@Override
		public ContratoVivienda mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			ContratoVivienda contratoVivienda = new ContratoVivienda();

			// Id Vivienda
			contratoVivienda.setVivienda(new Vivienda());
			contratoVivienda.getVivienda().setNumVivienda(
					UtilsResulSet.getBigDecimal(resultSet, "NUMVIVIENDA"));

			// Id Propietario
			contratoVivienda.setPropietario(new Propietario());
			contratoVivienda.getPropietario().setNumPropietario(
					resultSet.getBigDecimal("NUMPROPIETARIO"));

			// Num. Expediente
			contratoVivienda.setExpedienteVivienda(new ExpedienteVivienda());
			contratoVivienda.getExpedienteVivienda().setNumExpediente(
					resultSet.getString("NUMEXPEDIENTE"));

			// Tipo Transmisión
			TipoTransmision tipoTransmision = new TipoTransmision(
					resultSet.getString("CODTIPOTRANS"));
			contratoVivienda.setTipoTransmision((TipoTransmision) comboP28Dao
					.find(tipoTransmision));

			// Indicador Renuncia
			contratoVivienda.setIndRenuncia(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDRENUNCIA")));

			// Indicador Revocación (denegado)
			contratoVivienda.setIndRevocacion(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDREVOCACION")));

			// Fecha Transmision
			contratoVivienda.setFecTransmision(resultSet
					.getTimestamp("FECTRANSMISION"));

			// Fecha Escritura
			contratoVivienda.setFecEscritura(resultSet
					.getTimestamp("FECESCRITURA"));

			// Fecha Empadronamiento
			contratoVivienda.setFecEmpadron(resultSet
					.getTimestamp("FECEMPADRON"));

			// Indicador Visado
			contratoVivienda.setIndVisado(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDVISADO")));

			// Fecha Visado
			contratoVivienda.setFecVisado(resultSet.getTimestamp("FECVISADO"));

			// Id. Contrato
			contratoVivienda.setIdContrato(resultSet.getInt("IDCONTRATO"));

			// Fecha Contrato
			contratoVivienda.setFecEntContrato(resultSet
					.getTimestamp("FECENTCONTRATO"));

			// Observaciones
			contratoVivienda.setTexObservaciones(resultSet
					.getString("TEXOBSERVACIONES"));

			// Indicador Solicita Ayuda
			contratoVivienda.setIndSolicitaAyuda(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDSOLICITAAYUDA")));

			// Indicador Vivienda Adaptada
			contratoVivienda.setIndVivAdaptada(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDVIVADAPTADA")));

			// Fecha Renuncia
			contratoVivienda.setFecRenuncia(resultSet
					.getTimestamp("FECRENUNCIA"));

			// Fecha Revocacion
			contratoVivienda.setFecRevocacion(resultSet
					.getTimestamp("FECREVOCACION"));

			// DIRECCION
			contratoVivienda.setDireccion(UtilsResulSet.getDireccionCompleta(
					resultSet, null));
			
			// IDM62ED4T0
			contratoVivienda.setIdM62ed4T0(resultSet.getInt("IDM62ED4T0"));
			
			// AbstractClass
			UtilsResulSet.getAbstractClass(resultSet, contratoVivienda,
					ContratoViviendaDaoSql.ALIAS_CONTRATOVIV);

			return contratoVivienda;
		}
	};

	private final RowMapper<ContratoViviendaActMasiva> rwMapActMasiva = new RowMapper<ContratoViviendaActMasiva>() {
		@Override
		public ContratoViviendaActMasiva mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			ContratoViviendaActMasiva contratoViviendaActMasiva = new ContratoViviendaActMasiva();

			// Id Vivienda
			contratoViviendaActMasiva.setVivienda(new Vivienda());
			contratoViviendaActMasiva.getVivienda().setNumVivienda(
					UtilsResulSet.getBigDecimal(resultSet, "NUMVIVIENDA"));

			// Fecha Transmision
			contratoViviendaActMasiva.setFecTransmision(resultSet
					.getTimestamp("FECTRANSMISION"));

			// Fecha Escritura
			contratoViviendaActMasiva.setFecEscritura(resultSet
					.getTimestamp("FECESCRITURA"));

			// Fecha Empadronamiento
			contratoViviendaActMasiva.setFecEmpadron(resultSet
					.getTimestamp("FECEMPADRON"));

			// Indicador Visado
//			TipoSiNo indVisado = new TipoSiNo(resultSet.getString("INDVISADO"));
//			contratoViviendaActMasiva.setIndVisado((TipoSiNo) comboP28Dao
//					.find(indVisado));
			contratoViviendaActMasiva.setIndVisado(new TipoSiNo(UtilsResulSet
					.getIndicadorSN(resultSet, "INDVISADO")));
			contratoViviendaActMasiva.setIndVisado(UtilsResulSet.getComboP28(
					 resultSet, "INDVISADO",comboP28Dao,
					 TipoSiNo.class));
			
			// Fecha Visado
			contratoViviendaActMasiva.setFecVisado(resultSet
					.getTimestamp("FECVISADO"));

			// Compradores
			contratoViviendaActMasiva.setCompradores(resultSet
					.getString("COMPRADORES"));

			// NumContrato
			contratoViviendaActMasiva.setNumContrato(resultSet
					.getString("NUMCONTRATO"));

			return contratoViviendaActMasiva;
		}
	};

	private final RowMapper<ContratoVivienda> rwMapPK = new RowMapper<ContratoVivienda>() {
		@Override
		public ContratoVivienda mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new ContratoVivienda(resultSet.getInt("IDCONTRATO"));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @return ContratoVivienda
	 */
	@Override
	public ContratoVivienda add(ContratoVivienda contratoVivienda) {
		String query = ContratoViviendaDaoSql.SQL_INSERT;

		// Campos para la generación del contrato vivienda
		// Id Vivienda
		BigDecimal getIdViviendaAux = null;
		if (contratoVivienda.getVivienda() != null
				&& contratoVivienda.getVivienda().getNumVivienda() != null) {
			getIdViviendaAux = contratoVivienda.getVivienda().getNumVivienda();
		}
		// Num. Expediente
		String getNumExpedienteAux = null;
		if (contratoVivienda.getExpedienteVivienda() != null
				&& contratoVivienda.getExpedienteVivienda().getNumExpediente() != null) {
			getNumExpedienteAux = contratoVivienda.getExpedienteVivienda()
					.getNumExpediente();
		}
		// Tipo Transmision (codigo)
		String getCodTipoTransmision = null;
		if (contratoVivienda.getTipoTransmision() != null
				&& contratoVivienda.getTipoTransmision().getCodTabla() != null) {
			getCodTipoTransmision = contratoVivienda.getTipoTransmision()
					.getCodTabla();
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Id Vivienda
		params.add(getIdViviendaAux);
		// Id Propietario: maximo NUM_PROPIETARIO de la tabla CONTRATO_VIVIENDA
		// incrementandolo en 1
		if(contratoVivienda.getPropietario()==null){
			contratoVivienda.setPropietario(new Propietario());
		}
		contratoVivienda.getPropietario().setNumPropietario(this.findNumPropietarioMaxMasUno(getIdViviendaAux));
		params.add(contratoVivienda.getPropietario().getNumPropietario());
		// Num. Expediente
		params.add(getNumExpedienteAux);
		// Tipo Transmision (codigo)
		params.add(getCodTipoTransmision);
		// Indicador Renuncia
		if (contratoVivienda.getIndRenuncia() != null 
				&& contratoVivienda.getIndRenuncia().getCodTabla() != null && !contratoVivienda.getIndRenuncia().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndRenuncia().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Revocación (denegado)
		if (contratoVivienda.getIndRevocacion() != null 
				&& contratoVivienda.getIndRevocacion().getCodTabla() != null && !contratoVivienda.getIndRevocacion().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndRevocacion().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Transmision
		params.add(contratoVivienda.getFecTransmision());
		// Fecha Escritura
		params.add(contratoVivienda.getFecEscritura());
		// Fecha Empadronamiento
		params.add(contratoVivienda.getFecEmpadron());
		// Indicador Visado
		if (contratoVivienda.getIndVisado() != null 
				&& contratoVivienda.getIndVisado().getCodTabla() != null && !contratoVivienda.getIndVisado().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndVisado().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Visado
		params.add(contratoVivienda.getFecVisado());
		// Id. Contrato
		params.add(contratoVivienda.getIdContrato());
		// Fecha Contrato
		params.add(contratoVivienda.getFecEntContrato());
		// Observaciones
		params.add(contratoVivienda.getTexObservaciones());
		// Indicador Solicita Ayuda
		if (contratoVivienda.getIndSolicitaAyuda() != null  
				&& contratoVivienda.getIndSolicitaAyuda().getCodTabla() != null && !contratoVivienda.getIndSolicitaAyuda().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndSolicitaAyuda().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Vivienda Adaptada
		if (contratoVivienda.getIndVivAdaptada() != null 
				&& contratoVivienda.getIndVivAdaptada().getCodTabla() != null && !contratoVivienda.getIndVivAdaptada().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndVivAdaptada().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Renuncia
		params.add(contratoVivienda.getFecRenuncia());
		// Fecha Revocacion
		params.add(contratoVivienda.getFecRevocacion());
		// Direccion
		SQLGenericas
				.getParamsDireccion(params, contratoVivienda.getDireccion());
		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, contratoVivienda);

		this.jdbcTemplate.update(query, params.toArray());

		return contratoVivienda;
	}

	/**
	 * Updates a single row in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @return ContratoVivienda
	 */
	@Override
	public ContratoVivienda update(ContratoVivienda contratoVivienda) {
		String query = ContratoViviendaDaoSql.SQL_UPDATE;

		// Campos para la generación del contrato vivienda
		// Id Vivienda
		BigDecimal getIdViviendaAux = null;
		if (contratoVivienda.getVivienda() != null
				&& contratoVivienda.getVivienda().getNumVivienda() != null) {
			getIdViviendaAux = contratoVivienda.getVivienda().getNumVivienda();
		}
		// Id Propietario
		BigDecimal getIdPropietarioAux = null;
		if (contratoVivienda.getPropietario() != null
				&& contratoVivienda.getPropietario().getNumPropietario() != null) {
			getIdPropietarioAux = contratoVivienda.getPropietario()
					.getNumPropietario();
		}
		// Num. Expediente
		String getNumExpedienteAux = null;
		if (contratoVivienda.getExpedienteVivienda() != null
				&& contratoVivienda.getExpedienteVivienda().getNumExpediente() != null) {
			getNumExpedienteAux = contratoVivienda.getExpedienteVivienda()
					.getNumExpediente();
		}
		// Tipo Transmision (codigo)
		String getCodTipoTransmision = null;
		if (contratoVivienda.getTipoTransmision() != null
				&& contratoVivienda.getTipoTransmision().getCodTabla() != null) {
			getCodTipoTransmision = contratoVivienda.getTipoTransmision()
					.getCodTabla();
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Num. Expediente
		params.add(getNumExpedienteAux);
		// Tipo Transmision (codigo)
		params.add(getCodTipoTransmision);
		// Indicador Renuncia
		if (contratoVivienda.getIndRenuncia() != null 
				&& contratoVivienda.getIndRenuncia().getCodTabla() != null && !contratoVivienda.getIndRenuncia().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndRenuncia().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Revocación (denegado)
		if (contratoVivienda.getIndRevocacion() != null 
				&& contratoVivienda.getIndRevocacion().getCodTabla() != null && !contratoVivienda.getIndRevocacion().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndRevocacion().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Transmision
		params.add(contratoVivienda.getFecTransmision());
		// Fecha Escritura
		params.add(contratoVivienda.getFecEscritura());
		// Fecha Empadronamiento
		params.add(contratoVivienda.getFecEmpadron());
		// Indicador Visado
		if (contratoVivienda.getIndVisado() != null 
				&& contratoVivienda.getIndVisado().getCodTabla() != null && !contratoVivienda.getIndVisado().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndVisado().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Visado
		params.add(contratoVivienda.getFecVisado());
		// Id. Contrato
		params.add(contratoVivienda.getIdContrato());
		// Fecha Contrato
		params.add(contratoVivienda.getFecEntContrato());
		// Observaciones
		params.add(contratoVivienda.getTexObservaciones());
		// Indicador Solicita Ayuda
		if (contratoVivienda.getIndSolicitaAyuda() != null 
				&& contratoVivienda.getIndSolicitaAyuda().getCodTabla() != null && !contratoVivienda.getIndSolicitaAyuda().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndSolicitaAyuda().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Vivienda Adaptada
		if (contratoVivienda.getIndVivAdaptada() != null 
				&& contratoVivienda.getIndVivAdaptada().getCodTabla() != null && !contratoVivienda.getIndVivAdaptada().getCodTabla().isEmpty()) {
			params.add(contratoVivienda.getIndVivAdaptada().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Fecha Renuncia
		params.add(contratoVivienda.getFecRenuncia());
		// Fecha Revocacion
		params.add(contratoVivienda.getFecRevocacion());

		// Direccion
		SQLGenericas
				.getParamsDireccion(params, contratoVivienda.getDireccion());

		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, contratoVivienda);

		// Id Contrato
		params.add(contratoVivienda.getIdContrato());		

		this.jdbcTemplate.update(query, params.toArray());
		return contratoVivienda;
	}

	/**
	 * Finds a single row in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @return ContratoVivienda
	 */
	@Override
	@Transactional(readOnly = true)
	public ContratoVivienda find(ContratoVivienda contratoVivienda) {
		String query = ContratoViviendaDaoSql.SQL_FIND;

		List<ContratoVivienda> contratoViviendaList = this.jdbcTemplate.query(
				query, this.rwMap, contratoVivienda.getIdContrato());
		return DataAccessUtils.uniqueResult(contratoViviendaList);
	}

	/**
	 * FindForUpdate
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @return ContratoVivienda
	 */
	@Override
	public ContratoVivienda findForUpdate(ContratoVivienda contratoVivienda) {
		ContratoVivienda contratoViviendaBD;
		try {
			contratoViviendaBD = this.jdbcTemplate.queryForObject(ContratoViviendaDaoSql.SQL_FIND_FOR_UPDATE, this.rwMap,contratoVivienda.getIdContrato(),contratoVivienda.getFecHorUltimaModif());
		} catch (EmptyResultDataAccessException e) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		if (contratoViviendaBD == null) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		return contratoViviendaBD;
	}

	/**
	 * Removes a single row in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @return
	 */
	@Override
	public void remove(ContratoVivienda contratoVivienda) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(this.jdbcTemplate.getDataSource());
		m62FuncionesPLSQL.borrarContratoCompra(contratoVivienda.getIdContrato());
	}

	/**
	 * Finds a list of rows in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ContratoVivienda>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoVivienda> findAll(ContratoVivienda contratoVivienda,
			JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(contratoVivienda);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}
	
	/**
	 * Finds a list of rows in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ContratoVivienda>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoVivienda> findAllSolPlatea(ContratoVivienda contratoVivienda,
	        JQGridRequestDto jqGridRequestDto) {
	    StringBuilder query = new StringBuilder(
	            ContratoViviendaDaoSql.SQL_FIND_ALL);
	    
	    // Where clause & Params
	    Map<String, ?> mapaWhere = this.getWhereMap(contratoVivienda);
	    StringBuilder where = new StringBuilder(
	            ContratoViviendaDaoSql.SQL_WHERE_SOLPLATEA);
	    where.append(mapaWhere.get("query"));
	    query.append(where);
	    
	    List<?> params = (List<?>) mapaWhere.get("params");
	    
	    if (jqGridRequestDto != null) {
	        query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
	    }
	    
	    return this.jdbcTemplate.query(query.toString(), this.rwMap,
	            params.toArray());
	}

	/**
	 * Finds a list of rows in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ContratoVivienda>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoViviendaActMasiva> findAllActMasiva(
			ContratoViviendaActMasiva contratoVivienda,
			JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL_ACT_MASIVA);

		// Where clause & Params
		// En este caso solo llegará el num_vivienda
		Map<String, ?> mapaWhere = this.getWhereMapActMasiva(contratoVivienda);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMapActMasiva,
				params.toArray());
	}

	/**
	 * Finds rows in the ContratoVivienda table using like.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ContratoVivienda>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ContratoVivienda> findAllLike(
			ContratoVivienda contratoVivienda,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(contratoVivienda,
				startsWith);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the ContratoVivienda table.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllCount(ContratoVivienda contratoVivienda) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(contratoVivienda);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Counts rows in the ContratoVivienda table using like.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllLikeCount(ContratoVivienda contratoVivienda,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(contratoVivienda,
				startsWith);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Reorder the data list of ContratoVivienda selected for rup_table
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ContratoVivienda>>
	 */
	@Override
	public List<TableRowDto<ContratoVivienda>> reorderSelection(
			ContratoVivienda contratoVivienda,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL);

		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(contratoVivienda,
				startsWith);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(
				query, jqGridRequestDto, ContratoVivienda.class,
				filterParamList, ContratoViviendaDaoSql.PK);

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<ContratoVivienda>(this.rwMapPK,
						jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Returns a Boolean. If it exists in CONTRATO_VIVIENDA table true,
	 * otherwise false.
	 * 
	 * @param excepciones
	 *            Excepciones
	 * @return boolean
	 */
	@SuppressWarnings("unchecked")
	@Override
	@Transactional(readOnly = true)
	public boolean findAllCountContratosVivienda(
			ContratoVivienda contratoVivienda) {
		boolean resultado = false;
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(contratoVivienda);
		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<Object> params = (List<Object>) mapaWhere.get("params");
		// EDICIÓN: hay que tener en cuenta que no compare con el registro que
		// se está editando, ya que ya existe en BBDD
		if (contratoVivienda.getVivienda() != null
				&& contratoVivienda.getVivienda().getNumVivienda() != null
				&& !contratoVivienda.getVivienda().getNumVivienda().equals("")
				&& contratoVivienda.getPropietario() != null
				&& contratoVivienda.getPropietario().getNumPropietario() != null
				&& !contratoVivienda.getPropietario().getNumPropietario()
						.equals("")) {
			// añadimos tanto en la query como en los parámetros
			params.add(contratoVivienda.getVivienda().getNumVivienda());
			params.add(contratoVivienda.getPropietario().getNumPropietario());
			query.append(ContratoViviendaDaoSql.SQL_WHERE_NOT_ID);
		}

		Long registrosCount = this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
		// Si la consulta devuelve un mayor número a 0 es que existen contratos
		// en tabla, por lo tanto, devolvemos true. Si no, false.
		if (registrosCount > 0) {
			resultado = true;
		}
		return resultado;
	}

	/**
	 * Returns a BigDecimal. Máximo NUM_PROPIETARIO que hay para una vivienda en
	 * la tabla CONTRATO_VIVIENDA e incrementandolo en 1.
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 * @return BigDecimal
	 */
	@Override
	@Transactional(readOnly = true)
	public BigDecimal findNumPropietarioMaxMasUno(BigDecimal numVivienda) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_NUMPROP_MAX);
		Long registrosCount = this.jdbcTemplate.queryForLong(query.toString(),
				numVivienda);
		return new BigDecimal(registrosCount);

	}

	/**
	 * Returns a PlSqlOutputError. Funcion que valida si al visar un contrato se
	 * esta superando el importe maximo autorizado en la promocion.
	 * 
	 * @param numExpediente
	 *            String
	 * @param idVivienda
	 *            BigDecimal
	 * @return PlSqlOutputError
	 */
	@Override
	@Transactional(readOnly = true)
	public PlSqlOutputError validarImporteAutorizado(String numExpediente,
			BigDecimal idVivienda) {
		// Formateamos el numExpediente, para sustituir el caracter "¬" por la
		// barra "/"
		logger.info(" -> validarImporteAutorizado numExpediente: "
				+ numExpediente.replace("¬", "/"));
		logger.info(" -> validarImporteAutorizado numExpediente: "
				+ numExpediente);
		numExpediente = numExpediente.replace(".", "/");
		logger.info(" -> validarImporteAutorizado numExpediente: "
				+ numExpediente);
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		return m62FuncionesPLSQL.validarImporteAutorizado(numExpediente,
				idVivienda);
	}

	/**
	 * Returns a Long. Funcion que busca si existe algun contrato de primera
	 * transmision para la vivienda seleccionada.
	 * 
	 * @param BigDecimal
	 *            numVivienda
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findContratoPrimerTrans(BigDecimal numVivienda) {
		StringBuilder query = new StringBuilder(
				ContratoViviendaDaoSql.SQL_FIND_PRIMERTIPOTRANS);
		return this.jdbcTemplate.queryForLong(query.toString(), numVivienda);
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ContratoVivienda entity
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(ContratoVivienda contratoVivienda) {

		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (contratoVivienda != null) {
			if (contratoVivienda.getVivienda() != null) {
				// NumVivienda
				if (contratoVivienda.getVivienda().getNumVivienda() != null) {
					BDUtils.crearFiltroIgual(where, params,
							"CONTRATOVIV.NUM_VIVIENDA", contratoVivienda
									.getVivienda().getNumVivienda());
				}
			}
			// Fecha Transmision
			if (contratoVivienda.getFecTransmision() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"CONTRATOVIV.FEC_TRANSMISION",
						contratoVivienda.getFecTransmision());
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	private Map<String, ?> getWhereMapActMasiva(
			ContratoVivienda contratoVivienda) {

		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (contratoVivienda != null) {
			if (contratoVivienda.getVivienda() != null) {
				// NumVivienda
				if (contratoVivienda.getVivienda().getNumVivienda() != null) {
					BDUtils.crearFiltroIgual(where, params,
							"CONTRATOVIVACT.NUM_VIVIENDA", contratoVivienda
									.getVivienda().getNumVivienda());
				}
			}
			// Fecha Transmision
			if (contratoVivienda.getFecTransmision() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"CONTRATOVIVACT.FEC_TRANSMISION",
						contratoVivienda.getFecTransmision());
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ContratoVivienda entity
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(
			ContratoVivienda contratoVivienda, Boolean startsWith) {

		StringBuilder where = new StringBuilder(
				ContratoViviendaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (contratoVivienda != null) {
			if (contratoVivienda.getVivienda() != null) {
				// NumVivienda
				if (contratoVivienda.getVivienda().getNumVivienda() != null) {
					BDUtils.crearFiltroIgual(where, params,
							"CONTRATOVIV.NUM_VIVIENDA", contratoVivienda
									.getVivienda().getNumVivienda());
				}
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
