package com.ejie.m62fb.dao.registroVPP;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.utils.constants.IberErrorsMsgConstants;
import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.dao.ComboP28Dao;
import com.ejie.m62fb.dao.PersonaDao;
import com.ejie.m62fb.dao.SQLGenericas;
import com.ejie.m62fb.dao.registroVPP.plsql.M62FuncionesPLSQL;
import com.ejie.m62fb.model.ExpedienteVivienda;
import com.ejie.m62fb.model.Propietario;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.persona.Persona;
import com.ejie.m62fb.model.tipos.TipoParentesco;
import com.ejie.m62fb.model.tipos.TipoSiNo;
import com.ejie.m62fb.utils.sql.BDUtils;
import com.ejie.m62fb.utils.sql.UtilsResulSet;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * PropietarioDaoImpl generated by UDA, 23-sep-2015 11:53:47.
 * 
XXXX
 */

@Repository
@Transactional
public class PropietarioDaoImpl implements PropietarioDao {

	@Autowired
	@SuppressWarnings("rawtypes")
	ComboP28Dao comboP28Dao;

	@Autowired
	@SuppressWarnings("rawtypes")
	PersonaDao personaDao;

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private final RowMapper<Propietario> rwMap = new RowMapper<Propietario>() {
		@SuppressWarnings("unchecked")
		@Override
		public Propietario mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			Propietario propietario = new Propietario();

			// Id Propietario
			propietario.setNumPropietario(resultSet
					.getBigDecimal("NUMPROPIETARIO"));

			// Id Vivienda
			propietario.setVivienda(new Vivienda());
			propietario.getVivienda().setNumVivienda(
					UtilsResulSet.getBigDecimal(resultSet, "NUMVIVIENDA"));

			// Num. Expediente
			propietario.setExpedienteVivienda(new ExpedienteVivienda());
			propietario.getExpedienteVivienda().setNumExpediente(
					resultSet.getString("NUMEXPEDIENTE"));

			// Persona
			Persona persona = new Persona(resultSet.getBigDecimal("IDPERSONA"));
			propietario.setPersona(personaDao.find(persona));

			// Cod. Parentesco
			propietario.setTipoParentesco((TipoParentesco) comboP28Dao
					.find(new TipoParentesco(resultSet
							.getString("CODPARENTESCO"))));

			// Indicador Titular
			TipoSiNo indTitular = new TipoSiNo(
					resultSet.getString("INDTITULAR"));
			propietario.setIndPrimerSolic((TipoSiNo) comboP28Dao
					.find(indTitular));

			// Fecha Alta
			propietario.setFecAlta(resultSet.getTimestamp("FECALTA"));

			// Fecha/Hora Ultima Modificacion
			propietario.setFecHorUltimaModif(resultSet
					.getTimestamp("FEHULTMODIF"));

			// Codigo Usuario Ultima Moficacion
			propietario.setCodUsuarioUltimaModif(resultSet
					.getString("CODUSUULTMODIF"));

			// Indicador Vivienda Adaptada
			TipoSiNo indVivAdaptada = new TipoSiNo(
					resultSet.getString("INDVIVADAPTADA"));
			propietario.setIndVivAdaptada((TipoSiNo) comboP28Dao
					.find(indVivAdaptada));

			// Porcentaje Titularidad
			propietario.setPorTitularidad(resultSet
					.getBigDecimal("PORTITULARIDAD"));

			// Indicador Envio Registro Propio
			TipoSiNo indEnvRegProp = new TipoSiNo(
					resultSet.getString("INDENVREGPROP"));
			propietario.setIndEnvRegProp((TipoSiNo) comboP28Dao
					.find(indEnvRegProp));

			// Indicador Primer Solicitante
			TipoSiNo indPrimerSolic = new TipoSiNo(
					resultSet.getString("INDPRIMERSOLIC"));
			propietario.setIndPrimerSolic((TipoSiNo) comboP28Dao
					.find(indPrimerSolic));
			
			// ID_M62R04T0
			propietario.setIdM62r04t0(resultSet
					.getInt("IDM62R04T0"));
			// ID_CONTRATO
			propietario.setIdContrato(resultSet
					.getInt("IDCONTRATO"));

			return propietario;
		}
	};

	private final RowMapper<Propietario> rwMapPK = new RowMapper<Propietario>() {
		@Override
		public Propietario mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new Propietario(resultSet.getInt("IDM62R04T0"));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
	public Propietario add(Propietario propietario) {
		String query = PropietarioDaoSql.SQL_INSERT;

		// Campos para la generación del titular
		// Id Vivienda
		BigDecimal getIdViviendaAux = null;
		if (propietario.getVivienda() != null
				&& propietario.getVivienda().getNumVivienda() != null) {
			getIdViviendaAux = propietario.getVivienda().getNumVivienda();
		}
		// Num. Expediente
		String getNumExpedienteAux = null;
		if (propietario.getExpedienteVivienda() != null
				&& propietario.getExpedienteVivienda().getNumExpediente() != null) {
			getNumExpedienteAux = propietario.getExpedienteVivienda()
					.getNumExpediente();
		}
//		GAZAMAGO
//		// Persona (NIFTITULAR)
//		String getCodIdPersona = null;
//		if (propietario.getPersona() != null
//				&& propietario.getPersona().getCodigoId() != null) {
//			getCodIdPersona = propietario.getPersona().getCodigoId();
//		}
		// Tipo Parentesco (codigo)
		String getCodTipoParentesco = null;
		if (propietario.getTipoParentesco() != null
				&& propietario.getTipoParentesco().getCodTabla() != null) {
			getCodTipoParentesco = propietario.getTipoParentesco()
					.getCodTabla();
		}
		// Id Persona
		BigDecimal getIdPersona = null;
		if (propietario.getPersona() != null
				&& propietario.getPersona().getIdPersona() != null) {
			getIdPersona = propietario.getPersona().getIdPersona();
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Id Vivienda
		params.add(getIdViviendaAux);
		// Id Propietario
		params.add(propietario.getNumPropietario());
		// Id Persona
		params.add(getIdPersona);
		// Num. Expediente
		params.add(getNumExpedienteAux);		
//		GAZAMAGO
//		// Persona (NIFTITULAR)
//		params.add(getCodIdPersona);
		// Tipo Parentesco
		params.add(getCodTipoParentesco);
		// Indicador Titular
		if (propietario.getIndTitular() != null) {
			params.add(propietario.getIndTitular().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Vivienda Adaptada
		if (propietario.getIndVivAdaptada() != null) {
			params.add(propietario.getIndVivAdaptada().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Porcentaje Titularidad
		params.add(propietario.getPorTitularidad());
		// Indicador Envio Reg. Propio
		if (propietario.getIndEnvRegProp() != null) {
			params.add(propietario.getIndEnvRegProp().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Primer Solicitante
		if (propietario.getIndPrimerSolic() != null) {
			params.add(propietario.getIndPrimerSolic().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Id Contrato
		params.add(propietario.getIdContrato());

		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, propietario);

		this.jdbcTemplate.update(query, params.toArray());
		return propietario;
	}

	/**
	 * Updates a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
	public Propietario update(Propietario propietario) {
		String query = PropietarioDaoSql.SQL_UPDATE;

		// Num. Expediente
		String getNumExpedienteAux = null;
		if (propietario.getExpedienteVivienda() != null
				&& propietario.getExpedienteVivienda().getNumExpediente() != null) {
			getNumExpedienteAux = propietario.getExpedienteVivienda()
					.getNumExpediente();
		}
//		GAZAMAGO
//		// Persona (NIFTITULAR)
//		String getCodIdPersona = null;
//		if (propietario.getPersona() != null
//				&& propietario.getPersona().getCodigoId() != null) {
//			getCodIdPersona = propietario.getPersona().getCodigoId();
//		}
		// Tipo Parentesco (codigo)
		String getCodTipoParentesco = null;
		if (propietario.getTipoParentesco() != null
				&& propietario.getTipoParentesco().getCodTabla() != null) {
			getCodTipoParentesco = propietario.getTipoParentesco()
					.getCodTabla();
		}
		// Id Persona
		BigDecimal getIdPersona = null;
		if (propietario.getPersona() != null
				&& propietario.getPersona().getIdPersona() != null) {
			getIdPersona = propietario.getPersona().getIdPersona();
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Num. Expediente
		params.add(getNumExpedienteAux);
//		GAZAMAGO
//		// Persona (NIFTITULAR)
//		params.add(getCodIdPersona);
		// Tipo Parentesco
		params.add(getCodTipoParentesco);
		// Indicador Titular
		if (propietario.getIndTitular() != null) {
			params.add(propietario.getIndTitular().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Vivienda Adaptada
		if (propietario.getIndVivAdaptada() != null) {
			params.add(propietario.getIndVivAdaptada().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Porcentaje Titularidad
		params.add(propietario.getPorTitularidad());
		// Indicador Envio Reg. Propio
		if (propietario.getIndEnvRegProp() != null) {
			params.add(propietario.getIndEnvRegProp().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}
		// Indicador Primer Solicitante
		if (propietario.getIndPrimerSolic() != null) {
			params.add(propietario.getIndPrimerSolic().getCodTabla());
		} else {
			params.add(BDUtils.FALSE);
		}

		// CodUsuario
		SQLGenericas.getParamsAbstractClass(params, propietario);

		// Campos WHERE
		// Id_M62R04T0
		params.add(propietario.getIdM62r04t0());
		
		this.jdbcTemplate.update(query, params.toArray());
		return propietario;
	}

	/**
	 * Finds a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
	@Transactional(readOnly = true)
	public Propietario find(Propietario propietario) {
		String query = PropietarioDaoSql.SQL_FIND;
		List<Propietario> propietarioList = this.jdbcTemplate.query(query,this.rwMap, propietario.getIdM62r04t0());
		return DataAccessUtils.uniqueResult(propietarioList);
	}

	/**
	 * FindForUpdate
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
	public Propietario findForUpdate(Propietario propietario) {
		Propietario propietarioBD;
		try {
			propietarioBD = this.jdbcTemplate.queryForObject(PropietarioDaoSql.SQL_FIND_FOR_UPDATE,this.rwMap, propietario.getIdM62r04t0(), propietario.getFecHorUltimaModif());
		} catch (EmptyResultDataAccessException e) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		if (propietarioBD == null) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		return propietarioBD;
	}

	/**
	 * Removes a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return
	 */
	@Override
	public void remove(Propietario propietario) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(this.jdbcTemplate.getDataSource());
		m62FuncionesPLSQL.borrarPropietarioContratoCompra(propietario.getIdM62r04t0());
	}

	/**
	 * Finds a list of rows in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Propietario>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<Propietario> findAll(Propietario propietario,
			JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(PropietarioDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(propietario);
		StringBuilder where = new StringBuilder(PropietarioDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/**
	 * Finds rows in the Propietario table using like.
	 * 
	 * @param propietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Propietario>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<Propietario> findAllLike(Propietario propietario,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(PropietarioDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this
				.getWhereLikeMap(propietario, startsWith);
		StringBuilder where = new StringBuilder(PropietarioDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllCount(Propietario propietario) {
		StringBuilder query = new StringBuilder(
				PropietarioDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(propietario);
		StringBuilder where = new StringBuilder(PropietarioDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Counts rows in the Propietario table using like.
	 * 
	 * @param propietario
	 *            Propietario
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Propietario propietario, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				PropietarioDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this
				.getWhereLikeMap(propietario, startsWith);
		StringBuilder where = new StringBuilder(PropietarioDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Reorder the data list of Propietario selected for rup_table
	 * 
	 * @param propietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Propietario>>
	 */
	@Override
	public List<TableRowDto<Propietario>> reorderSelection(
			Propietario propietario, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(PropietarioDaoSql.SQL_FIND_ALL);

		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this
				.getWhereLikeMap(propietario, startsWith);
		StringBuilder where = new StringBuilder(PropietarioDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(
				query, jqGridRequestDto, Propietario.class, filterParamList,
				PropietarioDaoSql.PK);

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Propietario>(this.rwMapPK,
						jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Calcula la suma del porcentaje de titularidad teniendo en cuenta el % de
	 * titularidad que se esta modificando. Si la suma de porcentajes es mayor
	 * que 100, true. En caso contrario, false.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return boolean
	 */
	@Override
	@Transactional(readOnly = true)
	public boolean findPorcentTitularidad(Propietario propietario) {
		boolean resultado = false;
		String query = "SELECT SUM(PROP.POR_TITULARIDAD) FROM PROPIETARIOS PROP WHERE PROP.ID_PERSONA != "
				+ propietario.getPersona().getIdPersona()
				+ " AND PROP.ID_CONTRATO = "
				+ propietario.getIdContrato();

		int porcentSumaBD = this.jdbcTemplate.queryForInt(query);
		// Al porcent devuelto por la query, sumamos el porcentaje del titular
		// que se esta modificando
		int porcentSumaTotal = porcentSumaBD
				+ propietario.getPorTitularidad().intValue();
		// Si la suma de porcentajes es mayor que 100, hay que devolver true
		// para luego mostrar un mensaje de error en pantalla.
		if (porcentSumaTotal > 100) {
			resultado = true;
		}
		return resultado;
	}

	/**
	 * Funcion que mira si hay algun otro titular en el contrato con el campo
	 * IND_PRIMER_SOLIC = 'S'.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return boolean
	 */
	@Override
	@Transactional(readOnly = true)
	public boolean findTitularPrimerSolic(Propietario propietario) {
		boolean resultado = false;
		String query = "SELECT COUNT(1) FROM PROPIETARIOS PROP WHERE IND_PRIMER_SOLIC = 'S' AND PROP.ID_PERSONA != "
				+ propietario.getPersona().getIdPersona()
				+ " AND PROP.ID_CONTRATO = "
				+ propietario.getIdContrato();
		Long registrosCount = this.jdbcTemplate.queryForLong(query);

		// Si la consulta devuelve un mayor número a 0 es que existe otro
		// titular como primer solicitante en tabla, por lo tanto, devolvemos
		// true. Si no, false.
		if (registrosCount > 0) {
			resultado = true;
		}
		return resultado;
	}

	/**
	 * Funcion que consulta si el titular esta en otro contrato de otra
	 * vivienda.
	 * 
	 * @param idVivienda
	 *            Integer
	 * @param idPersona
	 *            Integer
	 * @return boolean
	 */
	@Override
	@Transactional(readOnly = true)
	public boolean findTitularesContratos(Integer idVivienda, Integer idPersona) {
		boolean resultado = false;
		String query = "SELECT COUNT(1) FROM PROPIETARIOS PROP WHERE PROP.NUM_VIVIENDA != "
				+ idVivienda + " AND PROP.ID_PERSONA = " + idPersona;
		Long registrosCount = this.jdbcTemplate.queryForLong(query);

		// Si la consulta devuelve un mayor número a 0 es que existe ese titular
		// esta en otro contrato de otra vivienda por lo tanto, devolvemos true.
		// Si no, false.
		if (registrosCount > 0) {
			resultado = true;
		}
		return resultado;
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Propietario entity
	 * 
	 * @param propietario
	 *            Propietario Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Propietario propietario) {

		StringBuilder where = new StringBuilder(
				PropietarioDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (propietario != null) {
			if (propietario.getVivienda() != null) {
				// NumVivienda
				if (propietario.getVivienda().getNumVivienda() != null && propietario.getVivienda().getNumVivienda() != new BigDecimal(0)) {
					BDUtils.crearFiltroIgual(where, params,"PROP.NUM_VIVIENDA", 
							propietario.getVivienda().getNumVivienda());
				}
			}
			// NumPropietario
			if (propietario.getNumPropietario() != null) {
				BDUtils.crearFiltroIgual(where, params, "PROP.NUM_PROPIETARIO",
						propietario.getNumPropietario());
			}
			if (propietario.getPersona() != null) {
				// IdPersona
				if (propietario.getPersona().getIdPersona() != null) {
					BDUtils.crearFiltroIgual(where, params, "PROP.ID_PERSONA",
							propietario.getPersona().getIdPersona());
				}
			}
			// IdContrato
			if (propietario.getIdContrato() != null) {
				BDUtils.crearFiltroIgual(where, params, "PROP.ID_CONTRATO",
						propietario.getIdContrato());
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Propietario entity
	 * 
	 * @param propietario
	 *            Propietario Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Propietario propietario,
			Boolean startsWith) {

		StringBuilder where = new StringBuilder(
				PropietarioDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (propietario != null) {
			if (propietario.getVivienda() != null) {
				// NumVivienda
				if (propietario.getVivienda().getNumVivienda() != null && propietario.getVivienda().getNumVivienda() != new BigDecimal(0)) {
					BDUtils.crearFiltroIgual(where, params, "PROP.NUM_VIVIENDA", 
							propietario.getVivienda().getNumVivienda());
				}
			}
			// NumPropietario
			if (propietario.getNumPropietario() != null) {
				BDUtils.crearFiltroIgual(where, params, "PROP.NUM_PROPIETARIO",
						propietario.getNumPropietario());
			}
			if (propietario.getPersona() != null) {
				// IdPersona
				if (propietario.getPersona().getIdPersona() != null) {
					BDUtils.crearFiltroIgual(where, params, "PROP.ID_PERSONA",
							propietario.getPersona().getIdPersona());
				}
			}
			// IdContrato
			if (propietario.getIdContrato() != null) {
				BDUtils.crearFiltroIgual(where, params, "PROP.ID_CONTRATO",
						propietario.getIdContrato());
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
