package com.ejie.m62fb.dao.registroVPP;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.m62fb.utils.constants.IberErrorsMsgConstants;
import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.dao.SQLGenericas;
import com.ejie.m62fb.dao.registroVPP.plsql.M62FuncionesPLSQL;
import com.ejie.m62fb.model.AcuerdoVivienda;
import com.ejie.m62fb.model.ViviendaDisponible;
import com.ejie.m62fb.model.ViviendaReservada;
import com.ejie.m62fb.model.ViviendaSolicitada;
import com.ejie.m62fb.model.direccion.Direccion;
import com.ejie.m62fb.model.tipos.RegimenAcceso;
import com.ejie.m62fb.model.tipos.RegimenTenencia;
import com.ejie.m62fb.utils.constants.AppConstantes;
import com.ejie.m62fb.utils.sql.BDUtils;
import com.ejie.m62fb.utils.sql.UtilsResulSet;
import com.ejie.t17i.model.Municipio;
import com.ejie.t17i.model.Provincia;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ViviendaReservadaDaoImpl generated by UDA, 20-nov-2015 10:16:26.
 * 
XXXX
 */

@Repository
@Transactional
public class ViviendaReservadaDaoImpl implements ViviendaReservadaDao {
	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private final RowMapper<ViviendaReservada> rwMap = new RowMapper<ViviendaReservada>() {
		@Override
		public ViviendaReservada mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			ViviendaReservada viviendaReservada = new ViviendaReservada();

			// Identificador Vivienda Reservada
			viviendaReservada.setIdM62r43t0(UtilsResulSet.getBigDecimal(
					resultSet, "IDM62R43T0"));
			viviendaReservada.setIdM62r41t0(UtilsResulSet.getInteger(resultSet,
					"IDM62R41T0"));

			// Identificador Vivienda Solicitada
			ViviendaSolicitada viviendaSolicitada = new ViviendaSolicitada();
			viviendaSolicitada.setIdM62r40t0(UtilsResulSet.getBigDecimal(
					resultSet, "IDM62R40T0"));
			viviendaReservada.setViviendaSolicitada(viviendaSolicitada);

			// Vivienda Disponible
			ViviendaDisponible viviendaDisponible = new ViviendaDisponible();
			// -> Num. Vivienda
			viviendaDisponible.setNumVivienda(UtilsResulSet.getBigDecimal(
					resultSet, "NUMVIVIENDA"));
			// -> Indicador Vivienda Adaptada
			viviendaDisponible.setIndVivAdaptada(resultSet
					.getString("INDVIVADAPTADA"));
			// -> Nº Dormitorios
			viviendaDisponible.setNumDormitorios(UtilsResulSet.getLong(
					resultSet, "NUMDORMITORIOS"));
			// -> Superficie Util Vivienda
			viviendaDisponible.setSupUtilViv(UtilsResulSet.getBigDecimal(
					resultSet, "SUPUTILVIV"));
			// -> Numero Garaje
			viviendaDisponible.setCodElemG(UtilsResulSet.getString(resultSet,
					"CODELEMENTOG"));
			// -> Numero Trastero
			viviendaDisponible.setCodElemT(UtilsResulSet.getString(resultSet,
					"CODELEMENTOT"));

			// -> Provincia (Codigo/Descripcion)
			Direccion direccion = new Direccion();
			Provincia provincia = new Provincia();
			provincia.setId(resultSet.getString("CODPROVINCIA"));
			provincia.setNombre(resultSet.getString("DESCPROVINCIA"));
			direccion.setProvincia(provincia);
			// -> Municipio (Codigo/Descripcion)
			Municipio municipio = new Municipio();
			municipio.setId(resultSet.getString("CODMUNICIPIO"));
			municipio.setNombre(resultSet.getString("DESCMUNICIPIO"));
			direccion.setMunicipio(municipio);
			// -> Direccion (provincia y municipio)
			viviendaDisponible.setDireccion(direccion);

			// -> Regimen Acceso
			RegimenAcceso regimenAcceso = new RegimenAcceso();
			regimenAcceso.setCodTabla(UtilsResulSet.getString(resultSet,
					"CODREGACCESO"));
			regimenAcceso.setDescripcionEs(UtilsResulSet.getString(resultSet,
					"DESCREGACCESOES"));
			regimenAcceso.setDescripcionEu(UtilsResulSet.getString(resultSet,
					"DESCREGACCESOEU"));
			viviendaDisponible.setRegimenAcceso(regimenAcceso);

			// -> Regimen Tenencia
			RegimenTenencia regimenTenencia = new RegimenTenencia();
			regimenTenencia.setCodTabla(resultSet.getString("CODREGTENENCIA"));
			regimenTenencia.setDescripcionEs(resultSet
					.getString("DESCREGTENENCIAES"));
			regimenTenencia.setDescripcionEu(resultSet
					.getString("DESCREGTENENCIAEU"));
			viviendaDisponible.setRegimenTenencia(regimenTenencia);

			// -> Nº Expediente
			viviendaDisponible.setNumExpediente(resultSet
					.getString("NUMEXPEDIENTE"));
			// -> Importe PVP Total
			viviendaDisponible.setImcPvpTotal(UtilsResulSet.getBigDecimal(
					resultSet, "IMCPVPTOTAL"));
			// Set
			viviendaReservada.setViviendaDisponible(viviendaDisponible);
			// Importe PVP Total
			viviendaReservada.setImcPvpTotal(UtilsResulSet.getBigDecimal(
					resultSet, "IMCPVPTOTAL"));
			// Indicador Reserva Temporal
			viviendaReservada.setIndReservaTemp(UtilsResulSet.getIndicadorSN(
					resultSet, "INDRESERVATEMP"));
			// Fecha Reserva
			viviendaReservada.setFecReserva(resultSet
					.getTimestamp("FECRESERVA"));
			// Fecha Fin Reserva
			viviendaReservada.setFecFinReserva(resultSet
					.getTimestamp("FECFINRESERVA"));
			// Observaciones
			viviendaReservada.setTexObservaciones(UtilsResulSet.getString(
					resultSet, "TEXOBSERVACIONES"));

			return viviendaReservada;
		}
	};

	private final RowMapper<ViviendaReservada> rwMapPK = new RowMapper<ViviendaReservada>() {
		@Override
		public ViviendaReservada mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new ViviendaReservada(resultSet.getBigDecimal("IDM62R43T0"));
		}
	};

	/**
	 * Method use to set the datasource.
	 * 
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the ViviendaReservada table.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @return ViviendaReservada
	 */
	@Override
	public ViviendaReservada add(ViviendaReservada viviendareservada) {
		String query = ViviendaReservadaDaoSql.SQL_INSERT;
		// Campos para la generación del contrato vivienda
		BigDecimal getIdViviendaSolicAux = null;
		BigDecimal getNumViviendaDispAux = null;
		BigDecimal getImportePVPTotalAux = null;
		if (viviendareservada.getViviendaSolicitada() != null) {
			// Id Vivienda Solicitada
			if (viviendareservada.getViviendaSolicitada().getIdM62r40t0() != null) {
				getIdViviendaSolicAux = viviendareservada
						.getViviendaSolicitada().getIdM62r40t0();
			}
		}
		if (viviendareservada.getViviendaDisponible() != null) {
			// Num. Vivienda
			if (viviendareservada.getViviendaDisponible().getNumVivienda() != null) {
				getNumViviendaDispAux = viviendareservada
						.getViviendaDisponible().getNumVivienda();
			}
			// Importe PVP Total
			if (viviendareservada.getViviendaDisponible().getImcPvpTotal() != null) {
				getImportePVPTotalAux = viviendareservada
						.getViviendaDisponible().getImcPvpTotal();
			}
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Identificador Vivienda Reservada
		params.add(viviendareservada.getIdM62r43t0());
		// IdM62r41t0
		params.add(viviendareservada.getIdM62r41t0());
		// Id Vivienda Solicitada
		params.add(getIdViviendaSolicAux);
		// Num. Vivienda
		params.add(getNumViviendaDispAux);
		// Importe PVP Total
		params.add(getImportePVPTotalAux);
		// Indicador Reserva Temporal
		params.add(BDUtils.formatIndicadorToBD(viviendareservada
				.getIndReservaTemp()));
		// Fecha Reserva
		params.add(viviendareservada.getFecReserva());
		// Fecha Fin Reserva
		params.add(viviendareservada.getFecFinReserva());
		// Observaciones
		params.add(viviendareservada.getTexObservaciones());
		// AbstractM62FBModel
		SQLGenericas.getParamsAbstractClass(params, viviendareservada);

		this.jdbcTemplate.update(query, params.toArray());
		return viviendareservada;
	}

	/**
	 * Updates a single row in the ViviendaReservada table.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @return ViviendaReservada
	 */
	@Override
	public ViviendaReservada update(ViviendaReservada viviendareservada) {
		String query = ViviendaReservadaDaoSql.SQL_UPDATE;
		// Campos para la generación del contrato vivienda
		BigDecimal getIdViviendaSolicAux = null;
		BigDecimal getNumViviendaDispAux = null;
		BigDecimal getImportePVPTotalAux = null;
		if (viviendareservada.getViviendaSolicitada() != null) {
			// Id Vivienda Solicitada
			if (viviendareservada.getViviendaSolicitada().getIdM62r40t0() != null) {
				getIdViviendaSolicAux = viviendareservada
						.getViviendaSolicitada().getIdM62r40t0();
			}
		}
		if (viviendareservada.getViviendaDisponible() != null) {
			// Num. Vivienda
			if (viviendareservada.getViviendaDisponible().getNumVivienda() != null) {
				getNumViviendaDispAux = viviendareservada
						.getViviendaDisponible().getNumVivienda();
			}
			// Importe PVP Total
			if (viviendareservada.getViviendaDisponible().getImcPvpTotal() != null) {
				getImportePVPTotalAux = viviendareservada
						.getViviendaDisponible().getImcPvpTotal();
			}
		}

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// IdM62r41t0
		params.add(viviendareservada.getIdM62r41t0());
		// Id Vivienda Solicitada
		params.add(getIdViviendaSolicAux);
		// Num. Vivienda
		params.add(getNumViviendaDispAux);
		// Importe PVP Total
		params.add(getImportePVPTotalAux);
		// Indicador Reserva Temporal
		params.add(BDUtils.formatIndicadorToBD(viviendareservada
				.getIndReservaTemp()));
		// Fecha Reserva
		params.add(viviendareservada.getFecReserva());
		// Fecha Fin Reserva
		params.add(viviendareservada.getFecFinReserva());
		// Observaciones
		params.add(viviendareservada.getTexObservaciones());
		// AbstractM62FBModel
		SQLGenericas.getParamsAbstractClass(params, viviendareservada);

		// Identificador Vivienda Reservada (PK)
		params.add(viviendareservada.getIdM62r43t0());

		// Realizar Update
		this.jdbcTemplate.update(query, params.toArray());

		return viviendareservada;
	}

	/**
	 * Actualiza el acuerdo al que pertenecen todas las viviendas reservadas de
	 * una solicitud
	 * 
	 * @param acuerdoVivienda
	 *            AcuerdoVivienda
	 */
	@Override
	public void updateAcuerdo(AcuerdoVivienda acuerdoVivienda) {
		String query = ViviendaReservadaDaoSql.SQL_UPDATE_ACUERDO_VIV;

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// IdM62r41t0
		params.add(acuerdoVivienda.getIdM62r41t0());
		// AbstractM62FBModel
		SQLGenericas.getParamsAbstractClass(params, acuerdoVivienda);
		// IdM62r38t0
		params.add(acuerdoVivienda.getSolicitudReserva() != null ? acuerdoVivienda
				.getSolicitudReserva().getIdM62r38t0() : null);

		// Realizar Update
		this.jdbcTemplate.update(query, params.toArray());
	}

	/**
	 * Finds a single row in the ViviendaReservada table.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @return ViviendaReservada
	 */
	@Override
	@Transactional(readOnly = true)
	public ViviendaReservada find(ViviendaReservada viviendareservada) {
		String query = ViviendaReservadaDaoSql.SQL_FIND;

		List<ViviendaReservada> viviendaReservadaList = this.jdbcTemplate
				.query(query, this.rwMap, viviendareservada.getIdM62r43t0());
		return DataAccessUtils.uniqueResult(viviendaReservadaList);
	}

	/**
	 * FindForUpdate
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @return ViviendaReservada
	 */
	@Override
	public ViviendaReservada findForUpdate(ViviendaReservada viviendareservada) {
		ViviendaReservada viviendaReservadaBD;
		try {
			viviendaReservadaBD = this.jdbcTemplate.queryForObject(
					ViviendaReservadaDaoSql.SQL_FIND_FOR_UPDATE, this.rwMap,
					viviendareservada.getIdM62r43t0(),
					viviendareservada.getFecHorUltimaModif());
		} catch (EmptyResultDataAccessException e) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		if (viviendaReservadaBD == null) {
			throw new UFException(IberErrorsMsgConstants.CONCURRENCY_MSG);
		}
		return viviendaReservadaBD;
	}

	/**
	 * Removes a single row in the ViviendaReservada table.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @return
	 */
	@Override
	public void remove(ViviendaReservada viviendareservada) {
		/*****
		 * Llamada a function que implementa llamada a plsql
		 ****/
		M62FuncionesPLSQL m62FuncionesPLSQL = new M62FuncionesPLSQL(
				this.jdbcTemplate.getDataSource());
		m62FuncionesPLSQL.borrarViviendaReservada(viviendareservada
				.getIdM62r43t0());
	}

	/**
	 * Finds a list of rows in the ViviendaReservada table.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ViviendaReservada>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ViviendaReservada> findAll(ViviendaReservada viviendareservada,
			JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_FIND_ALL);
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(viviendareservada);
		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/**
	 * Finds rows in the ViviendaReservada table using like.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ViviendaReservada>
	 */
	@Override
	@Transactional(readOnly = true)
	public List<ViviendaReservada> findAllLike(
			ViviendaReservada viviendareservada,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_FIND_ALL);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(viviendareservada,
				startsWith);
		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the ViviendaReservada table.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllCount(ViviendaReservada viviendareservada) {
		StringBuilder query = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(viviendareservada);
		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Counts rows in the ViviendaReservada table using like.
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllLikeCount(ViviendaReservada viviendareservada,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_FIND_ALL_COUNT);

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(viviendareservada,
				startsWith);
		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * Reorder the data list of ViviendaReservada selected for rup_table
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ViviendaReservada>>
	 */
	@Override
	public List<TableRowDto<ViviendaReservada>> reorderSelection(
			ViviendaReservada viviendareservada,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_FIND_ALL);

		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(viviendareservada,
				startsWith);
		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoSql.SQL_WHERE_ALL);
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(
				query, jqGridRequestDto, ViviendaReservada.class,
				filterParamList, ViviendaReservadaDaoSql.PK);

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<ViviendaReservada>(this.rwMapPK,
						jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Actualiza el estado del catalogo de viviendas una vez la vivienda
	 * reservada ha sido dada de alta.
	 * 
	 * @param viviendaReservada
	 *            ViviendaReservada
	 * @return ViviendaReservada
	 */
	@Override
	public ViviendaReservada updateDisponibilidadVivienda(
			ViviendaReservada viviendareservada,boolean bEstaDisponible) {
		String query = ViviendaReservadaDaoSql.SQL_UPDATE_CATALOGO;

		// Preparar Parametros
		List<Object> params = new ArrayList<Object>();
		// Codigo situacion
		params.add(bEstaDisponible?AppConstantes.SITUACION_D:AppConstantes.SITUACION_R);
		// Indicador Disponible
		params.add(bEstaDisponible?AppConstantes.TRUE:AppConstantes.FALSE);
		// AbstractM62FBModel
		SQLGenericas.getParamsAbstractClass(params, viviendareservada);
		// Num. Vivienda
		if (viviendareservada.getViviendaDisponible() != null) {
			params.add(viviendareservada.getViviendaDisponible()
					.getNumVivienda());
		}

		// Realizar Update
		this.jdbcTemplate.update(query, params.toArray());

		return viviendareservada;
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ViviendaReservada entity
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(ViviendaReservada viviendareservada) {

		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (viviendareservada != null) {
			if (viviendareservada.getIdM62r43t0() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"VIVIENDARES.ID_M62R43T0",
						viviendareservada.getIdM62r43t0());
			}
			if (viviendareservada.getIdM62r41t0() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"VIVIENDARES.ID_M62R41T0",
						viviendareservada.getIdM62r41t0());
			}
			if (viviendareservada.getViviendaSolicitada() != null
					&& viviendareservada.getViviendaSolicitada()
							.getIdM62r40t0() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"VIVIENDARES.ID_M62R40T0", viviendareservada
								.getViviendaSolicitada().getIdM62r40t0());
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the ViviendaReservada entity
	 * 
	 * @param viviendareservada
	 *            ViviendaReservada Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(
			ViviendaReservada viviendareservada, Boolean startsWith) {

		StringBuilder where = new StringBuilder(
				ViviendaReservadaDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (viviendareservada != null) {
			if (viviendareservada.getIdM62r43t0() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"VIVIENDARES.ID_M62R43T0",
						viviendareservada.getIdM62r43t0());
			}
			if (viviendareservada.getIdM62r41t0() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"VIVIENDARES.ID_M62R41T0",
						viviendareservada.getIdM62r41t0());
			}
			if (viviendareservada.getViviendaSolicitada() != null
					&& viviendareservada.getViviendaSolicitada()
							.getIdM62r40t0() != null) {
				BDUtils.crearFiltroIgual(where, params,
						"VIVIENDARES.ID_M62R40T0", viviendareservada
								.getViviendaSolicitada().getIdM62r40t0());
			}
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}

	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
