package com.ejie.m62fb.model;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.map.annotate.JsonDeserialize;
import org.codehaus.jackson.map.annotate.JsonSerialize;

import com.ejie.m62fb.utils.serializers.JsonBigDecimalDeserializer;
import com.ejie.m62fb.utils.serializers.JsonBigDecimalSerializer;
import com.ejie.m62fb.model.base.AbstractM62FBModel;
import com.ejie.m62fb.model.direccion.Direccion;
import com.ejie.m62fb.model.documento.Documento;
import com.ejie.m62fb.model.persona.Persona;
import com.ejie.m62fb.model.tipos.RegimenAcceso;
import com.ejie.m62fb.model.tipos.RegimenTenencia;
import com.ejie.m62fb.model.tipos.TipoAcceso;
import com.ejie.m62fb.model.tipos.TipoAdjudicacion;
import com.ejie.m62fb.model.tipos.TipoComoAdjudicacion;
import com.ejie.m62fb.model.tipos.TipoElemento;
import com.ejie.m62fb.model.tipos.TipoIniciativa;
import com.ejie.m62fb.model.tipos.TipoPorqueAdjudicacion;
import com.ejie.m62fb.model.tipos.TipoPromotor;
import com.ejie.m62fb.model.tipos.TipoPropietarios;
import com.ejie.m62fb.model.tipos.TipoQuienAdjudicacion;
import com.ejie.m62fb.model.tipos.TipoRegulacion;
import com.ejie.x38.serialization.JsonDateDeserializer;
import com.ejie.x38.serialization.JsonDateSerializer;

/**
 * M62r01t0- Table BD reference
 * 
XXXX
 */
public class Vivienda extends AbstractM62FBModel implements
		java.io.Serializable {

	private static final long serialVersionUID = 1L;
	private String rowId;
	private BigDecimal numVivienda;

	private Direccion direccion;

	private BigDecimal superficieUtil;
	private Long numDormitorios;
	private Long numBanyos;
	private String indVivAdaptada;
	private Date fecCalificacion;
	// Rango fecCalificacion
	private Date fecCalificacionDesde;
	private Date fecCalificacionHasta;

	private Date fecDescalificacion;
	// Rango fecDescalificacion
	private Date fecDescalificacionDesde;
	private Date fecDescalificacionHasta;

	private Long numPlazoCalificacion;

	private RegistroPropiedad registroPropiedad;

	private String texObservaciones;
	private String numFijoIbi;

	private RegimenAcceso regimenAcceso;

	private Long numOrganoGestor;
	private Date fecIniGestion;
	private Date fecFinGestion;
	private Date fecEnvOrgGestor;
	private String indLocalEnViv;

	private TipoElemento tipoElemento;

	private Integer idPromocion;
	private String descSituacionElemento;
	private String indUsoIndiv;
	private String indCalifPerman;
	private String indBajaDensidad;

	private String indVivConcer;
	private Date fecFinCalif;
	private String indVivUnifamiliar;
	private String codRefCatastral;
	private String indFamiliaNumer;
	private Date fecInscripDerSup;
	private Date fecEscrituraSuel;
	private String indAri;
	private RegimenTenencia regimenTenencia;

	// Accesos a la R09
	private TipoPorqueAdjudicacion tipoPorqueAdjudicacion;
	private TipoAdjudicacion tipoAdjudicacion;
	private TipoComoAdjudicacion tipoComoAdjudicacion;
	private TipoRegulacion tipoRegulacion;
	private TipoPromotor tipoPromotor;
	private TipoAcceso tipoAcceso;
	private TipoIniciativa tipoIniciativa;
	private TipoPropietarios tipoPropietarios;
	private TipoQuienAdjudicacion tipoQuienAdjudicacion;

	private InformacionSuelo informacionSuelo;
	// Boolean para saber si hay informacion del suelo (Detalle)
	private boolean existeInfoSuelo;

	// Propiedades para el detalle
	private Elemento garaje;
	private Elemento trastero;
	private String indEliminarReserva;

	// Propiedades para las busquedas (Tablas y filtros)
	private ExpedienteVivienda expedienteVivienda;
	private Persona persona;

	@JsonIgnore
	private final List<ExpedienteVivienda> expedientesViviendaList = new ArrayList<ExpedienteVivienda>();
	@JsonIgnore
	private final List<Documento> documentosList = new ArrayList<Documento>();
	@JsonIgnore
	private final List<ContratoVivienda> m62r03t0sList = new ArrayList<ContratoVivienda>();
	@JsonIgnore
	private final List<ContratoAlquiler> m62r05t0sList = new ArrayList<ContratoAlquiler>();

	/**
	 * Method 'Vivienda'.
	 */
	public Vivienda() {
	}

	/**
	 * Method 'Vivienda'.
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 */
	public Vivienda(BigDecimal numVivienda) {
		this.numVivienda = numVivienda;
	}

	/**
	 * Method 'getNumVivienda'
	 * 
	 * @return numVivienda
	 */
	public BigDecimal getNumVivienda() {
		return this.numVivienda;
	}

	/**
	 * Method 'setNumVivienda'
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 */
	public void setNumVivienda(BigDecimal numVivienda) {
		this.numVivienda = numVivienda;
	}

	/**
	 * Method 'getDireccion'
	 * 
	 * @return direccion
	 */
	public Direccion getDireccion() {
		return this.direccion;
	}

	/**
	 * Method 'setDireccion'
	 * 
	 * @param direccion
	 *            Direccion
	 */
	public void setDireccion(Direccion direccion) {
		this.direccion = direccion;
	}

	/**
	 * Method 'getSuperficieUtil'
	 * 
	 * @return superficieUtil
	 */
	@JsonSerialize(using = JsonBigDecimalSerializer.class)
	public BigDecimal getSuperficieUtil() {
		return this.superficieUtil;
	}

	/**
	 * Method 'setSuperficieUtil'
	 * 
	 * @param superficieUtil
	 *            BigDecimal
	 */
	@JsonDeserialize(using = JsonBigDecimalDeserializer.class)
	public void setSuperficieUtil(BigDecimal superficieUtil) {
		this.superficieUtil = superficieUtil;
	}

	/**
	 * Method 'getNumDormitorios'
	 * 
	 * @return numDormitorios
	 */
	public Long getNumDormitorios() {
		return this.numDormitorios;
	}

	/**
	 * Method 'setNumDormitorios'
	 * 
	 * @param numDormitorios
	 *            Long
	 */
	public void setNumDormitorios(Long numDormitorios) {
		this.numDormitorios = numDormitorios;
	}

	/**
	 * Method 'getNumBanyos'
	 * 
	 * @return numBanyos
	 */
	public Long getNumBanyos() {
		return this.numBanyos;
	}

	/**
	 * Method 'setNumBanyos'
	 * 
	 * @param numBanyos
	 *            Long
	 */
	public void setNumBanyos(Long numBanyos) {
		this.numBanyos = numBanyos;
	}

	/**
	 * Method 'getIndVivAdaptada'
	 * 
	 * @return indVivAdaptada
	 */
	public String getIndVivAdaptada() {
		return this.indVivAdaptada;
	}

	/**
	 * Method 'setIndVivAdaptada'
	 * 
	 * @param indVivAdaptada
	 *            String
	 */
	public void setIndVivAdaptada(String indVivAdaptada) {
		this.indVivAdaptada = indVivAdaptada;
	}

	/**
	 * Method 'getFecCalificacion'
	 * 
	 * @return fecCalificacion
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecCalificacion() {
		return this.fecCalificacion;
	}

	/**
	 * Method 'setFecCalificacion'
	 * 
	 * @param fecCalificacion
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecCalificacion(Date fecCalificacion) {
		this.fecCalificacion = fecCalificacion;
	}

	/**
	 * Method 'getFecDescalificacion'
	 * 
	 * @return fecDescalificacion
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecDescalificacion() {
		return this.fecDescalificacion;
	}

	/**
	 * Method 'setFecDescalificacion'
	 * 
	 * @param fecDescalificacion
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecDescalificacion(Date fecDescalificacion) {
		this.fecDescalificacion = fecDescalificacion;
	}

	/**
	 * Method 'getNumPlazoCalificacion'
	 * 
	 * @return numPlazoCalificacion
	 */
	public Long getNumPlazoCalificacion() {
		return this.numPlazoCalificacion;
	}

	/**
	 * Method 'setNumPlazoCalificacion'
	 * 
	 * @param numPlazoCalificacion
	 *            Long
	 */
	public void setNumPlazoCalificacion(Long numPlazoCalificacion) {
		this.numPlazoCalificacion = numPlazoCalificacion;
	}

	/**
	 * Method 'getRegistroPropiedad'
	 * 
	 * @return registroPropiedad
	 */
	public RegistroPropiedad getRegistroPropiedad() {
		return this.registroPropiedad;
	}

	/**
	 * Method 'setRegistroPropiedad'
	 * 
	 * @param registroPropiedad
	 *            RegistroPropiedad
	 */
	public void setRegistroPropiedad(RegistroPropiedad registroPropiedad) {
		this.registroPropiedad = registroPropiedad;
	}

	/**
	 * Method 'getTexObservaciones'
	 * 
	 * @return texObservaciones
	 */
	public String getTexObservaciones() {
		return this.texObservaciones;
	}

	/**
	 * Method 'setTexObservaciones'
	 * 
	 * @param texObservaciones
	 *            String
	 */
	public void setTexObservaciones(String texObservaciones) {
		this.texObservaciones = texObservaciones;
	}

	/**
	 * Method 'getNumFijoIbi'
	 * 
	 * @return numFijoIbi
	 */
	public String getNumFijoIbi() {
		return this.numFijoIbi;
	}

	/**
	 * Method 'setNumFijoIbi'
	 * 
	 * @param numFijoIbi
	 *            String
	 */
	public void setNumFijoIbi(String numFijoIbi) {
		this.numFijoIbi = numFijoIbi;
	}

	/**
	 * Method 'getRegimenAcceso'
	 * 
	 * @return regimenAcceso
	 */
	public RegimenAcceso getRegimenAcceso() {
		return this.regimenAcceso;
	}

	/**
	 * Method 'setRegimenAcceso'
	 * 
	 * @param regimenAcceso
	 *            RegimenAcceso
	 */
	public void setRegimenAcceso(RegimenAcceso regimenAcceso) {
		this.regimenAcceso = regimenAcceso;
	}

	/**
	 * Method 'getNumOrganoGestor'
	 * 
	 * @return numOrganoGestor
	 */
	public Long getNumOrganoGestor() {
		return this.numOrganoGestor;
	}

	/**
	 * Method 'setNumOrganoGestor'
	 * 
	 * @param numOrganoGestor
	 *            Long
	 */
	public void setNumOrganoGestor(Long numOrganoGestor) {
		this.numOrganoGestor = numOrganoGestor;
	}

	/**
	 * Method 'getFecIniGestion'
	 * 
	 * @return fecIniGestion
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecIniGestion() {
		return this.fecIniGestion;
	}

	/**
	 * Method 'setFecIniGestion'
	 * 
	 * @param fecIniGestion
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecIniGestion(Date fecIniGestion) {
		this.fecIniGestion = fecIniGestion;
	}

	/**
	 * Method 'getFecFinGestion'
	 * 
	 * @return fecFinGestion
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecFinGestion() {
		return this.fecFinGestion;
	}

	/**
	 * Method 'setFecFinGestion'
	 * 
	 * @param fecFinGestion
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecFinGestion(Date fecFinGestion) {
		this.fecFinGestion = fecFinGestion;
	}

	/**
	 * Method 'getFecEnvOrgGestor'
	 * 
	 * @return fecEnvOrgGestor
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecEnvOrgGestor() {
		return this.fecEnvOrgGestor;
	}

	/**
	 * Method 'setFecEnvOrgGestor'
	 * 
	 * @param fecEnvOrgGestor
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecEnvOrgGestor(Date fecEnvOrgGestor) {
		this.fecEnvOrgGestor = fecEnvOrgGestor;
	}

	/**
	 * Method 'getIndLocalEnViv'
	 * 
	 * @return indLocalEnViv
	 */
	public String getIndLocalEnViv() {
		return this.indLocalEnViv;
	}

	/**
	 * Method 'setIndLocalEnViv'
	 * 
	 * @param indLocalEnViv
	 *            String
	 */
	public void setIndLocalEnViv(String indLocalEnViv) {
		this.indLocalEnViv = indLocalEnViv;
	}

	/**
	 * Method 'getTipoElemento'
	 * 
	 * @return tipoElemento
	 */
	public TipoElemento getTipoElemento() {
		return this.tipoElemento;
	}

	/**
	 * Method 'setTipoElemento'
	 * 
	 * @param tipoElemento
	 *            TipoElemento
	 */
	public void setTipoElemento(TipoElemento tipoElemento) {
		this.tipoElemento = tipoElemento;
	}

	/**
	 * Method 'getIdPromocion'
	 * 
	 * @return idPromocion
	 */
	public Integer getIdPromocion() {
		return this.idPromocion;
	}

	/**
	 * Method 'setIdPromocion'
	 * 
	 * @param idPromocion
	 *            Integer
	 */
	public void setIdPromocion(Integer idPromocion) {
		this.idPromocion = idPromocion;
	}

	/**
	 * Method 'getDescSituacionElemento'
	 * 
	 * @return descSituacionElemento
	 */
	public String getDescSituacionElemento() {
		return this.descSituacionElemento;
	}

	/**
	 * Method 'setDescSituacionElemento'
	 * 
	 * @param descSituacionElemento
	 *            String
	 */
	public void setDescSituacionElemento(String descSituacionElemento) {
		this.descSituacionElemento = descSituacionElemento;
	}

	/**
	 * Method 'getIndUsoIndiv'
	 * 
	 * @return indUsoIndiv
	 */
	public String getIndUsoIndiv() {
		return this.indUsoIndiv;
	}

	/**
	 * Method 'setIndUsoIndiv'
	 * 
	 * @param indUsoIndiv
	 *            String
	 */
	public void setIndUsoIndiv(String indUsoIndiv) {
		this.indUsoIndiv = indUsoIndiv;
	}

	/**
	 * Method 'getIndCalifPerman'
	 * 
	 * @return indCalifPerman
	 */
	public String getIndCalifPerman() {
		return this.indCalifPerman;
	}

	/**
	 * Method 'setIndCalifPerman'
	 * 
	 * @param indCalifPerman
	 *            String
	 */
	public void setIndCalifPerman(String indCalifPerman) {
		this.indCalifPerman = indCalifPerman;
	}

	/**
	 * Method 'getIndBajaDensidad'
	 * 
	 * @return indBajaDensidad
	 */
	public String getIndBajaDensidad() {
		return this.indBajaDensidad;
	}

	/**
	 * Method 'setIndBajaDensidad'
	 * 
	 * @param indBajaDensidad
	 *            String
	 */
	public void setIndBajaDensidad(String indBajaDensidad) {
		this.indBajaDensidad = indBajaDensidad;
	}

	/**
	 * Method 'getIndVivConcer'
	 * 
	 * @return indVivConcer
	 */
	public String getIndVivConcer() {
		return this.indVivConcer;
	}

	/**
	 * Method 'setIndVivConcer'
	 * 
	 * @param indVivConcer
	 *            String
	 */
	public void setIndVivConcer(String indVivConcer) {
		this.indVivConcer = indVivConcer;
	}

	/**
	 * Method 'getFecFinCalif'
	 * 
	 * @return fecFinCalif
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecFinCalif() {
		return this.fecFinCalif;
	}

	/**
	 * Method 'setFecFinCalif'
	 * 
	 * @param fecFinCalif
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecFinCalif(Date fecFinCalif) {
		this.fecFinCalif = fecFinCalif;
	}

	/**
	 * Method 'getIndVivUnifamiliar'
	 * 
	 * @return indVivUnifamiliar
	 */
	public String getIndVivUnifamiliar() {
		return this.indVivUnifamiliar;
	}

	/**
	 * Method 'setIndVivUnifamiliar'
	 * 
	 * @param indVivUnifamiliar
	 *            String
	 */
	public void setIndVivUnifamiliar(String indVivUnifamiliar) {
		this.indVivUnifamiliar = indVivUnifamiliar;
	}

	/**
	 * Method 'getCodRefCatastral'
	 * 
	 * @return codRefCatastral
	 */
	public String getCodRefCatastral() {
		return this.codRefCatastral;
	}

	/**
	 * Method 'setCodRefCatastral'
	 * 
	 * @param codRefCatastral
	 *            String
	 */
	public void setCodRefCatastral(String codRefCatastral) {
		this.codRefCatastral = codRefCatastral;
	}

	/**
	 * Method 'getIndFamiliaNumer'
	 * 
	 * @return indFamiliaNumer
	 */
	public String getIndFamiliaNumer() {
		return this.indFamiliaNumer;
	}

	/**
	 * Method 'setIndFamiliaNumer'
	 * 
	 * @param indFamiliaNumer
	 *            String
	 */
	public void setIndFamiliaNumer(String indFamiliaNumer) {
		this.indFamiliaNumer = indFamiliaNumer;
	}

	/**
	 * Method 'getFecInscripDerSup'
	 * 
	 * @return fecInscripDerSup
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecInscripDerSup() {
		return this.fecInscripDerSup;
	}

	/**
	 * Method 'setFecInscripDerSup'
	 * 
	 * @param fecInscripDerSup
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecInscripDerSup(Date fecInscripDerSup) {
		this.fecInscripDerSup = fecInscripDerSup;
	}

	/**
	 * Method 'getFecEscrituraSuel'
	 * 
	 * @return fecEscrituraSuel
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecEscrituraSuel() {
		return this.fecEscrituraSuel;
	}

	/**
	 * Method 'setFecEscrituraSuel'
	 * 
	 * @param fecEscrituraSuel
	 *            Date
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecEscrituraSuel(Date fecEscrituraSuel) {
		this.fecEscrituraSuel = fecEscrituraSuel;
	}

	/**
	 * Method 'getIndAri'
	 * 
	 * @return indAri
	 */
	public String getIndAri() {
		return this.indAri;
	}

	/**
	 * Method 'setIndAri'
	 * 
	 * @param indAri
	 *            String
	 */
	public void setIndAri(String indAri) {
		this.indAri = indAri;
	}

	/**
	 * Method 'getRegimenTenencia'
	 * 
	 * @return regimenTenencia
	 */
	public RegimenTenencia getRegimenTenencia() {
		return this.regimenTenencia;
	}

	/**
	 * Method 'setRegimenTenencia'
	 * 
	 * @param regimenTenencia
	 *            RegimenTenencia
	 */
	public void setRegimenTenencia(RegimenTenencia regimenTenencia) {
		this.regimenTenencia = regimenTenencia;
	}

	/**
	 * Method 'getTipoPorqueAdjudicacion'
	 * 
	 * @return tipoPorqueAdjudicacion
	 */
	public TipoPorqueAdjudicacion getTipoPorqueAdjudicacion() {
		return this.tipoPorqueAdjudicacion;
	}

	/**
	 * Method 'setTipoPorqueAdjudicacion'
	 * 
	 * @param tipoPorqueAdjudicacion
	 *            TipoPorqueAdjudicacion
	 */
	public void setTipoPorqueAdjudicacion(
			TipoPorqueAdjudicacion tipoPorqueAdjudicacion) {
		this.tipoPorqueAdjudicacion = tipoPorqueAdjudicacion;
	}

	/**
	 * Method 'getTipoAdjudicacion'
	 * 
	 * @return tipoAdjudicacion
	 */
	public TipoAdjudicacion getTipoAdjudicacion() {
		return this.tipoAdjudicacion;
	}

	/**
	 * Method 'setTipoAdjudicacion'
	 * 
	 * @param tipoAdjudicacion
	 *            TipoAdjudicacion
	 */
	public void setTipoAdjudicacion(TipoAdjudicacion tipoAdjudicacion) {
		this.tipoAdjudicacion = tipoAdjudicacion;
	}

	/**
	 * Method 'getTipoComoAdjudicacion'
	 * 
	 * @return tipoComoAdjudicacion
	 */
	public TipoComoAdjudicacion getTipoComoAdjudicacion() {
		return this.tipoComoAdjudicacion;
	}

	/**
	 * Method 'setTipoComoAdjudicacion'
	 * 
	 * @param tipoComoAdjudicacion
	 *            TipoComoAdjudicacion
	 */
	public void setTipoComoAdjudicacion(
			TipoComoAdjudicacion tipoComoAdjudicacion) {
		this.tipoComoAdjudicacion = tipoComoAdjudicacion;
	}

	/**
	 * Method 'getTipoRegulacion'
	 * 
	 * @return tipoRegulacion
	 */
	public TipoRegulacion getTipoRegulacion() {
		return this.tipoRegulacion;
	}

	/**
	 * Method 'setTipoRegulacion'
	 * 
	 * @param tipoRegulacion
	 *            TipoRegulacion
	 */
	public void setTipoRegulacion(TipoRegulacion tipoRegulacion) {
		this.tipoRegulacion = tipoRegulacion;
	}

	/**
	 * Method 'getTipoPromotor'
	 * 
	 * @return tipoPromotor
	 */
	public TipoPromotor getTipoPromotor() {
		return this.tipoPromotor;
	}

	/**
	 * Method 'setTipoPromotor'
	 * 
	 * @param tipoPromotor
	 *            TipoPromotor
	 */
	public void setTipoPromotor(TipoPromotor tipoPromotor) {
		this.tipoPromotor = tipoPromotor;
	}

	/**
	 * Method 'getTipoAcceso'
	 * 
	 * @return tipoAcceso
	 */
	public TipoAcceso getTipoAcceso() {
		return this.tipoAcceso;
	}

	/**
	 * Method 'setTipoAcceso'
	 * 
	 * @param tipoAcceso
	 *            TipoAcceso
	 */
	public void setTipoAcceso(TipoAcceso tipoAcceso) {
		this.tipoAcceso = tipoAcceso;
	}

	/**
	 * Method 'getTipoIniciativa'
	 * 
	 * @return tipoIniciativa
	 */
	public TipoIniciativa getTipoIniciativa() {
		return this.tipoIniciativa;
	}

	/**
	 * Method 'setTipoIniciativa'
	 * 
	 * @param tipoIniciativa
	 *            TipoIniciativa
	 */
	public void setTipoIniciativa(TipoIniciativa tipoIniciativa) {
		this.tipoIniciativa = tipoIniciativa;
	}

	/**
	 * Method 'getTipoPropietarios'
	 * 
	 * @return tipoPropietarios
	 */
	public TipoPropietarios getTipoPropietarios() {
		return this.tipoPropietarios;
	}

	/**
	 * Method 'setTipoPropietarios'
	 * 
	 * @param tipoPropietarios
	 *            TipoPropietarios
	 */
	public void setTipoPropietarios(TipoPropietarios tipoPropietarios) {
		this.tipoPropietarios = tipoPropietarios;
	}

	/**
	 * Method 'getTipoQuienAdjudicacion'
	 * 
	 * @return tipoQuienAdjudicacion
	 */
	public TipoQuienAdjudicacion getTipoQuienAdjudicacion() {
		return this.tipoQuienAdjudicacion;
	}

	/**
	 * Method 'setTipoQuienAdjudicacion'
	 * 
	 * @param tipoQuienAdjudicacion
	 *            TipoQuienAdjudicacion
	 */
	public void setTipoQuienAdjudicacion(
			TipoQuienAdjudicacion tipoQuienAdjudicacion) {
		this.tipoQuienAdjudicacion = tipoQuienAdjudicacion;
	}

	/**
	 * Method 'getInformacionSuelo'
	 * 
	 * @return informacionSuelo
	 */
	public InformacionSuelo getInformacionSuelo() {
		return this.informacionSuelo;
	}

	/**
	 * Method 'setInformacionSuelo'
	 * 
	 * @param informacionSuelo
	 *            InformacionSuelo
	 */
	public void setInformacionSuelo(InformacionSuelo informacionSuelo) {
		this.informacionSuelo = informacionSuelo;
	}

	/**
	 * Method 'isExisteInfoSuelo'
	 * 
	 * @return existeInfoSuelo
	 */
	public boolean isExisteInfoSuelo() {
		return this.existeInfoSuelo;
	}

	/**
	 * Method 'setExisteInfoSuelo'
	 * 
	 * @param existeInfoSuelo
	 *            boolean
	 */
	public void setExisteInfoSuelo(boolean existeInfoSuelo) {
		this.existeInfoSuelo = existeInfoSuelo;
	}

	/**
	 * Method 'getGaraje'
	 * 
	 * @return garaje
	 */
	public Elemento getGaraje() {
		return this.garaje;
	}

	/**
	 * Method 'setGaraje'
	 * 
	 * @param garaje
	 *            Elemento
	 */
	public void setGaraje(Elemento garaje) {
		this.garaje = garaje;
	}

	/**
	 * Method 'getTrastero'
	 * 
	 * @return trastero
	 */
	public Elemento getTrastero() {
		return this.trastero;
	}

	/**
	 * Method 'setTrastero'
	 * 
	 * @param trastero
	 *            Elemento
	 */
	public void setTrastero(Elemento trastero) {
		this.trastero = trastero;
	}

	/**
	 * Method 'getIndEliminarReserva'
	 * 
	 * @return indEliminarReserva
	 */
	public String getIndEliminarReserva() {
		return this.indEliminarReserva;
	}

	/**
	 * Method 'setIndEliminarReserva'
	 * 
	 * @param indEliminarReserva
	 *            String
	 */
	public void setIndEliminarReserva(String indEliminarReserva) {
		this.indEliminarReserva = indEliminarReserva;
	}

	/**
	 * Method 'getExpedienteVivienda'
	 * 
	 * @return expedienteVivienda
	 */
	public ExpedienteVivienda getExpedienteVivienda() {
		return this.expedienteVivienda;
	}

	/**
	 * Method 'setExpedienteVivienda'
	 * 
	 * @param expedienteVivienda
	 *            ExpedienteVivienda
	 */
	public void setExpedienteVivienda(ExpedienteVivienda expedienteVivienda) {
		this.expedienteVivienda = expedienteVivienda;
	}

	/**
	 * Method 'getPersona'
	 * 
	 * @return persona
	 */
	public Persona getPersona() {
		return this.persona;
	}

	/**
	 * Method 'setPersona'
	 * 
	 * @param persona
	 *            Persona
	 */
	public void setPersona(Persona persona) {
		this.persona = persona;
	}

	/**
	 * Method 'getExpedientesViviendaList'
	 * 
	 * @return expedientesViviendaList
	 */
	public List<ExpedienteVivienda> getExpedientesViviendaList() {
		return this.expedientesViviendaList;
	}

	/**
	 * Method 'getDocumentosList'
	 * 
	 * @return documentosList
	 */
	public List<Documento> getDocumentosList() {
		return this.documentosList;
	}

	/**
	 * Method 'getM62r03t0sList'
	 * 
	 * @return m62r03t0sList
	 */
	public List<ContratoVivienda> getM62r03t0sList() {
		return this.m62r03t0sList;
	}

	/**
	 * Method 'getM62r05t0sList'
	 * 
	 * @return m62r05t0sList
	 */
	public List<ContratoAlquiler> getM62r05t0sList() {
		return this.m62r05t0sList;
	}

	/**
	 * @return the fecCalificacionDesde
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecCalificacionDesde() {
		return fecCalificacionDesde;
	}

	/**
	 * @param fecCalificacionDesde
	 *            the fecCalificacionDesde to set
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecCalificacionDesde(Date fecCalificacionDesde) {
		this.fecCalificacionDesde = fecCalificacionDesde;
	}

	/**
	 * @return the fecCalificacionHasta
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecCalificacionHasta() {
		return fecCalificacionHasta;
	}

	/**
	 * @param fecCalificacionHasta
	 *            the fecCalificacionHasta to set
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecCalificacionHasta(Date fecCalificacionHasta) {
		this.fecCalificacionHasta = fecCalificacionHasta;
	}

	/**
	 * @return the fecDescalificacionDesde
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecDescalificacionDesde() {
		return fecDescalificacionDesde;
	}

	/**
	 * @param fecDescalificacionDesde
	 *            the fecDescalificacionDesde to set
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecDescalificacionDesde(Date fecDescalificacionDesde) {
		this.fecDescalificacionDesde = fecDescalificacionDesde;
	}

	/**
	 * @return the fecDescalificacionHasta
	 */
	@JsonSerialize(using = JsonDateSerializer.class)
	public Date getFecDescalificacionHasta() {
		return fecDescalificacionHasta;
	}

	/**
	 * @param fecDescalificacionHasta
	 *            the fecDescalificacionHasta to set
	 */
	@JsonDeserialize(using = JsonDateDeserializer.class)
	public void setFecDescalificacionHasta(Date fecDescalificacionHasta) {
		this.fecDescalificacionHasta = fecDescalificacionHasta;
	}

	/**
	 * @param rowId the rowId to set
	 */
	public void setRowId(String rowId) {
		this.rowId = rowId;
	}

	/**
	 * @return the rowId
	 */
	public String getRowId() {
		return rowId;
	}

}