package com.ejie.m62fb.service.registroVPP;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ResponseBody;

import com.ejie.m62fb.utils.constants.IberErrorsMsgConstants;
import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.dao.registroVPP.PropietarioDao;
import com.ejie.m62fb.model.Propietario;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * PropietarioServiceImpl generated by UDA, 23-sep-2015 11:53:47.
 * 
XXXX
 */

@Service(value = "propietarioService")
public class PropietarioServiceImpl implements PropietarioService {

	@Autowired
	private PropietarioDao propietarioDao;

	/**
	 * Inserts a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
	@Transactional(rollbackFor = Throwable.class)
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','create')")
	public Propietario add(Propietario propietario) {
		Propietario propietarioBBDD = null;
		// 1. El titular no debe estar dado de alta en el mismo contrato en
		// Etxelaguntza.
		propietarioBBDD = this.find(propietario);
		if (propietarioBBDD != null) {
			throw new UFException(IberErrorsMsgConstants.EXISTE_PROPIETARIO);
		} else {
			// 2. Calculamos la suma del porcentaje de titularidad teniendo en
			// cuenta el % de titularidad que se esta modificando. Si da una
			// cifra superior a 100, la funcion devuelve true y se lanza el
			// error. Si no, seguimos con las operaciones necesarias.
			boolean porcentTitularidadMayor = this.propietarioDao
					.findPorcentTitularidad(propietario);
			if (porcentTitularidadMayor) {
				// Mensaje error
				throw new UFException(
						IberErrorsMsgConstants.PORCENT_TITULARIDAD_ERROR);
			} else if (propietario.getIndPrimerSolic() != null
					&& propietario.getIndPrimerSolic().getCodTabla()
							.equals("S")) {
				// 3. Si esta seleccionado el check de primer solicitante,
				// validamos tambien que no haya otro titular en el contrato con
				// IND_PRIMER_SOLIC='S'.
				boolean existePrimerSolic = this.propietarioDao
						.findTitularPrimerSolic(propietario);
				if (existePrimerSolic) {
					// Mensaje error
					throw new UFException(
							IberErrorsMsgConstants.EXISTE_PRIMER_SOLIC);
				} else {
					propietarioBBDD = this.propietarioDao.add(propietario);
				}
			} else {
				propietarioBBDD = this.propietarioDao.add(propietario);
			}
		}

		return propietarioBBDD;
	}

	/**
	 * Updates a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
	@Transactional(rollbackFor = Throwable.class)
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','write')")
	public Propietario update(Propietario propietario) {
		boolean modificar = false;
		// find for update
		Propietario propietarioBD = this.propietarioDao
				.findForUpdate(propietario);
		// actualizar datos modificables
		propietarioBD.setPersona(propietario.getPersona());
		propietarioBD.getPersona().setDireccion(
				propietario.getPersona().getDireccion());
		propietarioBD.setIndVivAdaptada(propietario.getIndVivAdaptada());
		propietarioBD.setPorTitularidad(propietario.getPorTitularidad());
		propietarioBD.setIndPrimerSolic(propietario.getIndPrimerSolic());

		Propietario propietarioBBDD = null;
		// Calculamos la suma del porcentaje de titularidad teniendo en cuenta
		// el % de titularidad que se esta modificando. Si da una cifra superior
		// a 100, la funcion devuelve true y se lanza el error. Si no, seguimos
		// con las operaciones necesarias.
		boolean porcentTitularidadMayor = this.propietarioDao
				.findPorcentTitularidad(propietarioBD);
		if (porcentTitularidadMayor) {
			// Mensaje error
			throw new UFException(
					IberErrorsMsgConstants.PORCENT_TITULARIDAD_ERROR);
		} else if (propietarioBD.getIndPrimerSolic() != null
				&& propietarioBD.getIndPrimerSolic().getCodTabla().equals("S")) {
			// Si esta seleccionado el check de primer solicitante, validamos
			// tambien que no haya otro titular en el contrato con
			// IND_PRIMER_SOLIC='S'.
			boolean existePrimerSolic = this.propietarioDao
					.findTitularPrimerSolic(propietarioBD);
			if (existePrimerSolic) {
				// Mensaje error
				throw new UFException(
						IberErrorsMsgConstants.EXISTE_PRIMER_SOLIC);
			} else {
				modificar = true;
			}
		} else {
			modificar = true;
		}
		// Modificamos registro
		if (modificar) {
			propietarioBBDD = this.propietarioDao.update(propietarioBD);
		}
		return propietarioBBDD;
	}

	/**
	 * Finds a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @return Propietario
	 */
	@Override
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','read')")
	public Propietario find(Propietario propietario) {
		return this.propietarioDao.find(propietario);
	}

	/**
	 * Deletes a single row in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 */
	@Override
	@Transactional(rollbackFor = Throwable.class)
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','delete')")
	public void remove(Propietario propietario) {
		this.propietarioDao.remove(propietario);
	}

	/**
	 * Finds a list of rows in the Propietario table.
	 * 
	 * @param propietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Propietario>
	 */
	@Override
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','read')")
	public List<Propietario> findAll(Propietario propietario,
			JQGridRequestDto jqGridRequestDto) {
		return this.propietarioDao.findAll(propietario, jqGridRequestDto);
	}

	/**
	 * Finds rows in the Propietario table using like.
	 * 
	 * @param propietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Propietario>
	 */
	@Override
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','read')")
	public List<Propietario> findAllLike(Propietario propietario,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.propietarioDao.findAllLike(propietario, jqGridRequestDto,
				startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */
	/**
	 * Filter method in the Propietario table.
	 * 
	 * @param filterPropietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Propietario>
	 */
	@Override
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','read')")
	public JQGridResponseDto<Propietario> filter(Propietario filterPropietario,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<Propietario> listaPropietario = this.propietarioDao.findAllLike(
				filterPropietario, jqGridRequestDto, false);
		Long recordNum = this.propietarioDao.findAllLikeCount(
				filterPropietario != null ? filterPropietario
						: new Propietario(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<Propietario>> reorderSelection = this.propietarioDao
					.reorderSelection(filterPropietario, jqGridRequestDto,
							startsWith);
			return new JQGridResponseDto<Propietario>(jqGridRequestDto,
					recordNum, listaPropietario, reorderSelection);
		}
		return new JQGridResponseDto<Propietario>(jqGridRequestDto, recordNum,
				listaPropietario);
	}

	/**
	 * Reorder the selection made in Propietario table.
	 * 
	 * @param filterPropietario
	 *            Propietario
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	@Override
@PreAuthorize("hasPermission('PROPIETARIO_SERVICE','SecureService','read')")
	public Object reorderSelection(Propietario filterPropietario,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.propietarioDao.reorderSelection(filterPropietario,
				jqGridRequestDto, startsWith);
	}

	/**
	 * Funcion que consulta si el titular esta en otro contrato de otra
	 * vivienda.
	 * 
	 * @param idVivienda
	 *            Integer
	 * @param idPersona
	 *            Integer
	 * @return Integer
	 */
	@Override
	@Transactional(rollbackFor = Throwable.class)
	public @ResponseBody
	boolean findTitularesContratos(Integer idVivienda, Integer idPersona) {
		return this.propietarioDao
				.findTitularesContratos(idVivienda, idPersona);
	}
}
