package com.ejie.t17i.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.t17i.filters.LocalidadFilter;
import com.ejie.t17i.model.Localidad;
import com.ejie.t17i.model.Municipio;
import com.ejie.t17i.model.Provincia;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;

/**
 * <p>
 * Implementaci&oacute;n de servicios DAO para acceder a la tabla LOCALIDAD de
 * NORA
 * </p>
 * 
XXXX
 * @see com.ejie.t17i.model.Localidad
 * @see com.ejie.t17i.model.Municipio
 * @see com.ejie.t17i.model.Provincia
 */
@Repository
@Transactional
public class LocalidadDaoImpl implements LocalidadDao {
	private JdbcTemplate jdbcTemplate;
	@Autowired
	private MunicipioDao municipioDao;

	private final RowMapper<Localidad> rwMap = new RowMapper<Localidad>() {
		@Override
		public Localidad mapRow(ResultSet resultSet, int rowNum)
				throws SQLException {
			return new Localidad(resultSet.getBigDecimal("ID"),
					resultSet.getString("NOMBRE"),
					resultSet.getLong("ENTIDADID"),
					municipioDao.find(new Municipio(resultSet
							.getString("MUNICIPIOID"), new Provincia(resultSet
							.getString("PROVINCIAID")))));
		}
	};

	/**
	 * M&eacute;todo que asigna el DataSource a usar para el acceso a BBDD.
	 * 
	 * @param dataSource
	 *            El DataSource a usar.
	 */
	@Resource
	public void setDataSource(final DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@Transactional(readOnly = true)
	public Localidad find(final Localidad localidad) {
		List<Localidad> localidadList = this.jdbcTemplate.query(
				LocalidadDaoSql.SQL_FIND, this.rwMap, localidad.getId());
		return DataAccessUtils.uniqueResult(localidadList);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@Transactional(readOnly = true)
	public List<Localidad> findAll(final Localidad localidad,
			final JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				LocalidadDaoSql.SQL_FIND_ALL.length())
				.append(LocalidadDaoSql.SQL_FIND_ALL);
		final List<Object> params = new ArrayList<Object>();

		LocalidadFilter filtro = new LocalidadFilter(localidad, params, query,
				null);
		filtro.generarWhereBE();
		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		} else {
			query.append(LocalidadDaoSql.ORDER);
		}

		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@Transactional(readOnly = true)
	public Long findAllCount(final Localidad localidad) {

		final StringBuilder query = new StringBuilder(
				LocalidadDaoSql.SQL_FIND_ALL_COUNT.length())
				.append(LocalidadDaoSql.SQL_FIND_ALL_COUNT);
		final List<Object> params = new ArrayList<Object>();

		LocalidadFilter filtro = new LocalidadFilter(localidad, params, query,
				null);
		filtro.generarWhereBE();

		return this.jdbcTemplate.queryForLong(query.toString(),
				params.toArray());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	@Transactional(readOnly = true)
	public List<Localidad> findAllLike(final Localidad localidad,
			final JQGridRequestDto jqGridRequestDto, final Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				LocalidadDaoSql.SQL_FIND_ALL.length())
				.append(LocalidadDaoSql.SQL_FIND_ALL);
		final List<Object> params = new ArrayList<Object>();

		LocalidadFilter filtro = new LocalidadFilter(localidad, params, query,
				new com.ejie.m62fb.utils.sql.LikeInWhere(true, startsWith));
		filtro.generarWhereBE();

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query);
		} else {
			query.append(LocalidadDaoSql.ORDER);
		}
		return this.jdbcTemplate.query(query.toString(), this.rwMap,
				params.toArray());
	}
}
