package com.ejie.m62fb.control.registroVPP;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.control.PersonaController;
import com.ejie.m62fb.model.AlojamientoDotacional;
import com.ejie.m62fb.model.tipos.R09Tipos;
import com.ejie.m62fb.service.registroVPP.AlojamientoDotacionalService;
import com.ejie.m62fb.util.UtilSeguridad;
import com.ejie.m62fb.utils.constants.AppConstantes;
import com.ejie.m62fb.utils.properties.UtilProperties;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.reports.ReportData;

/**
 * AlojamientoDotacionalController.
 * 
XXXX
 */
@Controller
@RequestMapping(value = "/alojamientoDotacional/seleccion")
public class AlojamientoDotacionalController {

	private static final Logger logger = LoggerFactory
			.getLogger(AlojamientoDotacionalController.class);

	@Autowired
	private AlojamientoDotacionalService alojamientoDotacionalService;

	@Autowired
	private PersonaController personaController;

	@Autowired
	@SuppressWarnings("rawtypes")
	private ComboController comboController;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idAlojReg
	 *            Integer
	 * @return AlojamientoDotacional Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idAlojReg}", method = RequestMethod.GET)
	public @ResponseBody
	AlojamientoDotacional get(@PathVariable Integer idAlojReg) {
		AlojamientoDotacional alojamientoDotacional = new AlojamientoDotacional();
		alojamientoDotacional.setIdAlojReg(idAlojReg);
		alojamientoDotacional = this.alojamientoDotacionalService
				.find(alojamientoDotacional);
		AlojamientoDotacionalController.logger
				.info("[GET - findBy_PK] : Obtener AlojamientoDotacional por PK");
		return alojamientoDotacional;
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param alojamientoDotacional
	 *            AlojamientoDotacional Bean que contiene la informacion a
	 *            modificar.
	 * @return AlojamientoDotacional Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	AlojamientoDotacional edit(
			@RequestBody AlojamientoDotacional alojamientoDotacional) {
		AlojamientoDotacional alojamientoDotacionalAux = this.alojamientoDotacionalService
				.update(alojamientoDotacional);
		AlojamientoDotacionalController.logger
				.info("[PUT] : AlojamientoDotacional actualizado correctamente");
		return alojamientoDotacionalAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param alojamientoDotacional
	 *            AlojamientoDotacional Bean que contiene la informacion con la
	 *            que se va a crear el nuevo registro.
	 * @return AlojamientoDotacional Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	AlojamientoDotacional add(
			@RequestBody AlojamientoDotacional alojamientoDotacional) {
		AlojamientoDotacional alojamientoDotacionalAux = this.alojamientoDotacionalService
				.add(alojamientoDotacional);
		AlojamientoDotacionalController.logger
				.info("[POST] : AlojamientoDotacional insertado correctamente");
		return alojamientoDotacionalAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idAlojReg
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return AlojamientoDotacional Bean eliminado.
	 */
	@RequestMapping(value = "/{idAlojReg}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	AlojamientoDotacional remove(@PathVariable Integer idAlojReg) {
		AlojamientoDotacional alojamientoDotacional = new AlojamientoDotacional();
		alojamientoDotacional.setIdAlojReg(idAlojReg);
		this.alojamientoDotacionalService.remove(alojamientoDotacional);
		AlojamientoDotacionalController.logger
				.info("[DELETE] : AlojamientoDotacional borrado correctamente");
		return alojamientoDotacional;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		AlojamientoDotacionalController.logger
				.info("[GET - View] : alojamientodotacional");
		this.precargarDatosPantalla(model);
		model.addAttribute("regVPPAdaptada", UtilSeguridad.bUsuarioTieneRol("REG_VPP_ADAPTADA"));
		model.addAttribute("urlBaseDocumentos", UtilProperties.getProperty(AppConstantes.URL_DOCUMENTOS));
		return "seleccionAlojamientos";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterAlojDotacional
	 *            AlojamientoDotacional Bean que contiene los parametros de
	 *            filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<AlojamientoDotacional> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<AlojamientoDotacional> filter(
			@RequestJsonBody(param = "filter") AlojamientoDotacional filterAlojDotacional,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AlojamientoDotacionalController.logger
				.info("[POST - filter] : Obtener AlojamientoDotacionals");
		return this.alojamientoDotacionalService.filter(filterAlojDotacional,
				jqGridRequestDto, false);
	}

	/**
	 * Funcion encargada de realizar la exportacion a Excel
	 * 
	 * @param filterAlojDotacional
	 *            AlojamientoDotacional
	 * @param columns
	 *            String
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param modelMap
	 *            ModelMap
	 * @param request
	 *            ModelMap
	 * @return ModelAndView
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = { "xlsReport", "xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(
			@ModelAttribute AlojamientoDotacional filterAlojDotacional,
			@RequestParam(value = "columns", required = false) String columns,
			@ModelAttribute JQGridRequestDto jqGridRequestDto,
			ModelMap modelMap, HttpServletRequest request) {

		// Acceso a BD para recuperar datos
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);
		List<AlojamientoDotacional> listAlojDotAll = (List<AlojamientoDotacional>) this.alojamientoDotacionalService
				.filter(filterAlojDotacional, jqGridRequestDto, false)
				.getRows();

		// Datos
		List<Object> reportData = new ArrayList<Object>();

		// Hoja 1
		ReportData<AlojamientoDotacional> alojDotExcelDataAll = new ReportData<AlojamientoDotacional>();
		// Nombre hoja
		alojDotExcelDataAll.setSheetName("Aloj. Dotacionales Filtrados");
		// Cabeceras hoja
		alojDotExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));
		// Datos hoja
		alojDotExcelDataAll.setModelData(listAlojDotAll);
		reportData.add(alojDotExcelDataAll);

		// Nombre fichero
		modelMap.put("fileName", "AlojamientosDotacionales");
		// Excel Data
		modelMap.put("reportData", reportData);

		// Generación del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}
	}

	/**
	 * Funcion encargada de precargar los datos para la pantalla.
	 * 
	 * Combos, parametros...
	 * 
	 * @param model
	 *            Model
	 */
	@SuppressWarnings("unchecked")
	private void precargarDatosPantalla(Model model) {

		/** CONTRATOS ALQUILER **/
		// Combos Persona (Lupas Contrato Alquiler)
		this.personaController.precargarCombosFiltro(model);

		/** CLASIFICACION **/
		// Combo Tipo Regulacion (Detalle)
		model.addAttribute("comboTipoRegulacion", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_REGULACION)));
		// Combo Tipo Acceso (Detalle)
		model.addAttribute("comboTipoAcceso", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_ACCESO)));
		// Combo Tipo Iniciativa (Detalle)
		model.addAttribute("comboTipoIniciativa", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_INICIATIVA)));
		// Combo Tipo Promotor (Detalle)
		model.addAttribute("comboTipoPromotor", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PROMOTOR)));
		// Combo Tipo Propietarios (Detalle)
		model.addAttribute("comboTipoPropietarios", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PROPIETARIOS)));
		// Combo Tipo Adjudicacion (Detalle)
		model.addAttribute("comboTipoAdjudicacion", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_ADJUDICACION)));
		// Combo Tipo QuienAdjudica (Detalle)
		model.addAttribute("comboTipoQuienAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_QUIEN_ADJUDICA)));
		// Combo Tipo ComoAdjudica (Detalle)
		model.addAttribute("comboTipoComoAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_COMO_ADJUDICA)));
	}
}