package com.ejie.m62fb.control.registroVPP;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.AlojamientoDotacional;
import com.ejie.m62fb.model.ContratoAlquilerad;
import com.ejie.m62fb.service.registroVPP.AlojamientoDotacionalService;
import com.ejie.m62fb.service.registroVPP.ContratoAlquileradService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ContratoAlquileradController generated by UDA, 08-oct-2015 9:02:59.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/contratoalquilerad")
public class ContratoAlquileradController {

	private static final Logger logger = LoggerFactory
			.getLogger(ContratoAlquileradController.class);
	// @Autowired
	// private PersonaController personaController;
	@Autowired
	private ContratoAlquileradService contratoAlquileradService;
	@Autowired
	private ReloadableResourceBundleMessageSource messageErrorsSource;
	@Autowired
	private AlojamientoDotacionalService alojamientoDotacionalService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idContrato
	 *            Integer
	 * @return ContratoAlquilerad Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idContrato}", method = RequestMethod.GET)
	public @ResponseBody
	ContratoAlquilerad get(@PathVariable Integer idContrato) {
		ContratoAlquilerad contratoAlquilerad = new ContratoAlquilerad();
		contratoAlquilerad.setIdContrato(idContrato);
		contratoAlquilerad = this.contratoAlquileradService
				.find(contratoAlquilerad);
		ContratoAlquileradController.logger
				.info("[GET - findBy_PK] : Obtener ContratoAlquilerad por PK");
		return contratoAlquilerad;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterContratoAlquilerad
	 *            ContratoAlquilerad Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ContratoAlquilerad> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ContratoAlquilerad> getAll(
			@ModelAttribute ContratoAlquilerad filterContratoAlquilerad) {
		ContratoAlquileradController.logger
				.info("[GET - find_ALL] : Obtener ContratoAlquilerad por filtro");
		return this.contratoAlquileradService.findAll(filterContratoAlquilerad,
				null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param contratoAlquilerad
	 *            ContratoAlquilerad Bean que contiene la informacion a
	 *            modificar.
	 * @return ContratoAlquilerad Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	ContratoAlquilerad edit(@RequestBody ContratoAlquilerad contratoAlquilerad) {
		ContratoAlquilerad contratoAlquileradAux = this.contratoAlquileradService
				.update(contratoAlquilerad);
		ContratoAlquileradController.logger
				.info("[PUT] : ContratoAlquilerad actualizado correctamente");
		return contratoAlquileradAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param contratoAlquilerad
	 *            ContratoAlquilerad Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return ContratoAlquilerad Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	ContratoAlquilerad add(@RequestBody ContratoAlquilerad contratoAlquilerad) {
		ContratoAlquilerad contratoAlquileradAux = this.contratoAlquileradService
				.add(contratoAlquilerad);
		ContratoAlquileradController.logger
				.info("[POST] : ContratoAlquilerad insertado correctamente");
		return contratoAlquileradAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idContrato
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return ContratoAlquilerad Bean eliminado.
	 */
	@RequestMapping(value = "/{idContrato}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	ContratoAlquilerad remove(@PathVariable Integer idContrato) {
		ContratoAlquilerad contratoAlquilerad = new ContratoAlquilerad();
		contratoAlquilerad.setIdContrato(idContrato);
		this.contratoAlquileradService.remove(contratoAlquilerad);
		ContratoAlquileradController.logger
				.info("[DELETE] : ContratoAlquilerad borrado correctamente");
		return contratoAlquilerad;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	// /**
	// * Metodo de presentacion del RUP_TABLE.
	// *
	// * @param model Model
	// * @return String
	// */
	// @RequestMapping(value = "/maint", method = RequestMethod.GET)
	// public String getFormEdit(Model model) {
	// ContratoAlquileradController.logger.info("[GET - View] : contratoalquilerad");
	// personaController.precargarCombosFiltro(model);
	// return "contratoalquilerad";
	// }

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterContratoAlquilerad
	 *            ContratoAlquilerad Bean que contiene los parametros de
	 *            filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<ContratoAlquilerad> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ContratoAlquilerad> filter(
			@RequestJsonBody(param = "filter") ContratoAlquilerad filterContratoAlquilerad,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ContratoAlquileradController.logger
				.info("[POST - filter] : Obtener ContratoAlquilerads");
		return this.contratoAlquileradService.filter(filterContratoAlquilerad,
				jqGridRequestDto, false);
	}

	/**
	 * Operacion CRUD Read. Se valida la coherencia de las fechas inicio y fin.
	 * 
	 * @param filterContratoAlquilerAd
	 *            ContratoAlquilerAd
	 * @return String
	 */
	@RequestMapping(value = "/validarFechasIniFin", method = RequestMethod.GET)
	public @ResponseBody
	String validarFechasIniFin(
			@ModelAttribute ContratoAlquilerad filterContratoAlquilerAd) {
		String resultado = this.contratoAlquileradService
				.validarFechasIniFin(filterContratoAlquilerAd);
		ContratoAlquileradController.logger
				.info("[validarFechasIniFin - findBy_IDCONTRATO] : Se valida la coherencia de las fechas inicio/fin");
		if (resultado != null) {
			resultado = messageErrorsSource.getMessage(resultado, null,
					LocaleContextHolder.getLocale());
		}
		return resultado;
	}

	/**
	 * Operacion CRUD Read. Si estamos modificando el ultimo contrato del
	 * alojamiento dotacional, se valida si debemos modificar la fecha fin del
	 * contrato anterior.
	 * 
	 * @param filterContratoAlquilerad
	 *            ContratoAlquilerad
	 * @return String
	 */
	@RequestMapping(value = "/validarFechaFinContrato", method = RequestMethod.GET)
	public @ResponseBody
	Integer validarFechaFinContrato(
			@ModelAttribute ContratoAlquilerad filterContratoAlquilerad) {
		Integer resultado = this.contratoAlquileradService
				.validarFechaFinContrato(filterContratoAlquilerad);
		ContratoAlquileradController.logger
				.info("[validarFechaFinContrato - findBy_IDALOJREG_NUMCONTRATO] : Se valida si debemos modificar la fecha fin del contrato anterior");
		return resultado;
	}

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idalojreg
	 *            Integer
	 * @return ExpedienteVivienda
	 */
	@RequestMapping(value = "/getNumContrato/{idalojreg}", method = RequestMethod.GET)
	public @ResponseBody
	Integer getNumContrato(@PathVariable Integer idalojreg) {

		Integer numContrato = this.contratoAlquileradService.findNumContratoMaxMasUno(idalojreg);

		ContratoAlquileradController.logger
				.info("[getNumContrato - findBy_IDALOJREG] : Obtener Num Contratos de un alojamiento dado");
		return numContrato;
	}
}