package com.ejie.m62fb.control.registroVPP;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.ContratoAlquiler;
import com.ejie.m62fb.service.registroVPP.ContratoAlquilerService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ContratosAlquileresController generated by UDA, 20-oct-2015 12:03:31.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/contratosalquileres")
public class ContratosAlquileresController {

	private static final Logger logger = LoggerFactory
			.getLogger(ContratosAlquileresController.class);

	@Autowired
	private ContratoAlquilerService contratoAlquilerService;
	@Autowired
	private ReloadableResourceBundleMessageSource messageErrorsSource;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idContrato
	 *            Integer
	 * @return ContratoAlquiler Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idContrato}", method = RequestMethod.GET)
	public @ResponseBody
	ContratoAlquiler get(@PathVariable Integer idContrato) {
		ContratoAlquiler contratoAlquiler = new ContratoAlquiler(idContrato);
		contratoAlquiler = this.contratoAlquilerService.find(contratoAlquiler);
		ContratosAlquileresController.logger.info("[GET - findBy_PK] : Obtener ContratoAlquiler por PK");
		return contratoAlquiler;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterContratoAlquiler
	 *            ContratoAlquiler Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ContratoAlquiler> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ContratoAlquiler> getAll(
			@ModelAttribute ContratoAlquiler filterContratoAlquiler) {
		ContratosAlquileresController.logger
				.info("[GET - find_ALL] : Obtener ContratoAlquiler por filtro");
		return this.contratoAlquilerService.findAll(filterContratoAlquiler,
				null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler Bean que contiene la informacion a modificar.
	 * @return ContratoAlquiler Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	ContratoAlquiler edit(@RequestBody ContratoAlquiler contratoAlquiler) {
		ContratoAlquiler contratoAlquilerAux = this.contratoAlquilerService
				.update(contratoAlquiler);
		ContratosAlquileresController.logger
				.info("[PUT] : ContratoAlquiler actualizado correctamente");
		return contratoAlquilerAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param contratoAlquiler
	 *            ContratoAlquiler Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return ContratoAlquiler Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	ContratoAlquiler add(@RequestBody ContratoAlquiler contratoAlquiler) {
		ContratoAlquiler contratoAlquilerAux = this.contratoAlquilerService
				.add(contratoAlquiler);
		ContratosAlquileresController.logger
				.info("[POST] : ContratoAlquiler insertado correctamente");
		return contratoAlquilerAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idContrato
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return ContratoAlquiler Bean eliminado.
	 */
	@RequestMapping(value = "/{idContrato}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	ContratoAlquiler remove(@PathVariable Integer idContrato) {
		ContratoAlquiler contratoAlquiler = new ContratoAlquiler(idContrato);
		this.contratoAlquilerService.remove(contratoAlquiler);
		ContratosAlquileresController.logger.info("[DELETE] : ContratoAlquiler borrado correctamente");
		return contratoAlquiler;
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterContratosAlquileres
	 *            ContratoAlquiler Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @param request
	 *            HttpServletRequest
	 * @return JQGridResponseDto<ContratoAlquiler> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ContratoAlquiler> filter(
			@RequestJsonBody(param = "filter") ContratoAlquiler filterContratosAlquileres,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		ContratosAlquileresController.logger
				.info("[POST - filter] : Obtener ContratosAlquileres");
		return this.contratoAlquilerService.filter(filterContratosAlquileres,
				jqGridRequestDto, false);
	}

	/**
	 * @param filterContratoAlquiler
	 *            ContratoAlquiler
	 * @return Date
	 */
	@RequestMapping(value = "/calcFecFinProrForzosa", method = RequestMethod.GET)
	public @ResponseBody
	ContratoAlquiler calcFecFinProrForzosa(
			@ModelAttribute ContratoAlquiler filterContratoAlquiler) {
		Date fecFinProrForzosa = this.contratoAlquilerService
				.calcFecFinProrForzosa(filterContratoAlquiler.getFecInicio(),
						filterContratoAlquiler.getFecFin(),
						filterContratoAlquiler.getFecEntContrato());
		// Se añade al objeto la fecha fin de prorroga forzosa calculada
		filterContratoAlquiler.setFecFinForzosa(fecFinProrForzosa);
		ContratosAlquileresController.logger
				.info("[calcFecFinProrForzosa] : calculo fecha fin prorroga forzosa");
		return filterContratoAlquiler;
	}

	/**
	 * Operacion CRUD Read. Se valida la coherencia de las fechas inicio y fin.
	 * 
	 * @param filterContratoAlquiler
	 *            ContratoAlquiler
	 * @return String
	 */
	@RequestMapping(value = "/validarFechasIniFin", method = RequestMethod.GET)
	public @ResponseBody
	String validarFechasIniFin(
			@ModelAttribute ContratoAlquiler filterContratoAlquiler) {
		String resultado = this.contratoAlquilerService
				.validarFechasIniFin(filterContratoAlquiler);
		ContratosAlquileresController.logger
				.info("[validarFechasIniFin - findBy_NUMVIVIENDA_IDINQU] : Se valida la coherencia de las fechas inicio/fin");
		if (resultado != null) {
			resultado = messageErrorsSource.getMessage(resultado, null,
					LocaleContextHolder.getLocale());
		}
		return resultado;
	}

	/**
	 * Operacion CRUD Read. Si estamos modificando el ultimo contrato de la
	 * vivienda, se valida si debemos modificar la fecha fin del contrato
	 * anterior.
	 * 
	 * @param filterContratoAlquiler
	 *            ContratoAlquiler
	 * @return String
	 */
	@RequestMapping(value = "/validarFechaFinContrato", method = RequestMethod.GET)
	public @ResponseBody
	Integer validarFechaFinContrato(
			@ModelAttribute ContratoAlquiler filterContratoAlquiler) {
		Integer resultado = this.contratoAlquilerService
				.validarFechaFinContrato(filterContratoAlquiler);
		ContratosAlquileresController.logger
				.info("[validarFechaFinContrato - findBy_NUMVIVIENDA_IDINQU] : Se valida si debemos modificar la fecha fin del contrato anterior");
		return resultado;
	}
}
