package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.control.PersonaController;
import com.ejie.m62fb.model.ContratoAlquilerElemento;
import com.ejie.m62fb.model.tipos.P28Tipos;
import com.ejie.m62fb.service.registroVPP.ContratoAlquilerElementoService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ContratoAlquilerElementoController generated by UDA, 09-dic-2015 10:41:42.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/contratosalquilereselemento")
public class ContratosAlquileresElementoController {

	private static final Logger logger = LoggerFactory
			.getLogger(ContratosAlquileresElementoController.class);

	@Autowired
	private ContratoAlquilerElementoService contratoAlquilerElementoService;
	@SuppressWarnings("rawtypes")
	@Autowired
	private ComboController comboController;
	@Autowired
	private PersonaController personaController;
	@Autowired
	private ReloadableResourceBundleMessageSource messageErrorsSource;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idContratoAlq
	 *            BigDecimal
	 * @return ContratoAlquilerElemento Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idContratoAlq}", method = RequestMethod.GET)
	public @ResponseBody
	ContratoAlquilerElemento get(@PathVariable BigDecimal idContratoAlq) {
		ContratoAlquilerElemento contratoAlquilerElemento = new ContratoAlquilerElemento();
		contratoAlquilerElemento.setIdContratoAlquiler(idContratoAlq);
		contratoAlquilerElemento = this.contratoAlquilerElementoService
				.find(contratoAlquilerElemento);
		ContratosAlquileresElementoController.logger
				.info("[GET - findBy_PK] : Obtener ContratoAlquilerElemento por PK");
		return contratoAlquilerElemento;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterContratoAlquilerElemento
	 *            ContratoAlquilerElemento Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ContratoAlquilerElemento> Lista de objetos correspondientes
	 *         a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ContratoAlquilerElemento> getAll(
			@ModelAttribute ContratoAlquilerElemento filterContratoAlquilerElemento) {
		ContratosAlquileresElementoController.logger
				.info("[GET - find_ALL] : Obtener ContratoAlquilerElemento por filtro");
		return this.contratoAlquilerElementoService.findAll(
				filterContratoAlquilerElemento, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param contratoAlquilerElemento
	 *            ContratoAlquilerElemento Bean que contiene la informacion a
	 *            modificar.
	 * @return ContratoAlquilerElemento Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	ContratoAlquilerElemento edit(
			@RequestBody ContratoAlquilerElemento contratoAlquilerElemento) {
		ContratoAlquilerElemento contratoAlquilerElementoAux = this.contratoAlquilerElementoService
				.update(contratoAlquilerElemento);
		ContratosAlquileresElementoController.logger
				.info("[PUT] : ContratoAlquilerElemento actualizado correctamente");
		return contratoAlquilerElementoAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param contratoAlquilerElemento
	 *            ContratoAlquilerElemento Bean que contiene la informacion con
	 *            la que se va a crear el nuevo registro.
	 * @return ContratoAlquilerElemento Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	ContratoAlquilerElemento add(
			@RequestBody ContratoAlquilerElemento contratoAlquilerElemento) {
		ContratoAlquilerElemento contratoAlquilerElementoAux = this.contratoAlquilerElementoService
				.add(contratoAlquilerElemento);
		ContratosAlquileresElementoController.logger
				.info("[POST] : ContratoAlquilerElemento insertado correctamente");
		return contratoAlquilerElementoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idContratoAlq
	 *            BigDecimal Identificador del objeto que se desea eliminar.
	 * @return ContratoAlquilerElemento Bean eliminado.
	 */
	@RequestMapping(value = "/{idContratoAlq}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	ContratoAlquilerElemento remove(@PathVariable BigDecimal idContratoAlq) {
		ContratoAlquilerElemento contratoAlquilerElemento = new ContratoAlquilerElemento();
		contratoAlquilerElemento.setIdContratoAlquiler(idContratoAlq);
		this.contratoAlquilerElementoService.remove(contratoAlquilerElemento);
		ContratosAlquileresElementoController.logger
				.info("[DELETE] : ContratoAlquilerElemento borrado correctamente");
		return contratoAlquilerElemento;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		ContratosAlquileresElementoController.logger
				.info("[GET - View] : contratosalquilereselemento");
		this.precargarCombosDetalle(model);
		return "contratosalquilereselemento";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterContratoAlquilerElemento
	 *            ContratoAlquilerElemento Bean que contiene los parametros de
	 *            filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<ContratoAlquilerElemento> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ContratoAlquilerElemento> filter(
			@RequestJsonBody(param = "filter") ContratoAlquilerElemento filterContratoAlquilerElemento,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ContratosAlquileresElementoController.logger
				.info("[POST - filter] : Obtener ContratoAlquilerElementos");
		return this.contratoAlquilerElementoService.filter(
				filterContratoAlquilerElemento, jqGridRequestDto, false);
	}

	/**
	 * Funcion encargada de precargar las combos para el detalle
	 * 
	 * @param model
	 *            Model
	 */
	@SuppressWarnings("unchecked")
	public void precargarCombosDetalle(Model model) {
		// Tipo Contrato Alquiler
		model.addAttribute("comboTipoContrato", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.TIPO_CONTRATO_ALQUILER)));
		// Carga combos Persona (estado civil, sexo, nacionalidad)
		this.personaController.precargarCombosFiltro(model);
	}

	/**
	 * Operacion CRUD Read. Se valida la coherencia de las fechas inicio y fin.
	 * 
	 * @param filterContratoAlquilerElemento
	 *            ContratoAlquilerElemento
	 * @return String
	 */
	@RequestMapping(value = "/validarFechasIniFin", method = RequestMethod.GET)
	public @ResponseBody
	String validarFechasIniFin(
			@ModelAttribute ContratoAlquilerElemento filterContratoAlquilerElemento) {
		String resultado = this.contratoAlquilerElementoService
				.validarFechasIniFin(filterContratoAlquilerElemento);
		ContratosAlquileresElementoController.logger
				.info("[validarFechasIniFin - findBy_IDELEMENTO_IDCONTRATOALQ] : Se valida la coherencia de las fechas inicio/fin");
		if (resultado != null) {
			resultado = messageErrorsSource.getMessage(resultado, null,
					LocaleContextHolder.getLocale());
		}
		return resultado;
	}

	/**
	 * Operacion CRUD Read. Si estamos modificando el ultimo contrato de la
	 * vivienda, se valida si debemos modificar la fecha fin del contrato
	 * anterior.
	 * 
	 * @param filterContratoAlquilerElemento
	 *            ContratoAlquilerElemento
	 * @return String
	 */
	@RequestMapping(value = "/validarFechaFinContrato", method = RequestMethod.GET)
	public @ResponseBody
	Integer validarFechaFinContrato(
			@ModelAttribute ContratoAlquilerElemento filterContratoAlquilerElemento) {
		Integer resultado = this.contratoAlquilerElementoService
				.validarFechaFinContrato(filterContratoAlquilerElemento);
		ContratosAlquileresElementoController.logger
				.info("[validarFechaFinContrato - findBy_IDELEMENTO_IDCONTRATOALQ] : Se valida si debemos modificar la fecha fin del contrato anterior");
		return resultado;
	}
}