package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.ContratoVivienda;
import com.ejie.m62fb.model.ExpedienteVivienda;
import com.ejie.m62fb.model.Propietario;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.plsql.PlSqlOutputError;
import com.ejie.m62fb.service.registroVPP.ContratoViviendaService;
import com.ejie.m62fb.service.registroVPP.ExpedienteViviendaService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ElementoController generated by UDA, 04-sep-2015 12:04:01.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/contratosviviendas")
public class ContratosViviendasController {

	private static final Logger logger = LoggerFactory
			.getLogger(ContratosViviendasController.class);

	@Autowired
	private ContratoViviendaService contratoViviendaService;

	@Autowired
	private ExpedienteViviendaService expedienteViviendaService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idContrato
	 *            Integer
	 * @return ContratoVivienda Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idContrato}", method = RequestMethod.GET)
	public @ResponseBody
	ContratoVivienda get(@PathVariable Integer idContrato) {
		ContratoVivienda contratoVivienda = new ContratoVivienda(idContrato);
		contratoVivienda = this.contratoViviendaService.find(contratoVivienda);
		ContratosViviendasController.logger
				.info("[GET - findBy_PK] : Obtener ContratoVivienda por PK");
		return contratoVivienda;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterContratoVivienda
	 *            ContratoVivienda Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ContratoVivienda> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ContratoVivienda> getAll(
			@ModelAttribute ContratoVivienda filterContratoVivienda) {
		ContratosViviendasController.logger
				.info("[GET - find_ALL] : Obtener ContratoVivienda por filtro");
		return this.contratoViviendaService.findAll(filterContratoVivienda,
				null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda Bean que contiene la informacion a modificar.
	 * @return ContratoVivienda Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	ContratoVivienda edit(@RequestBody ContratoVivienda contratoVivienda) {
		ContratoVivienda contratoViviendaAux = this.contratoViviendaService
				.update(contratoVivienda);
		ContratosViviendasController.logger
				.info("[PUT] : ContratoVivienda actualizado correctamente");
		return contratoViviendaAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param contratoVivienda
	 *            ContratoVivienda Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return ContratoVivienda Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	ContratoVivienda add(@RequestBody ContratoVivienda contratoVivienda) {
		ContratoVivienda contratoViviendaAux = this.contratoViviendaService
				.add(contratoVivienda);
		ContratosViviendasController.logger
				.info("[POST] : ContratoVivienda insertado correctamente");
		return contratoViviendaAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idVivienda
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @param idPropietario
	 *            BigDecimal Identificador del objeto que se desea eliminar.
	 * @return ContratoVivienda Bean eliminado.
	 */
	@RequestMapping(value = "/{idContrato}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	ContratoVivienda remove(@PathVariable Integer idContrato) {
		ContratoVivienda contratoVivienda = new ContratoVivienda(idContrato);
		this.contratoViviendaService.remove(contratoVivienda);
		ContratosViviendasController.logger
				.info("[DELETE] : ContratoVivienda borrado correctamente");
		return contratoVivienda;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	// /**
	// * Metodo de presentacion del RUP_TABLE.
	// *
	// * @param model
	// * Model
	// * @return String
	// */
	// @RequestMapping(value = "/maint", method = RequestMethod.GET)
	// public String getFormEdit(Model model) {
	// ContratosViviendasController.logger
	// .info("[GET - View] : contratosviviendas");
	// this.precargarCombosDetalle(model);
	// return "contratosviviendas";
	// }

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterContratosViviendas
	 *            ContratoVivienda Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @param request
	 *            HttpServletRequest
	 * @return JQGridResponseDto<ContratoVivienda> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ContratoVivienda> filter(
			@RequestJsonBody(param = "filter") ContratoVivienda filterContratosViviendas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		ContratosViviendasController.logger
				.info("[POST - filter] : Obtener ContratosViviendas");
		return this.contratoViviendaService.filter(filterContratosViviendas,
				jqGridRequestDto, false);
	}

	// /**
	// * Funcion encargada de precargar las combos para el detalle
	// *
	// * @param model
	// * Model
	// */
	// @SuppressWarnings("unchecked")
	// public void precargarCombosDetalle(Model model) {
	// // Tipo Transmision
	// model.addAttribute("comboTipoTransmision", P28Tipos
	// .serializeComboP28(this.comboController
	// .getComboP28(P28Tipos.TIPO_TRANSMISION)));
	// Carga combos Persona (estado civil, sexo, nacionalidad)
	// this.personaController.precargarCombosFiltro(model);
	// }

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idVivienda
	 *            Integer
	 * @return ExpedienteVivienda
	 */
	@RequestMapping(value = "/getNumExpediente/{idVivienda}", method = RequestMethod.GET)
	public @ResponseBody
	ExpedienteVivienda getNumExpediente(@PathVariable Integer idVivienda) {
		ExpedienteVivienda expedienteVivienda = this.expedienteViviendaService
				.findExpedienteVivienda(idVivienda);
		ContratosViviendasController.logger
				.info("[getNumExpediente - findBy_NUMVIVIENDA] : Obtener NumExpediente por numVivienda");
		return expedienteVivienda;
	}

	/**
	 * Operacion CRUD Read. Esta funcion valida si al visar un contrato se esta
	 * superando el importe maximo autorizado en la promocion.
	 * 
	 * @param numExpediente
	 *            String
	 * @param idVivienda
	 *            BigDecimal
	 * @return PlSqlOutputError
	 */
	@RequestMapping(value = "/validarImporteAutorizado/{numExpediente}/{idVivienda}", method = RequestMethod.GET)
	public @ResponseBody
	PlSqlOutputError validarImporteAutorizado(
			@PathVariable String numExpediente,
			@PathVariable BigDecimal idVivienda) {
		PlSqlOutputError plSqlOutputError = this.contratoViviendaService
				.validarImporteAutorizado(numExpediente, idVivienda);
		ContratosViviendasController.logger
				.info("[validarImporteAutorizado] : validarImporteAutorizado por numExpediente e idVivienda");
		return plSqlOutputError;
	}

	/**
	 * Operacion CRUD Read. Funcion que busca si existe algun contrato de
	 * primera transmision para la vivienda seleccionada.
	 * 
	 * @param idVivienda
	 *            BigDecimal
	 * @return Long
	 */
	@RequestMapping(value = "/getContratoPrimerTrans/{idVivienda}", method = RequestMethod.GET)
	public @ResponseBody
	Long getContratoPrimerTrans(@PathVariable BigDecimal idVivienda) {
		Long countRegistros = this.contratoViviendaService
				.findContratoPrimerTrans(idVivienda);
		ContratosViviendasController.logger
				.info("[getContratoPrimerTrans] : getContratoPrimerTrans por idVivienda");
		return countRegistros;
	}

	// /**
	// * Operacion CRUD Read. Consulta si el titular esta en otro contrato de
	// otra
	// * vivienda.
	// *
	// * @param idVivienda
	// * Integer
	// * @param idPersona
	// * Integer
	// * @return boolean
	// */
	// @RequestMapping(value =
	// "/getTitularesContratos/{idVivienda}/{idPersona}", method =
	// RequestMethod.GET)
	// public @ResponseBody
	// boolean getTitularesContratos(@PathVariable Integer idVivienda,
	// @PathVariable Integer idPersona) {
	// boolean resultado = this.propietarioService.findTitularesContratos(
	// idVivienda, idPersona);
	// ContratosViviendasController.logger
	// .info("[getTitularesContratos - findBy_NUMVIVIENDA_IDPERS] : Consulta si el titular esta en otro contrato de otra vivienda");
	// return resultado;
	// }
}