package com.ejie.m62fb.control.registroVPP;


import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.model.Elemento;
import com.ejie.m62fb.model.ElementoActMasiva;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.tipos.P28Tipos;
import com.ejie.m62fb.service.registroVPP.ElementosService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.reports.ReportData;

/**
 * ElementoController generated by UDA, 04-sep-2015 12:04:01.
XXXX
 */
 
@Controller
@RequestMapping (value = "/elementos/actmasiva")

public class ElementosActMasivaController  {

	private static final Logger logger = LoggerFactory.getLogger(ElementosActMasivaController.class);

	@Autowired
	private ElementosService elementosService;
	@Autowired
	private ComboController comboController;
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param ideElemento Integer
	 * @return Elemento 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{ideElemento}", method = RequestMethod.GET)
	public @ResponseBody Elemento get(@PathVariable BigDecimal ideElemento) {
        Elemento elemento = new Elemento();
		elemento.setIdeElemento(ideElemento);
        elemento = this.elementosService.find(elemento);
        ElementosActMasivaController.logger.info("[GET - findBy_PK] : Obtener Elemento por PK");
        return elemento;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterElemento Elemento
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<Elemento> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Elemento> getAll(@ModelAttribute Elemento filterElemento) {
		ElementosActMasivaController.logger.info("[GET - find_ALL] : Obtener Elemento por filtro");
	    return this.elementosService.findAll(filterElemento, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param elemento Elemento 
	 *            Bean que contiene la informacion a modificar.
	 * @return Elemento 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody Elemento edit(@RequestBody ElementoActMasiva elemento) {	
		Elemento elementoFinal = null;
		if(elemento.getType().equals("PORTAL")){
			elementoFinal=this.elementosService.updatePortal(elemento);
		}else if(elemento.getType().equals("COD_POSTAL")){
			elementoFinal=this.elementosService.updateDireccion(elemento);
		}
		return elementoFinal;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param elemento Elemento 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return Elemento
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Elemento add(@RequestBody ElementoActMasiva elemento) {
//		if(elemento.getType().equals("PORTAL")){
//			this.elementosService.updatePortal(elemento);
//		}else if(elemento.getType().equals("COD_POSTAL")){
//			this.elementosService.updateDireccion(elemento);
//		}
		return null;
//        Elemento elementoAux = this.elementosService.add(elemento);
//        ElementosActMasivaController.logger.info("[POST] : Elemento insertado correctamente");
//    	return elementoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param ideElemento Integer
	 *            Identificador del objeto que se desea eliminar.
	 * @return Elemento
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{ideElemento}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody Elemento remove(@PathVariable BigDecimal ideElemento) {
        Elemento elemento = new Elemento();
        elemento.setIdeElemento(ideElemento);
        this.elementosService.remove(elemento);
       	ElementosActMasivaController.logger.info("[DELETE] : Elemento borrado correctamente");
       	return elemento;
    }
	
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 */
	@RequestMapping(value = "/seleccion/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		ElementosActMasivaController.logger.info("[GET - View] : elemento");
		this.precargarCombosFiltro(model);
		return "elementoActMasiva";
	}
	 
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterElemento Elemento
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Elemento>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Elemento> filter(
			@RequestJsonBody(param="filter") Elemento filterElemento,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		ElementosActMasivaController.logger.info("[POST - filter] : Obtener Elementos");
		return this.elementosService.filter(filterElemento, jqGridRequestDto, false);
	}
	
	/**
	 * Funcion encargada de precargar los combos para el filtro
	 * 
	 * @param model
	 *            Model
	 */
	private void precargarCombosFiltro(Model model) {
		// Tipo de elemento
		model.addAttribute("comboTipoElemento", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_ELEMENTO)));
		// Tipo de vinculación
		model.addAttribute("comboTipoVinculacion", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_VINCULACION)));
		// Tipo Si No
		model.addAttribute("comboTipoSiNo", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_SI_NO)));
	}
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterElemento Elemento
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Elemento>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
//	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
//	public @ResponseBody JQGridResponseDto<JerarquiaDto<Elemento>> jerarquia(
//			@RequestJsonBody(param="filter") Elemento filterElemento,
//			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
//		ElementosController.logger.info("[POST - jerarquia] : Obtener Elementos jerarquia");
//		return this.elementosService.jerarquia(filterElemento, jqGridRequestDto, false);
//	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterElemento Elemento
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Elemento>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
//	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
//	public @ResponseBody JQGridResponseDto<JerarquiaDto<Elemento>> jerarquiaChildren(
//			@RequestJsonBody(param="filter") Elemento  filterElemento ,
//			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
//		ElementosController.logger.info("[POST - jerarquia] : Obtener Elementos jerarquia - Hijos");
//		return this.elementosService.jerarquiaChildren(filterElemento, jqGridRequestDto);
//	}
	@RequestMapping(value = {"xlsReport" , "xlsxReport"}, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute Elemento filterElemento, @ModelAttribute JQGridRequestDto jqGridRequestDto,
			ModelMap modelMap,
			@RequestParam(value = "columns", required = false) String columns,
			HttpServletRequest request){
		
		//Acceso a BD para recuperar datos
		List<Elemento> listElementoPage = this.elementosService.findAllLike(filterElemento, jqGridRequestDto, false);
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);
		List<Elemento> listElementoAll = this.elementosService.findAllLike(filterElemento, jqGridRequestDto, false);
		
		//Nombre fichero
		modelMap.put("fileName", "Elementos");
		
		//Datos
		List<Object> reportData = new ArrayList<Object>();
			//Hoja 1
			ReportData<Elemento> elementoExcelDataAll = new ReportData<Elemento>();
				//nombre hoja
				elementoExcelDataAll.setSheetName("Todos los Elementos");
				//cabeceras hoja
				elementoExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));
				//datos hoja
				elementoExcelDataAll.setModelData(listElementoAll);
			reportData.add(elementoExcelDataAll);
			//Hoja 2
			ReportData<Elemento> elementoExcelDataPage = new ReportData<Elemento>();
				//nombre hoja
				elementoExcelDataPage.setSheetName("Página 1 de Elementos");
				//cabeceras hoja
				elementoExcelDataPage.setHeaderNames(ReportData.parseColumns(columns));
				//datos hoja
				elementoExcelDataPage.setModelData(listElementoPage);
			reportData.add(elementoExcelDataPage);
		modelMap.put("reportData", reportData);
		
		//Generación del XLS o XLSX
//		String reportView = request.getServletPath().substring(9);
		if (request.getServletPath().indexOf("xlsReport")!=-1){
			return new ModelAndView("xlsReport", modelMap);
		}else{
			return new ModelAndView("xlsxReport", modelMap);
		}
//		return new ModelAndView(reportView, modelMap);
		
	}
	
}	
