package com.ejie.m62fb.control.registroVPP;


import java.math.BigDecimal;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.control.PersonaController;
import com.ejie.m62fb.model.Elemento;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.persona.Persona;
import com.ejie.m62fb.model.plsql.PlSqlOutputError;
import com.ejie.m62fb.model.tipos.P28Tipos;
import com.ejie.m62fb.model.tipos.TipoClave;
import com.ejie.m62fb.model.tipos.TipoElemento;
import com.ejie.m62fb.service.registroVPP.ElementosService;
import com.ejie.m62fb.service.registroVPP.PersonaService;
import com.ejie.m62fb.util.UtilSeguridad;
import com.ejie.m62fb.utils.constants.AppConstantes;
import com.ejie.m62fb.utils.properties.UtilProperties;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.reports.ReportData;

/**
 * ElementoController generated by UDA, 04-sep-2015 12:04:01.
XXXX
 */
 
@Controller
@RequestMapping (value = "/elementos")

public class ElementosController  {

	private static final Logger logger = LoggerFactory.getLogger(ElementosController.class);

	@Autowired
	private ElementosService elementosService;
	@Autowired
	private ComboController comboController;
	@Autowired
	private PersonaController personaController;
	@Autowired
	private PersonaService personaService;
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param ideElemento Integer
	 * @return Elemento 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = {"/{ideElemento}","/garaje/{ideElemento}","/trastero/{ideElemento}"}, method = RequestMethod.GET)
	public @ResponseBody Elemento get(@PathVariable BigDecimal ideElemento) {
        Elemento elemento = new Elemento();
		elemento.setIdeElemento(ideElemento);
        elemento = this.elementosService.find(elemento);
        ElementosController.logger.info("[GET - findBy_PK] : Obtener Elemento por PK");
        return elemento;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterElemento Elemento
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<Elemento> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Elemento> getAll(@ModelAttribute Elemento filterElemento) {
		ElementosController.logger.info("[GET - find_ALL] : Obtener Elemento por filtro");
	    return this.elementosService.findAll(filterElemento, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param elemento Elemento 
	 *            Bean que contiene la informacion a modificar.
	 * @return Elemento 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody Elemento edit(@RequestBody Elemento elemento) {		
        Elemento elementoAux = this.elementosService.update(elemento);
		ElementosController.logger.info("[PUT] : Elemento actualizado correctamente");
        return elementoAux;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param elemento Elemento 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return Elemento
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Elemento add(@RequestBody Elemento elemento) {		
        Elemento elementoAux = this.elementosService.add(elemento);
        ElementosController.logger.info("[POST] : Elemento insertado correctamente");
    	return elementoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param ideElemento Integer
	 *            Identificador del objeto que se desea eliminar.
	 * @return Elemento
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{ideElemento}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody Elemento remove(@PathVariable BigDecimal ideElemento) {
        Elemento elemento = new Elemento();
        elemento.setIdeElemento(ideElemento);
        this.elementosService.remove(elemento);
       	ElementosController.logger.info("[DELETE] : Elemento borrado correctamente");
       	return elemento;
    }
	/**
	 * 
	 * @param idPersona
	 * @param tipoCodigoId
	 * @param codigoId
	 * @param nombre
	 * @param apellido1
	 * @param apellido2
	 * @param codProvincia
	 * @param codMunicipio
	 * @return
	 */
	@RequestMapping(value = "/garaje/findAllLike", method = RequestMethod.GET)
	public @ResponseBody
	List<Elemento> findAllLikeGaraje(
			@RequestParam(value = "ideElemento", required = false) String ideElemento) {
		// Filtro
		Elemento elemento = new Elemento();
		if (ideElemento != null && !ideElemento.isEmpty()) {
			elemento.setIdeElemento(new BigDecimal(ideElemento));
			elemento.setTipoElemento(new TipoElemento(TipoElemento.GARAJE));
		}
		return this.elementosService.findAllLike(elemento, null, false);
	}
	/**
	 * 
	 * @param idPersona
	 * @param tipoCodigoId
	 * @param codigoId
	 * @param nombre
	 * @param apellido1
	 * @param apellido2
	 * @param codProvincia
	 * @param codMunicipio
	 * @return
	 */
	@RequestMapping(value = "/trastero/findAllLike", method = RequestMethod.GET)
	public @ResponseBody
	List<Elemento> findAllLikeTrastero(
			@RequestParam(value = "ideElemento", required = false) String ideElemento) {
		// Filtro
		Elemento elemento = new Elemento();
		if (ideElemento != null && !ideElemento.isEmpty()) {
			elemento.setIdeElemento(new BigDecimal(ideElemento));
			elemento.setTipoElemento(new TipoElemento(TipoElemento.TRASTERO));
		}
		return this.elementosService.findAllLike(elemento, null, false);
	}
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 * @throws MalformedURLException 
	 */
	@RequestMapping(value = "/seleccion/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model,@RequestParam(value = "numElemento", required = false) String numElemento) throws MalformedURLException {
		ElementosController.logger.info("[GET - View] : elemento");
		this.precargarCombosFiltro(model);
//		Persona persona = new Persona();
//		persona.setTipoCodigoId(new TipoClave(TipoClave.DNI));
//		persona.setCodigoId("30000529L");
//		persona.setApellido1("GARCIA");
//		personaService.insertPeticionSVDI(persona);
		personaController.precargarCombosFiltro(model);
		numElemento = numElemento==null?"''":numElemento;
		model.addAttribute("numElemento", numElemento);
		model.addAttribute("regVPPAdaptada", UtilSeguridad.bUsuarioTieneRol("REG_VPP_ADAPTADA"));
		model.addAttribute("urlBaseDocumentos", UtilProperties.getProperty(AppConstantes.URL_DOCUMENTOS));
		return "elemento";
	}
	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 * @throws MalformedURLException 
	 */
	@RequestMapping(value = "/seleccion/maint/{numElemento}", method = RequestMethod.GET)
	public String getFormEdit(Model model,@PathVariable BigDecimal numElemento) throws MalformedURLException {
		ElementosController.logger.info("[GET - View] : elemento");
		return getFormEdit(model,numElemento.toString());
//		this.precargarCombosFiltro(model);
//		personaController.precargarCombosFiltro(model);
//		model.addAttribute("numElemento", numElemento);
//		model.addAttribute("regVPPAdaptada", UtilSeguridad.bUsuarioTieneRol("REG_VPP_ADAPTADA"));
//		return "elemento";
	}
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterElemento Elemento
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Elemento>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Elemento> filter(
			@RequestJsonBody(param="filter") Elemento filterElemento,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		ElementosController.logger.info("[POST - filter] : Obtener Elementos");
		return this.elementosService.filter(filterElemento, jqGridRequestDto, false);
	}
	
	/**
	 * Funcion encargada de precargar los combos para el filtro
	 * 
	 * @param model
	 *            Model
	 */
	private void precargarCombosFiltro(Model model) {
		// Tipo de elemento
		model.addAttribute("comboTipoElemento", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_ELEMENTO)));
		// Tipo de vinculación
		model.addAttribute("comboTipoVinculacion", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_VINCULACION)));
		// Tipo Si No
		model.addAttribute("comboTipoSiNo", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_SI_NO)));
		
		// Tipo Contrato Alquiler
		model.addAttribute("comboTipoContrato", P28Tipos
				.serializeComboP28(this.comboController.getComboP28(P28Tipos.TIPO_CONTRATO_ALQUILER)));
	}
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterElemento Elemento
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Elemento>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
//	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
//	public @ResponseBody JQGridResponseDto<JerarquiaDto<Elemento>> jerarquia(
//			@RequestJsonBody(param="filter") Elemento filterElemento,
//			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
//		ElementosController.logger.info("[POST - jerarquia] : Obtener Elementos jerarquia");
//		return this.elementosService.jerarquia(filterElemento, jqGridRequestDto, false);
//	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterElemento Elemento
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Elemento>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
//	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
//	public @ResponseBody JQGridResponseDto<JerarquiaDto<Elemento>> jerarquiaChildren(
//			@RequestJsonBody(param="filter") Elemento  filterElemento ,
//			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
//		ElementosController.logger.info("[POST - jerarquia] : Obtener Elementos jerarquia - Hijos");
//		return this.elementosService.jerarquiaChildren(filterElemento, jqGridRequestDto);
//	}
	@RequestMapping(value = {"xlsReport" , "xlsxReport"}, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute Elemento filterElemento, 
			@ModelAttribute JQGridRequestDto jqGridRequestDto,
			ModelMap modelMap,
			@RequestParam(value = "columns", required = false) String columns,
			HttpServletRequest request){
		
		//Acceso a BD para recuperar datos
		//List<Elemento> listElementoPage = this.elementosService.findAllLike(filterElemento, jqGridRequestDto, false);
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);
		List<Elemento> listElementoAll = this.elementosService.findAllLike(filterElemento, jqGridRequestDto, false);
		
		//Nombre fichero
		modelMap.put("fileName", "Elementos");
		
		//Datos
		List<Object> reportData = new ArrayList<Object>();
			//Hoja 1
			ReportData<Elemento> elementoExcelDataAll = new ReportData<Elemento>();
				//nombre hoja
				elementoExcelDataAll.setSheetName("Todos los Elementos");
				//cabeceras hoja
				elementoExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));
				//datos hoja
				elementoExcelDataAll.setModelData(listElementoAll);
			reportData.add(elementoExcelDataAll);
//			//Hoja 2
//			ReportData<Elemento> elementoExcelDataPage = new ReportData<Elemento>();
//				//nombre hoja
//				elementoExcelDataPage.setSheetName("Página 1 de Elementos");
//				//cabeceras hoja
//				elementoExcelDataPage.setHeaderNames(ReportData.parseColumns(columns));
//				//datos hoja
//				elementoExcelDataPage.setModelData(listElementoPage);
//			reportData.add(elementoExcelDataPage);
		modelMap.put("reportData", reportData);
		
		//Generación del XLS o XLSX
//		String reportView = request.getServletPath().substring(9);
		if (request.getServletPath().indexOf("xlsReport")!=-1){
			return new ModelAndView("xlsReport", modelMap);
		}else{
			return new ModelAndView("xlsxReport", modelMap);
		}
//		return new ModelAndView(reportView, modelMap);
		
	}
	/**
	 * * <p>Desvincular un elemento a una vivienda</p>
	 * @param ideElemento
	 * @param numVivienda
	 * @return
	 */
	@RequestMapping(value = "/desvincularElemento/{ideElemento}/{numVivienda}",method = RequestMethod.PUT)
    public @ResponseBody boolean desvincularElemento(@PathVariable BigDecimal ideElemento,@PathVariable BigDecimal numVivienda) {
        boolean desvincular=this.elementosService.desvincularElemento(ideElemento,numVivienda);
		ElementosController.logger.info("[PUT] : Elemento desvinculado correctamente");
        return desvincular;
    }
	/**
	 * Devuelve si un elemento tiene contratos
	 * 
	 * @param ideElemento
	 *            BigDecimal
	 * @return PlSqlOutputError
	 */
	@RequestMapping(value = "/existenContratos/{ideElemento}", method = RequestMethod.GET)
	public @ResponseBody
	boolean existenContratos(@PathVariable BigDecimal ideElemento) {
		ElementosController.logger.info("[GET - existenContratos] : si tiene contratos el Elemento");
		return this.elementosService.existenContratos(new Elemento(ideElemento));
	}
	/**
	 * Establece en <code>session</code> n&uacute;mero de vivienda 
	 *
	 * @param numVivienda BigDecimal 
	 * @param httpSession HttpSession
	 * @return boolean true 
	 */
	@RequestMapping(value = "/setNumVivienda/{numVivienda}",method = RequestMethod.POST)
	public  @ResponseBody boolean setNumVivienda(@PathVariable String numVivienda,HttpSession httpSession) {	
		ElementosController.logger.info("[POST - setNumVivienda] : Vivienda -->" + numVivienda);
		httpSession.setAttribute("NUM_VIVIENDA", numVivienda);
    	return true;
	}
}	
