package com.ejie.m62fb.control.registroVPP;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.Inquilino;
import com.ejie.m62fb.service.registroVPP.InquilinoService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * InquilinosController generated by UDA, 21-oct-2015 12:55:00.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/inquilinos")
public class InquilinosController {

	private static final Logger logger = LoggerFactory
			.getLogger(InquilinosController.class);

	@Autowired
	private InquilinoService inquilinoService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idVivienda
	 *            Integer
	 * @return Inquilino Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idM62r06t0}", method = RequestMethod.GET)
	public @ResponseBody Inquilino get(@PathVariable Integer idM62r06t0) {
		Inquilino inquilino = new Inquilino(idM62r06t0);
		inquilino = this.inquilinoService.find(inquilino);
		InquilinosController.logger.info("[GET - findBy_PK] : Obtener Inquilino por PK");
		return inquilino;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterInquilino
	 *            Inquilino Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Inquilino> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<Inquilino> getAll(@ModelAttribute Inquilino filterInquilino) {
		InquilinosController.logger
				.info("[GET - find_ALL] : Obtener Inquilino por filtro");
		return this.inquilinoService.findAll(filterInquilino, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param inquilino
	 *            Inquilino Bean que contiene la informacion a modificar.
	 * @return Inquilino Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	Inquilino edit(@RequestBody Inquilino inquilino) {
		Inquilino inquilinoAux = this.inquilinoService.update(inquilino);
		InquilinosController.logger
				.info("[PUT] : Inquilino actualizado correctamente");
		return inquilinoAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param inquilino
	 *            Inquilino Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Inquilino Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	Inquilino add(@RequestBody Inquilino inquilino) {
		Inquilino inquilinoAux = this.inquilinoService.add(inquilino);
		InquilinosController.logger
				.info("[POST] : Inquilino insertado correctamente");
		return inquilinoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idM62r06t0
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Inquilino Bean eliminado.
	 */
	@RequestMapping(value = "/{idM62r06t0}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Inquilino remove(@PathVariable Integer idM62r06t0) {
		Inquilino inquilino = new Inquilino(idM62r06t0);
		this.inquilinoService.remove(inquilino);
		InquilinosController.logger.info("[DELETE] : Inquilino borrado correctamente");
		return inquilino;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterInquilinos
	 *            Inquilino Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @param request
	 *            HttpServletRequest
	 * @return JQGridResponseDto<Inquilino> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<Inquilino> filter(
			@RequestJsonBody(param = "filter") Inquilino filterInquilinos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		InquilinosController.logger
				.info("[POST - filter] : Obtener Inquilinos");
		return this.inquilinoService.filter(filterInquilinos, jqGridRequestDto,
				false);
	}
}