package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.InquilinoElemento;
import com.ejie.m62fb.service.registroVPP.InquilinoElementoService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * InquilinosElementoController generated by UDA, 16-dic-2015 17:30:00.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/inquilinoselem")
public class InquilinosElementoController {

	private static final Logger logger = LoggerFactory
			.getLogger(InquilinosElementoController.class);

	@Autowired
	private InquilinoElementoService inquilinoElementoService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idInquilino
	 *            BigDecimal
	 * @return InquilinoElemento Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idInquilino}", method = RequestMethod.GET)
	public @ResponseBody
	InquilinoElemento get(@PathVariable BigDecimal idInquilino) {
		InquilinoElemento inquilinoElemento = new InquilinoElemento(idInquilino);
		inquilinoElemento = this.inquilinoElementoService
				.find(inquilinoElemento);
		InquilinosElementoController.logger
				.info("[GET - findBy_PK] : Obtener InquilinoElemento Elemento por PK");
		return inquilinoElemento;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterInquilinoElemento
	 *            InquilinoElemento Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<InquilinoElemento> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<InquilinoElemento> getAll(
			@ModelAttribute InquilinoElemento filterInquilinoElemento) {
		InquilinosElementoController.logger
				.info("[GET - find_ALL] : Obtener InquilinoElemento por filtro");
		return this.inquilinoElementoService.findAll(filterInquilinoElemento,
				null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param inquilinoElemento
	 *            InquilinoElemento Bean que contiene la informacion a
	 *            modificar.
	 * @return InquilinoElemento Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	InquilinoElemento edit(@RequestBody InquilinoElemento inquilinoElemento) {
		InquilinoElemento inquilinoAux = this.inquilinoElementoService
				.update(inquilinoElemento);
		InquilinosElementoController.logger
				.info("[PUT] : InquilinoElemento actualizado correctamente");
		return inquilinoAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param inquilinoElemento
	 *            InquilinoElemento Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return InquilinoElemento Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	InquilinoElemento add(@RequestBody InquilinoElemento inquilinoElemento) {
		InquilinoElemento inquilinoAux = this.inquilinoElementoService
				.add(inquilinoElemento);
		InquilinosElementoController.logger
				.info("[POST] : InquilinoElemento insertado correctamente");
		return inquilinoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idInquilino
	 *            BigDecimal Identificador del objeto que se desea eliminar.
	 * @return InquilinoElemento Bean eliminado.
	 */
	@RequestMapping(value = "/{idInquilino}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	InquilinoElemento remove(@PathVariable BigDecimal idInquilino) {
		InquilinoElemento inquilinoElemento = new InquilinoElemento(idInquilino);
		this.inquilinoElementoService.remove(inquilinoElemento);
		InquilinosElementoController.logger
				.info("[DELETE] : InquilinoElemento borrado correctamente");
		return inquilinoElemento;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterInquilinosElemento
	 *            InquilinoElemento Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @param request
	 *            HttpServletRequest
	 * @return JQGridResponseDto<InquilinoElemento> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<InquilinoElemento> filter(
			@RequestJsonBody(param = "filter") InquilinoElemento filterInquilinosElemento,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		InquilinosElementoController.logger
				.info("[POST - filter] : Obtener Inquilinos");
		return this.inquilinoElementoService.filter(filterInquilinosElemento,
				jqGridRequestDto, false);
	}
}