package com.ejie.m62fb.control.registroVPP;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.Propietario;
import com.ejie.m62fb.service.registroVPP.PropietarioService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ElementoController generated by UDA, 04-sep-2015 12:04:01.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/propietarios")
public class PropietariosController {

	private static final Logger logger = LoggerFactory
			.getLogger(PropietariosController.class);

	@Autowired
	private PropietarioService propietarioService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idM62r04t0
	 *            Integer	
	 * @return Propietario Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idM62r04t0}", method = RequestMethod.GET)
	public @ResponseBody
	Propietario get(@PathVariable Integer idM62r04t0) {
		Propietario propietario = new Propietario(idM62r04t0);
		propietario = this.propietarioService.find(propietario);
		PropietariosController.logger
				.info("[GET - findBy_PK] : Obtener Propietario por PK");
		return propietario;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterPropietario
	 *            Propietario Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Propietario> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<Propietario> getAll(@ModelAttribute Propietario filterPropietario) {
		PropietariosController.logger
				.info("[GET - find_ALL] : Obtener Propietario por filtro");
		return this.propietarioService.findAll(filterPropietario, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param propietario
	 *            Propietario Bean que contiene la informacion a modificar.
	 * @return Propietario Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	Propietario edit(@RequestBody Propietario propietario) {
		Propietario propietarioAux = this.propietarioService
				.update(propietario);
		PropietariosController.logger
				.info("[PUT] : Propietario actualizado correctamente");
		return propietarioAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param propietario
	 *            Propietario Bean que contiene la informacion con la que se va
	 *            a crear el nuevo registro.
	 * @return Propietario Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	Propietario add(@RequestBody Propietario propietario) {
		Propietario propietarioAux = this.propietarioService.add(propietario);
		PropietariosController.logger
				.info("[POST] : Propietario insertado correctamente");
		return propietarioAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idM62r04t0
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Propietario Bean eliminado.
	 */
	@RequestMapping(value = "/{idM62r04t0}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	Propietario remove(@PathVariable Integer idM62r04t0) {
		Propietario propietario = new Propietario(idM62r04t0);
		this.propietarioService.remove(propietario);
		PropietariosController.logger
				.info("[DELETE] : Propietario borrado correctamente");
		return propietario;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterPropietarios
	 *            Propietario Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @param request
	 *            HttpServletRequest
	 * @return JQGridResponseDto<Propietario> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<Propietario> filter(
			@RequestJsonBody(param = "filter") Propietario filterPropietarios,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto,
			HttpServletRequest request) {
		PropietariosController.logger
				.info("[POST - filter] : Obtener Propietarios");
		return this.propietarioService.filter(filterPropietarios,
				jqGridRequestDto, false);
	}
}
