package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.m62fb.utils.error.UFException;
import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.control.PersonaController;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.ViviendaActMasiva;
import com.ejie.m62fb.model.tipos.P28Tipos;
import com.ejie.m62fb.model.tipos.R09Tipos;
import com.ejie.m62fb.service.registroVPP.ViviendaService;
import com.ejie.m62fb.utils.constants.AppConstantes;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.reports.ReportData;

/**
 * SeleccionViviendasController.
 * 
XXXX
 */
@Controller
@RequestMapping(value = "/viviendas/actmasiva")
public class SeleccionViviendasActMasivaController {

	private static final Logger logger = LoggerFactory
			.getLogger(SeleccionViviendasActMasivaController.class);

	@Autowired
	private ViviendaService viviendaService;

	@Autowired
	@SuppressWarnings("rawtypes")
	private ComboController comboController;
	@Autowired
	private PersonaController personaController;
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param numVivienda
	 *            Integer
	 * @return Vivienda Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{numVivienda}", method = RequestMethod.GET)
	public @ResponseBody
	Vivienda get(@PathVariable BigDecimal numVivienda) {
		Vivienda vivienda = new Vivienda(numVivienda);
		vivienda = this.viviendaService.find(vivienda);
		SeleccionViviendasActMasivaController.logger
				.info("[GET - findBy_PK] : Obtener Vivienda por PK");
		return vivienda;
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param vivienda
	 *            Vivienda Bean que contiene la informacion a modificar.
	 * @return Vivienda Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	Vivienda edit(@RequestBody ViviendaActMasiva vivienda) {
		try{
			String type = vivienda.getType();
			Vivienda viviendaFinal;
			if(type.equals(AppConstantes.ALT_MASV_CONTRATOS_ALQUILER)||type.equals(AppConstantes.ALT_MASV_CONTRATOS_COMPRA)){
				viviendaFinal =  this.viviendaService.updateActMasivaContratos(vivienda);
			}
			else{
				viviendaFinal = this.viviendaService.updateActMasiva(vivienda);
			}
			return null;
		}catch(UFException ufe){
			throw ufe;
		}catch(Exception e){
			SeleccionViviendasActMasivaController.logger
			.info("[edit - edit] "+e.toString());
			return null;
		}
		
		//TODO ver como devolver al grid la vivienda
	}


	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param vivienda
	 *            Vivienda Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Vivienda Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	Vivienda add(@RequestBody ViviendaActMasiva vivienda) {
		Vivienda viviendaAux = this.viviendaService.add(vivienda);
		SeleccionViviendasActMasivaController.logger
				.info("[POST] : Vivienda insertada correctamente");
		return viviendaAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param numVivienda
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Vivienda Bean eliminado.
	 */
	@RequestMapping(value = "/{numVivienda}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	Vivienda remove(@PathVariable BigDecimal numVivienda) {
		Vivienda vivienda = new Vivienda(numVivienda);
		this.viviendaService.remove(vivienda);
		SeleccionViviendasActMasivaController.logger
				.info("[DELETE] : Vivienda borrado correctamente");
		return vivienda;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		SeleccionViviendasActMasivaController.logger.info("[GET - View] : elemento");
		this.precargarDatosPantalla(model);
		return "seleccionViviendasActMasiva";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterVivienda
	 *            Vivienda Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Vivienda> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ViviendaActMasiva> filter(
			@RequestJsonBody(param = "filter") Vivienda filterVivienda,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SeleccionViviendasActMasivaController.logger
				.info("[POST - filter] : Obtener Viviendas");
		return this.viviendaService.filterSeleccionViviendasForActMasiva(filterVivienda,
				jqGridRequestDto, false);
	}

	/**
	 * Funcion encargada de realizar la exportacion a Excel
	 * 
	 * @param filterVivienda
	 *            Vivienda
	 * @param columns
	 *            String
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param modelMap
	 *            ModelMap
	 * @param request
	 *            ModelMap
	 * @return ModelAndView
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = { "xlsReport", "xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute ViviendaActMasiva filterVivienda,
			@RequestParam(value = "columns", required = false) String columns,
			@ModelAttribute JQGridRequestDto jqGridRequestDto,
			ModelMap modelMap, HttpServletRequest request) {
		try{
			// Acceso a BD para recuperar datos
			jqGridRequestDto.setPage(null);
			jqGridRequestDto.setRows(null);
			columns=columns.replace("[\"rupInfoCol\",\"\"],[","[");
	//		List<ViviendaActMasiva> listViviendaAll = (List<ViviendaActMasiva>) this.viviendaService
	//				.filterSeleccionViviendas(filterVivienda, jqGridRequestDto,
	//						false).getRows();
			List<ViviendaActMasiva> listViviendaAll = (List<ViviendaActMasiva>) this.viviendaService.
					filterSeleccionViviendasForActMasiva(filterVivienda,
							jqGridRequestDto, false).getRows();
	
			//List<ViviendaActMasiva> listViviendaActMasivaAll =(List<ViviendaActMasiva>)listViviendaAll;
			// Datos
			List<Object> reportData = new ArrayList<Object>();
	
			// Hoja 1
			ReportData<ViviendaActMasiva> viviendaExcelDataAll = new ReportData<ViviendaActMasiva>();
			// Nombre hoja
			viviendaExcelDataAll.setSheetName("Viviendas Filtradas");
			// Cabeceras hoja
			viviendaExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));
			// Datos hoja
			viviendaExcelDataAll.setModelData(listViviendaAll);
			reportData.add(viviendaExcelDataAll);
	
			// Nombre fichero
			modelMap.put("fileName", "ViviendaActMasiva");
			// Excel Data
			modelMap.put("reportData", reportData);
	
			// Generación del XLS o XLSX
			if (request.getServletPath().indexOf("xlsReport") != -1) {
				return new ModelAndView("xlsReport", modelMap);
			} else {
				return new ModelAndView("xlsxReport", modelMap);
			}
		}catch(Exception e){
			SeleccionViviendasActMasivaController.logger
			.info("[xlsReport ERROR]" + e.toString());
			return null;
		}
	}
	/**
	 * Funcion encargada de precargar los datos para la pantalla.
	 * 
	 * Combos, parametros...
	 * 
	 * @param model
	 *            Model
	 */
	@SuppressWarnings("unchecked")
	private void precargarDatosPantalla(Model model) {
		// Combo Regimen Acceso (Filtro y Detalle)
		model.addAttribute("comboRegimenAcceso", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.REGIMEN_ACCESO)));
		// Combo Regimen Tenencia (Detalle)
		model.addAttribute("comboRegimenTenencia", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.REGIMEN_TENENCIA)));
		// Combo Tipo Regulacion (Detalle)
		model.addAttribute("comboTipoRegulacion", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_REGULACION)));
		// Combo Tipo Acceso (Detalle)
		model.addAttribute("comboTipoAcceso", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_ACCESO)));
		// Combo Tipo Iniciativa (Detalle)
		model.addAttribute("comboTipoIniciativa", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_INICIATIVA)));
		// Combo Tipo Promotor (Detalle)
		model.addAttribute("comboTipoPromotor", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PROMOTOR)));
		// Combo Tipo Propietarios (Detalle)
		model.addAttribute("comboTipoPropietarios", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PROPIETARIOS)));
		// Combo Tipo Adjudicacion (Detalle)
		model.addAttribute("comboTipoAdjudicacion", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_ADJUDICACION)));
		// Combo Tipo QuienAdjudica (Detalle)
		model.addAttribute("comboTipoQuienAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_QUIEN_ADJUDICA)));
		// Combo Tipo ComoAdjudica (Detalle)
		model.addAttribute("comboTipoComoAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_COMO_ADJUDICA)));
		// Combo Tipo PorqueAdjudica (Detalle)
		model.addAttribute("comboTipoPorqueAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PORQUE_ADJUDICA)));
		// Tipo Transmision
		model.addAttribute("comboTipoTransmision", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.TIPO_TRANSMISION)));
		// Carga combos Persona (estado civil, sexo, nacionalidad)
		personaController.precargarCombosFiltro(model);
	}
}