package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.control.PersonaController;
import com.ejie.m62fb.model.Vivienda;
import com.ejie.m62fb.model.tipos.P28Tipos;
import com.ejie.m62fb.model.tipos.R09Tipos;
import com.ejie.m62fb.service.registroVPP.ViviendaService;
import com.ejie.m62fb.util.UtilSeguridad;
import com.ejie.m62fb.utils.constants.AppConstantes;
import com.ejie.m62fb.utils.properties.UtilProperties;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.reports.ReportData;

/**
 * SeleccionViviendasController.
 * 
XXXX
 */
@Controller
@RequestMapping(value = "/viviendas/seleccion")
public class SeleccionViviendasController {

	private static final Logger logger = LoggerFactory
			.getLogger(SeleccionViviendasController.class);

	@Autowired
	private ViviendaService viviendaService;

	@Autowired
	@SuppressWarnings("rawtypes")
	private ComboController comboController;

	@Autowired
	private PersonaController personaController;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param numVivienda
	 *            Integer
	 * @return Vivienda Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{rowId}", method = RequestMethod.GET)
	public @ResponseBody
	Vivienda get(@PathVariable String rowId) {
		Vivienda vivienda = new Vivienda();
		vivienda.setRowId(rowId);
		vivienda = this.viviendaService.find(vivienda);
		SeleccionViviendasController.logger
				.info("[GET - findBy_PK] : Obtener Vivienda por PK");
		return vivienda;
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param vivienda
	 *            Vivienda Bean que contiene la informacion a modificar.
	 * @return Vivienda Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	Vivienda edit(@RequestBody Vivienda vivienda) {
		try {
			Vivienda viviendasAux = this.viviendaService.update(vivienda);
			SeleccionViviendasController.logger
					.info("[PUT] : Vivienda actualizada correctamente");
			return viviendasAux;
		} catch (RuntimeException re) {
			logger.error("Error recibido", re);
			throw re;
		}
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param vivienda
	 *            Vivienda Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Vivienda Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	Vivienda add(@RequestBody Vivienda vivienda) {
		Vivienda viviendaAux = this.viviendaService.add(vivienda);
		SeleccionViviendasController.logger
				.info("[POST] : Vivienda insertada correctamente");
		return viviendaAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param numVivienda
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Vivienda Bean eliminado.
	 */
	@RequestMapping(value = "/{rowId}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	Vivienda remove(@PathVariable String rowId) {
		Vivienda vivienda = new Vivienda();
		vivienda.setRowId(rowId);
		this.viviendaService.remove(vivienda);
		SeleccionViviendasController.logger
				.info("[DELETE] : Vivienda borrado correctamente");
		return vivienda;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @param httpSession HttpSession            
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model,HttpSession httpSession,@RequestParam(value = "numVivienda", required = false) String numVivienda) {
		SeleccionViviendasController.logger.info("[GET - View] : seleccionViviendas");
		this.precargarDatosPantalla(model);
		//si ha NUM_VIVIENDA en session, se recoge, se guarda en model y se borra de la sesion
		//String numVivienda = "''";
		
		if (httpSession.getAttribute("NUM_VIVIENDA")!=null ){
			numVivienda= httpSession.getAttribute("NUM_VIVIENDA").toString();	
			httpSession.removeAttribute("NUM_VIVIENDA");
		}else{
			numVivienda = numVivienda==null?"''":numVivienda;
		}
		model.addAttribute("numVivienda", numVivienda);
		model.addAttribute("regVPPSancionador", UtilSeguridad.bUsuarioTieneRol("REG_VPP_SANCIONADOR"));
		model.addAttribute("regVPPAdaptada", UtilSeguridad.bUsuarioTieneRol("REG_VPP_ADAPTADA"));
		setModelCommonAttributes(model);
		return "seleccionViviendas";
	}
	
	@RequestMapping(value = "detalle/maint", method = RequestMethod.GET)
	public String getFormEditLimpia(Model model,HttpSession httpSession) {
		SeleccionViviendasController.logger.info("[GET - View] : seleccionViviendas");
		this.precargarDatosPantalla(model);
		model.addAttribute("numVivienda", "''");
		model.addAttribute("regVPPSancionador", UtilSeguridad.bUsuarioTieneRol("REG_VPP_SANCIONADOR"));
		model.addAttribute("regVPPAdaptada", UtilSeguridad.bUsuarioTieneRol("REG_VPP_ADAPTADA"));
		setModelCommonAttributes(model);
		return "seleccionViviendasDetalle";
	}
	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @param httpSession HttpSession            
	 * @return String
	 */
	@RequestMapping(value = "/maint/{numVivienda}", method = RequestMethod.GET)
	public String getFormEdit(Model model,HttpSession httpSession,@PathVariable BigDecimal numVivienda)  {
		SeleccionViviendasController.logger.info("[GET - View] : seleccionViviendas");
		return getFormEdit(model, httpSession, numVivienda.toString());
	}
	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterVivienda
	 *            Vivienda Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Vivienda> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<Vivienda> filter(
			@RequestJsonBody(param = "filter") Vivienda filterVivienda,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SeleccionViviendasController.logger
				.info("[POST - filter] : Obtener Viviendas");
		return this.viviendaService.filterSeleccionViviendas(filterVivienda,
				jqGridRequestDto, false);
	}

	/**
	 * Funcion encargada de realizar la exportacion a Excel
	 * 
	 * @param filterVivienda
	 *            Vivienda
	 * @param columns
	 *            String
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param modelMap
	 *            ModelMap
	 * @param request
	 *            ModelMap
	 * @return ModelAndView
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value = { "xlsReport", "xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute Vivienda filterVivienda,
			@RequestParam(value = "columns", required = false) String columns,
			@ModelAttribute JQGridRequestDto jqGridRequestDto,
			ModelMap modelMap, HttpServletRequest request) {

		// Acceso a BD para recuperar datos
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);
		List<Vivienda> listViviendaAll = (List<Vivienda>) this.viviendaService
				.filterSeleccionViviendas(filterVivienda, jqGridRequestDto,
						false).getRows();

		// Datos
		List<Object> reportData = new ArrayList<Object>();

		// Hoja 1
		ReportData<Vivienda> viviendaExcelDataAll = new ReportData<Vivienda>();
		// Nombre hoja
		viviendaExcelDataAll.setSheetName("Viviendas Filtradas");
		// Cabeceras hoja
		viviendaExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));
		// Datos hoja
		viviendaExcelDataAll.setModelData(listViviendaAll);
		reportData.add(viviendaExcelDataAll);

		// Nombre fichero
		modelMap.put("fileName", "Viviendas");
		// Excel Data
		modelMap.put("reportData", reportData);

		// Generación del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}
	}

	/**
	 * Funcion encargada de precargar los datos para la pantalla.
	 * 
	 * Combos, parametros...
	 * 
	 * @param model
	 *            Model
	 */
	@SuppressWarnings("unchecked")
	private void precargarDatosPantalla(Model model) {

		/** FILTRO **/
		// Combo Regimen Acceso (Filtro y Detalle)
		model.addAttribute("comboRegimenAcceso", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.REGIMEN_ACCESO)));

		/** DATOS VIVIENDA **/
		// Combo Regimen Tenencia (Detalle)
		model.addAttribute("comboRegimenTenencia", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.REGIMEN_TENENCIA)));

		/** ANEJOS **/
		// Combo Tipo de vinculación (Lupa Elemento)
		model.addAttribute("comboTipoVinculacion", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.TIPO_VINCULACION)));
		// Combo Tipo de elemento (Lupa Elemento)
		model.addAttribute("comboTipoElemento", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.TIPO_ELEMENTO)));

		/** CONTRATOS COMPRA **/
		// Tipo Transmision (Detalle Contrato Compra)
		model.addAttribute("comboTipoTransmision", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.TIPO_TRANSMISION)));

		/** CONTRATOS ALQUILER **/
		// Tipo Contrato Alquiler (Detalle Contrato Alquiler)
		model.addAttribute("comboTipoContrato", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.TIPO_CONTRATO_ALQUILER)));

		// Combos Persona (Lupas Contrato Compra y Alquiler)
		this.personaController.precargarCombosFiltro(model);

		/** CLASIFICACION **/
		// Combo Tipo Regulacion (Detalle)
		model.addAttribute("comboTipoRegulacion", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_REGULACION)));
		// Combo Tipo Acceso (Detalle)
		model.addAttribute("comboTipoAcceso", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_ACCESO)));
		// Combo Tipo Iniciativa (Detalle)
		model.addAttribute("comboTipoIniciativa", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_INICIATIVA)));
		// Combo Tipo Promotor (Detalle)
		model.addAttribute("comboTipoPromotor", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PROMOTOR)));
		// Combo Tipo Propietarios (Detalle)
		model.addAttribute("comboTipoPropietarios", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PROPIETARIOS)));
		// Combo Tipo Adjudicacion (Detalle)
		model.addAttribute("comboTipoAdjudicacion", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_ADJUDICACION)));
		// Combo Tipo QuienAdjudica (Detalle)
		model.addAttribute("comboTipoQuienAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_QUIEN_ADJUDICA)));
		// Combo Tipo ComoAdjudica (Detalle)
		model.addAttribute("comboTipoComoAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_COMO_ADJUDICA)));
		// Combo Tipo PorqueAdjudica (Detalle)
		model.addAttribute("comboTipoPorqueAdjudica", R09Tipos
				.serializeComboR09(this.comboController
						.getComboR09(R09Tipos.TIPO_PORQUE_ADJUDICA)));
	}
	
	@RequestMapping(method = RequestMethod.GET,value="/lupa/findAllLike")
	public @ResponseBody List<Vivienda> getAll(@ModelAttribute Vivienda filterVivienda) {
		SeleccionViviendasController.logger.info("[GET - find_ALL] : Obtener Viviendas por filtro");
		//False para que busque el %numVivienda%
	    return this.viviendaService.findAllLike(filterVivienda, null, false);
	}

	@RequestMapping(value = "/lupa/{numVivienda}", method = RequestMethod.GET)
	public @ResponseBody
	Vivienda get(@PathVariable Long numVivienda) {
		Vivienda vivienda = new Vivienda();
		vivienda.setNumVivienda(BigDecimal.valueOf(numVivienda));
		vivienda = this.viviendaService.findForLupa(vivienda);
		SeleccionViviendasController.logger
				.info("[GET - findBy_PK] : Obtener Vivienda por numero vivienda");
		return vivienda;
	}
	@RequestMapping(value = "cleanMaint", method = RequestMethod.GET)
	public String getFormEditSinTemplate(Model model,HttpSession httpSession) {
		SeleccionViviendasController.logger.info("[GET - View] : seleccionViviendas");
		this.precargarDatosPantalla(model);

		model.addAttribute("numVivienda", "''");
		model.addAttribute("regVPPSancionador", UtilSeguridad.bUsuarioTieneRol("REG_VPP_SANCIONADOR"));
		model.addAttribute("regVPPAdaptada", UtilSeguridad.bUsuarioTieneRol("REG_VPP_ADAPTADA"));
		setModelCommonAttributes(model);
		return "seleccionViviendasSinMenu";
	}
	
	private void setModelCommonAttributes(Model model){
	    model.addAttribute("urlSubsanacion", UtilProperties.getProperty(AppConstantes.URL_SUBSANACION));
	    model.addAttribute("urlBaseDocumentos", UtilProperties.getProperty(AppConstantes.URL_DOCUMENTOS));
	    model.addAttribute("urlBaseVisado", UtilProperties.getProperty(AppConstantes.URL_VISADO));
	}
//	@RequestMapping(value = "/getByNumVivienda/{numVivienda}", method = RequestMethod.GET)
//	public @ResponseBody List<Vivienda> getByNumVivienda(@PathVariable BigDecimal numVivienda) {
//		SeleccionViviendasController.logger.info("[GET - find_ALL] : Obtener Viviendas por filtro");
//		Vivienda filterVivienda = new Vivienda();
//		filterVivienda.setNumVivienda(numVivienda);
//	    return this.viviendaService.findAll(filterVivienda, null);
//	}

	
}