package com.ejie.m62fb.control.registroVPP;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.m62fb.control.ComboController;
import com.ejie.m62fb.model.SolicitudReserva;
import com.ejie.m62fb.model.tipos.P28Tipos;
import com.ejie.m62fb.service.registroVPP.SolicitudReservaService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.reports.ReportData;

/**
 * SolicitudesReservaController generated by UDA, 17-nov-2015 12:55:16.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/reservas")
public class SolicitudesReservaController {

	private static final Logger logger = LoggerFactory
			.getLogger(SolicitudesReservaController.class);

	@Autowired
	private SolicitudReservaService solicitudesReservaService;

	@Autowired
	@SuppressWarnings("rawtypes")
	private ComboController comboController;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idM62r38t0
	 *            Integer
	 * @return SolicitudesReserva Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idM62r38t0}", method = RequestMethod.GET)
	public @ResponseBody
	SolicitudReserva get(@PathVariable Integer idM62r38t0) {
		SolicitudReserva solicitudReserva = new SolicitudReserva();
		solicitudReserva.setIdM62r38t0(idM62r38t0);
		solicitudReserva = this.solicitudesReservaService
				.find(solicitudReserva);
		SolicitudesReservaController.logger
				.info("[GET - findBy_PK] : Obtener SolicitudesReserva por PK");
		return solicitudReserva;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterSolicitudesReserva
	 *            SolicitudReserva Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<SolicitudReserva> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<SolicitudReserva> getAll(
			@ModelAttribute SolicitudReserva filterSolicitudesReserva) {
		SolicitudesReservaController.logger
				.info("[GET - find_ALL] : Obtener SolicitudesReserva por filtro");
		return this.solicitudesReservaService.findAll(filterSolicitudesReserva,
				null);
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/solicitudesreservas/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		SolicitudesReservaController.logger
				.info("[GET - View] : solicitudesreserva");
		this.precargarCombosFiltro(model);
		return "solicitudesreservas";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterSolicitudesReserva
	 *            SolicitudReserva Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<SolicitudReserva> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<SolicitudReserva> filter(
			@RequestJsonBody(param = "filter") SolicitudReserva filterSolicitudesReserva,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		SolicitudesReservaController.logger
				.info("[POST - filter] : Obtener SolicitudesReservas");
		return this.solicitudesReservaService.filter(filterSolicitudesReserva,
				jqGridRequestDto, false);
	}

	/**
	 * Funcion encargada de precargar el combo de estado del filtro
	 * 
	 * @param model
	 *            Model
	 */
	@SuppressWarnings("unchecked")
	private void precargarCombosFiltro(Model model) {
		/** FILTRO **/
		// Combo Estado
		model.addAttribute("comboEstado", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.ESTADO_TRAMIT)));

		/** COMBOS LUPA VIVIENDAS CATALOGO **/
		// Combo Regimen Acceso
		model.addAttribute("comboRegimenAcceso", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.REGIMEN_ACCESO)));
		// Combo Regimen Tenencia
		model.addAttribute("comboRegimenTenencia", P28Tipos
				.serializeComboP28(this.comboController
						.getComboP28(P28Tipos.REGIMEN_TENENCIA)));
	}

	/**
	 * Generación Excel
	 * 
	 * @param filterSolicitudReserva
	 *            SolicitudReserva
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param modelMap
	 *            ModelMap
	 * @param columns
	 *            String
	 * @param request
	 *            HttpServletRequest
	 * @return ModelAndView
	 */
	@RequestMapping(value = { "xlsReport", "xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(
			@ModelAttribute SolicitudReserva filterSolicitudReserva,
			@ModelAttribute JQGridRequestDto jqGridRequestDto,
			ModelMap modelMap,
			@RequestParam(value = "columns", required = false) String columns,
			HttpServletRequest request) {

		// Acceso a BD para recuperar datos
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);
		List<SolicitudReserva> listSolicitudReservaAll = this.solicitudesReservaService
				.findAllLike(filterSolicitudReserva, jqGridRequestDto, false);

		// Nombre fichero
		modelMap.put("fileName", "SolicitudesReservas");

		// Datos
		List<Object> reportData = new ArrayList<Object>();
		// Hoja 1
		ReportData<SolicitudReserva> elementoExcelDataAll = new ReportData<SolicitudReserva>();
		// nombre hoja
		elementoExcelDataAll.setSheetName("Todas Solicitudes de Reserva");
		// cabeceras hoja
		elementoExcelDataAll.setHeaderNames(ReportData.parseColumns(columns));
		// datos hoja
		elementoExcelDataAll.setModelData(listSolicitudReservaAll);
		reportData.add(elementoExcelDataAll);

		modelMap.put("reportData", reportData);

		// Generación del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}
	}
}