package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.ViviendaDisponible;
import com.ejie.m62fb.service.registroVPP.ViviendaDisponibleService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ViviendasDisponiblesController generated by UDA, 02-dic-2015 10:36:33.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/viviendasdisponibles")
public class ViviendasDisponiblesController {

	private static final Logger logger = LoggerFactory
			.getLogger(ViviendasDisponiblesController.class);

	@Autowired
	private ViviendaDisponibleService viviendaDisponibleService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 * @return ViviendaDisponible Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{numVivienda}", method = RequestMethod.GET)
	public @ResponseBody
	ViviendaDisponible get(@PathVariable BigDecimal numVivienda) {
		ViviendaDisponible viviendaDisponible = new ViviendaDisponible();
		viviendaDisponible.setNumVivienda(numVivienda);
		viviendaDisponible = this.viviendaDisponibleService
				.find(viviendaDisponible);
		ViviendasDisponiblesController.logger
				.info("[GET - findBy_PK] : Obtener ViviendaDisponible por PK");
		return viviendaDisponible;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterViviendaDisponible
	 *            ViviendaDisponible Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ViviendaDisponible> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ViviendaDisponible> getAll(
			@ModelAttribute ViviendaDisponible filterViviendaDisponible) {
		ViviendasDisponiblesController.logger
				.info("[GET - find_ALL] : Obtener ViviendaDisponible por filtro");
		return this.viviendaDisponibleService.findAll(filterViviendaDisponible,
				null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param viviendaDisponible
	 *            ViviendaDisponible Bean que contiene la informacion a
	 *            modificar.
	 * @return ViviendaDisponible Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	ViviendaDisponible edit(@RequestBody ViviendaDisponible viviendaDisponible) {
		ViviendaDisponible viviendasDisponiblesVAux = this.viviendaDisponibleService
				.update(viviendaDisponible);
		ViviendasDisponiblesController.logger
				.info("[PUT] : ViviendaDisponible actualizado correctamente");
		return viviendasDisponiblesVAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param viviendaDisponible
	 *            ViviendaDisponible Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return ViviendaDisponible Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	ViviendaDisponible add(@RequestBody ViviendaDisponible viviendaDisponible) {
		ViviendaDisponible viviendasDisponiblesVAux = this.viviendaDisponibleService
				.add(viviendaDisponible);
		ViviendasDisponiblesController.logger
				.info("[POST] : ViviendaDisponible insertado correctamente");
		return viviendasDisponiblesVAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 * @return ViviendaDisponible Bean eliminado.
	 */
	@RequestMapping(value = "/{numVivienda}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	ViviendaDisponible remove(@PathVariable BigDecimal numVivienda) {
		ViviendaDisponible viviendaDisponible = new ViviendaDisponible();
		viviendaDisponible.setNumVivienda(numVivienda);
		this.viviendaDisponibleService.remove(viviendaDisponible);
		ViviendasDisponiblesController.logger
				.info("[DELETE] : ViviendaDisponible borrado correctamente");
		return viviendaDisponible;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		ViviendasDisponiblesController.logger
				.info("[GET - View] : viviendasdisponibles");
		return "viviendasdisponibles";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterViviendaDisponible
	 *            ViviendaDisponible Bean que contiene los parametros de
	 *            filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<ViviendaDisponible> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ViviendaDisponible> filter(
			@RequestJsonBody(param = "filter") ViviendaDisponible filterViviendaDisponible,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ViviendasDisponiblesController.logger
				.info("[POST - filter] : Obtener ViviendasDisponibles");
		return this.viviendaDisponibleService.filter(filterViviendaDisponible,
				jqGridRequestDto, false);
	}

	/**
	 * Find Lupa
	 * 
	 * @param numVivienda
	 *            BigDecimal
	 * @return List<ViviendaDisponible>
	 */
	@RequestMapping(value = "/findAllLike", method = RequestMethod.GET)
	public @ResponseBody
	List<ViviendaDisponible> findAllLikeLupa(
			@RequestParam(value = "numVivienda", required = false) BigDecimal numVivienda) {
		// Filtro
		return this.viviendaDisponibleService.findAllLike(
				new ViviendaDisponible(numVivienda), null, false);
	}
}