package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.m62fb.model.ViviendaReservada;
import com.ejie.m62fb.service.registroVPP.ViviendaReservadaService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ViviendasReservadasController generated by UDA, 20-nov-2015 10:16:33.
 * 
XXXX
 */

@Controller
@RequestMapping(value = "/viviendasreservadas")
public class ViviendasReservadasController {

	private static final Logger logger = LoggerFactory
			.getLogger(ViviendasReservadasController.class);

	@Autowired
	private ViviendaReservadaService viviendaReservadaService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idM62r43t0
	 *            Integer
	 * @return ViviendaReservada Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idM62r43t0}", method = RequestMethod.GET)
	public @ResponseBody
	ViviendaReservada get(@PathVariable BigDecimal idM62r43t0) {
		ViviendaReservada viviendaReservada = new ViviendaReservada();
		viviendaReservada.setIdM62r43t0(idM62r43t0);
		viviendaReservada = this.viviendaReservadaService
				.find(viviendaReservada);
		ViviendasReservadasController.logger
				.info("[GET - findBy_PK] : Obtener ViviendaReservada por PK");
		return viviendaReservada;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterViviendaReservada
	 *            ViviendaReservada Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ViviendaReservada> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ViviendaReservada> getAll(
			@ModelAttribute ViviendaReservada filterViviendaReservada) {
		ViviendasReservadasController.logger
				.info("[GET - find_ALL] : Obtener ViviendaReservada por filtro");
		return this.viviendaReservadaService.findAll(filterViviendaReservada,
				null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 * 
	 * @param viviendaReservada
	 *            ViviendaReservada Bean que contiene la informacion a
	 *            modificar.
	 * @return ViviendaReservada Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody
	ViviendaReservada edit(@RequestBody ViviendaReservada viviendaReservada) {
		ViviendaReservada viviendasReservadasVAux = this.viviendaReservadaService
				.update(viviendaReservada);
		ViviendasReservadasController.logger
				.info("[PUT] : ViviendaReservada actualizado correctamente");
		return viviendasReservadasVAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 * 
	 * @param viviendaReservada
	 *            ViviendaReservada Bean que contiene la informacion con la que
	 *            se va a crear el nuevo registro.
	 * @return ViviendaReservada Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody
	ViviendaReservada add(@RequestBody ViviendaReservada viviendaReservada) {
		ViviendaReservada viviendasReservadasVAux = this.viviendaReservadaService
				.add(viviendaReservada);
		ViviendasReservadasController.logger
				.info("[POST] : ViviendaReservada insertado correctamente");
		return viviendasReservadasVAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 * 
	 * @param idM62r43t0
	 *            Integer
	 * @return ViviendaReservada Bean eliminado.
	 */
	@RequestMapping(value = "/{idM62r43t0}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody
	ViviendaReservada remove(@PathVariable BigDecimal idM62r43t0) {
		ViviendaReservada viviendaReservada = new ViviendaReservada();
		viviendaReservada.setIdM62r43t0(idM62r43t0);
		this.viviendaReservadaService.remove(viviendaReservada);
		ViviendasReservadasController.logger
				.info("[DELETE] : ViviendaReservada borrado correctamente");
		return viviendaReservada;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		ViviendasReservadasController.logger
				.info("[GET - View] : viviendasreservadas");
		return "viviendasreservadas";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterViviendaReservada
	 *            ViviendaReservada Bean que contiene los parametros de filtrado
	 *            a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<ViviendaReservada> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ViviendaReservada> filter(
			@RequestJsonBody(param = "filter") ViviendaReservada filterViviendaReservada,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ViviendasReservadasController.logger
				.info("[POST - filter] : Obtener ViviendasReservadas");
		return this.viviendaReservadaService.filter(filterViviendaReservada,
				jqGridRequestDto, false);
	}
}
