package com.ejie.m62fb.control.registroVPP;

import java.math.BigDecimal;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.ejie.m62fb.model.ViviendaSolicitada;
import com.ejie.m62fb.service.registroVPP.ViviendaSolicitadaService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ViviendasSolicitadasController.
 * 
XXXX
 */
@Controller
@RequestMapping(value = "/viviendassolicitadas")
public class ViviendasSolicitadasController {

	private static final Logger logger = LoggerFactory
			.getLogger(ViviendasSolicitadasController.class);

	@Autowired
	private ViviendaSolicitadaService viviendaSolicitadaService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idM62r40t0
	 *            Integer
	 * @return ViviendaSolicitada Objeto correspondiente al identificador
	 *         indicado.
	 */
	@RequestMapping(value = "/{idM62r40t0}", method = RequestMethod.GET)
	public @ResponseBody
	ViviendaSolicitada get(@PathVariable BigDecimal idM62r40t0) {
		ViviendaSolicitada viviendaSolicitada = new ViviendaSolicitada();
		viviendaSolicitada.setIdM62r40t0(idM62r40t0);
		viviendaSolicitada = this.viviendaSolicitadaService
				.find(viviendaSolicitada);
		ViviendasSolicitadasController.logger
				.info("[GET - findBy_PK] : Obtener ViviendasSolicitadas por PK");
		return viviendaSolicitada;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 * 
	 * @param filterViviendaSolicitada
	 *            ViviendaSolicitada Objeto que contiene los parametros de
	 *            filtrado utilizados en la busqueda.
	 * @return List<ViviendaSolicitada> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody
	List<ViviendaSolicitada> getAll(
			@ModelAttribute ViviendaSolicitada filterViviendaSolicitada) {
		ViviendasSolicitadasController.logger
				.info("[GET - find_ALL] : Obtener ViviendaSolicitada por filtro");
		return this.viviendaSolicitadaService.findAll(filterViviendaSolicitada,
				null);
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		ViviendasSolicitadasController.logger
				.info("[GET - View] : viviendassolicitadas");
		return "viviendassolicitadas";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterViviendaSolicitada
	 *            ViviendaSolicitada Bean que contiene los parametros de
	 *            filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<ViviendaSolicitada> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody
	JQGridResponseDto<ViviendaSolicitada> filter(
			@RequestJsonBody(param = "filter") ViviendaSolicitada filterViviendaSolicitada,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ViviendasSolicitadasController.logger
				.info("[POST - filter] : Obtener ViviendasSolicitadass");
		return this.viviendaSolicitadaService.filter(filterViviendaSolicitada,
				jqGridRequestDto, false);
	}
}
